""" extension to generator RPC with marshal/unmarshal MgtObject or Protobuf or even userObject """
import pdb
import re
import os
import sys
import logging
from pyang import plugin
from string import Template
from globals import *

''' Rules convert: 
Yang 
  1/ Module   : => lower camel case namespace i.e safplusAmf
  2/ container/List/Leaf: => upper camel case Protobuf Message/Field i.e SafplusAmf
'''

HEADER_PROTO_FILE = ['''/* 
 * Copyright (C) 2002-2015 OpenClovis Solutions Inc.  All Rights Reserved.
 */''']

HEADER_OPTION_SERVICES=['option cc_generic_services = false;','option py_generic_services = true;','option java_generic_services = false;']


CHILD_RPC_STMTS = ['container', 'list', 'leaf-list', 'leaf', 'uses']

SERVICE_TEMPLATE = Template('''service ${serviceName} {
${services}
}
''')

RPC_TEMPLATE = Template('''  rpc ${name}(${msgInput}) returns(${msgOutput});''')

MESSAGE_TEMPLATE = Template('''message ${messageName} {
${fields}
  extensions 1000 to max;
}
''')

''' 
message fields template
'''
FIELD_TEMPLATE = Template('''  ${specifying} ${type} ${name} = ${index}${extra};''')

'''
import template
'''
IMPORT_TEMPLATE = Template('''import "${module}.proto";''')

ENUM_TEMPLATE = Template('''enum ${enum} {
  option allow_alias = true;
${enumList}
} 
''')

RPC_NAMESPACE = 'SAFplus.Rpc.'

SPECIFYING_FIELDS_MAP = {'container': 'optional', 'list':'repeated', 'leaf-list':'repeated', 'leaf':'optional', 'uses': 'optional', 'anyxml':'optional'}

class RpcGenerator(object):
    """Used to generate proto.proto file. Then do generator c++ RPC from a yang module"""

    def __init__(self, module, path, ctx):
        self.stmt = module
        self.path = path
        self.ctx = ctx
        self.module_arg = self.stmt.arg
        self.messages = []
        self.rpcs = []

        # SAFplus protobuf extension to customize internal SAFplus RPC
        self.imports = set(["SAFplusPBExt"])

        self.mapping_typedef_enum = {}
        self.message_stmt_args = []
        self.c_existings = {}
        self.identities = {}

        ''' Lookup identity type '''
        for ch in self.stmt.substmts:
          if ch.keyword == 'identity':
              base = ch.search_one('base')
              if base is not None:
                  if base.arg not in self.identities:
                      self.identities[base.arg] = set([])
                  self.identities[base.arg].add(ch.arg)

        self.generate()


    def generate(self):
        """Generates proto.proto for module"""

        for stmt in self.stmt.substmts:
            if stmt.keyword == 'rpc':
                self.generate_rpc_statement(stmt)
            elif stmt.keyword in ('container', 'list', 'typedef', 'grouping'):
                if stmt.keyword in ('typedef',):
                    key_type = search_one(stmt, 'type')
                    """ Find enum type """
                    if key_type.arg in ('enumeration',):
                        idx = 0;
                        self.messages.append("enum %s {" %normalize(stmt.arg))
                        self.messages.append("  option allow_alias = true;")
                        ''' Specific enum constant '''
                        for x in map(enumize, search(key_type, 'enum')):
                            if x[1] is None:
                                self.messages.append("  %s_%s = %d;" %(normalize(stmt.arg), x[0], idx))
                                idx+=1
                            else:
                                self.messages.append("  %s_%s = %s;" %(normalize(stmt.arg), x[0], x[1]))
                                idx = int(x[1]) + 1
                        self.messages.append("}")
                        self.messages.append("")

                        self.mapping_typedef_enum[stmt.arg] = normalize(stmt.arg)

                    elif search_one(stmt, C_EXISTING) is None:
                        type = self.getProtoType(stmt, self.ctx)
                        field = FIELD_TEMPLATE.safe_substitute(specifying='required', type=type, name=camelize(type), index=1, extra='')
                        self.messages.append(MESSAGE_TEMPLATE.safe_substitute(messageName= "%s" %normalize(stmt.arg), fields=field))

                else:
                    self.messages.append(MESSAGE_TEMPLATE.safe_substitute(messageName= "%s" %normalize(stmt.arg), fields="\n".join(self.generator_container_message(stmt))))

                # Store protobuf message
                self.message_stmt_args.append(normalize(stmt.arg))

        ''' Output to service.proto '''
        filename = "%s.proto" % camelize(self.module_arg)
        write_file(self.path, filename, HEADER_PROTO_FILE + ["package %s%s;" %(RPC_NAMESPACE, camelize(self.module_arg))] + map(lambda x: IMPORT_TEMPLATE.safe_substitute(module=camelize(x)), self.imports) + [''] + HEADER_OPTION_SERVICES + [''] + self.messages + SERVICE_TEMPLATE.safe_substitute(serviceName=camelize(self.module_arg), services="\n".join(self.rpcs)).splitlines(), None, F_OVERWRITE)


    def generate_rpc_statement(self, stmt):
        '''
        RFC6020:
        7.13.2.  The input Statement
           The "input" statement, which is optional
           The "output" statement, which is optional
        '''
        rpc_name_msg = normalize(stmt.arg)
        rpc_name_function = camelize(stmt.arg)

        input_stmt = search_one(stmt, 'input')
        output_stmt = search_one(stmt, 'output')

        rpc_msg_input = "%sRequest" %rpc_name_msg
        rpc_msg_output = "%sResponse" %rpc_name_msg

        fields = []
        if input_stmt != None:
            fields.extend(self.generator_container_message(input_stmt))

        ''' Generate message request '''
        self.messages.append(MESSAGE_TEMPLATE.safe_substitute(messageName=rpc_msg_input, fields="\n".join(fields)))

        fields = []
        if output_stmt != None and len(output_stmt.substmts) > 0:
            fields.extend(self.generator_container_message(output_stmt))
            ''' Generate message response '''
            self.messages.append(MESSAGE_TEMPLATE.safe_substitute(messageName=rpc_msg_output, fields="\n".join(fields)))
        else:
            rpc_msg_output = "NO_RESPONSE"

        ''' Generate a service '''
        self.rpcs.append(RPC_TEMPLATE.safe_substitute(name=rpc_name_function, msgInput=rpc_msg_input, msgOutput=rpc_msg_output))

    def generator_container_message(self, stmt):
        fields = []

        numIndex = 1
        for ch in stmt.substmts:

            specifying = 'optional'
            try:
                specifying = SPECIFYING_FIELDS_MAP.get(ch.keyword)
            except:
                specifying = 'optional'
                pass

            if ch.keyword in ('leaf', 'leaf-list', 'anyxml'):
                type = self.getProtoType(ch, self.ctx)
                extra = ''
                key_override = search_one(ch, C_TYPE)
                if key_override or type in ('binary',):
                    ''' 
                    TODO: 1 field bytes enough for reuse C++ structure???
                    
                    '''
                    type = 'bytes'
                elif type in self.c_existings.keys():
                    extra = ' [(c_existing)="%s"]' % self.c_existings.get(type)
                    type = 'bytes'

                type_stmt = search_one(ch, 'type')

                if type_stmt and type_stmt.arg in ('enumeration',) and type not in self.message_stmt_args:
                    idx = 0;
                    self.messages.append("enum %s {" %type)
                    self.messages.append("  option allow_alias = true;")
                    ''' Specific enum constant '''
                    for x in map(enumize, search(type_stmt, 'enum')):
                        if x[1] is None:
                            self.messages.append("  %s_%s = %d;" %(normalize(type), x[0], idx))
                            idx+=1
                        else:
                            self.messages.append("  %s_%s = %s;" %(normalize(type), x[0], x[1]))
                            idx = int(x[1]) + 1
                    self.messages.append("}")
                    self.messages.append("")

                fields.append(FIELD_TEMPLATE.safe_substitute(specifying=specifying, type=type, name=camelize(ch.arg), index=numIndex, extra=extra))
                numIndex+=1

            elif ch.keyword in ('container', 'list',):
                if normalize(ch.arg) not in self.message_stmt_args:
                    self.messages.append(MESSAGE_TEMPLATE.safe_substitute(messageName= "%s" %normalize(ch.arg), fields="\n".join(self.generator_container_message(ch))))
                    # Store protobuf message
                    self.message_stmt_args.append(normalize(ch.arg))

                fields.append(FIELD_TEMPLATE.safe_substitute(specifying=specifying, type=normalize(ch.arg), name=camelize(ch.arg), index=numIndex, extra=''))
                numIndex+=1

            elif ch.keyword in ('uses',):
                type = normalize(ch.arg)
                try:
                    #Using external class from imported
                    import_arg, message = str(ch.arg).split(":")
                    ns = camelize(mapping_module_import_prefix[self.module_arg][import_arg])
                    self.imports.add(ns)
                    type = "%s%s.%s" %(RPC_NAMESPACE, ns, normalize(message))
                except:
                    pass
                fields.append(FIELD_TEMPLATE.safe_substitute(specifying=specifying, type=type, name=camelize(type), index=numIndex, extra=''))
                numIndex+=1

        return fields

    def getProtoType(self, yang_type, ctx):
        if yang_type.keyword == 'anyxml':
            return 'string'

        typestmt = search_one(yang_type, 'type')
        if typestmt.arg == 'leafref':
            return getProtoType(typestmt.parent.i_leafref.i_target_node, ctx)

        ''' work-around for identityref/union '''
        if typestmt.arg in ('identityref', 'union'):
            return 'string'

        primitive = typestmt.arg
        if typestmt.arg in ('boolean',):
            primitive = 'bool'
        elif typestmt.arg == 'decimal64':
            primitive = 'sfixed64'
        elif typestmt.arg == 'enumeration':
            primitive = normalize(getParent(typestmt).arg)
        elif typestmt.arg == 'instance-identifier':
            primitive = 'string'
        elif typestmt.arg in ('int8', 'int16', 'int32', 'uint8', 'uint16', 'uint32'):
            if typestmt.arg[:1] == 'u':  # Unsigned
                primitive = "fixed32"
            else:
                primitive = "sint32"
        elif typestmt.arg in ('int64', 'uint64'):
            if typestmt.arg[:1] == 'u':  # Unsigned
                primitive = "uint64"
            else:
                primitive = "sint64"

        else:
          try:
              primitive = mapping_typedef_enum["%s" %primitive]
          except:
              try:
                  typedef = typestmt.i_typedef
                  if typedef is None: raise AttributeError()
              except AttributeError, e:
                  if typestmt.keyword == 'typedef':
                      primitive = normalize(typestmt.arg)
              else:
                  if getParent(typedef).keyword in ('module', 'submodule'):
                      ns = getNamespace(typedef, ctx)
                      typedef_arg = normalize(typedef.arg)
                      if ns != camelize(self.module_arg):
                        self.imports.add(ns)
                        typedef_arg = "%s%s.%s" %(RPC_NAMESPACE, ns, normalize(typedef.arg))
                      primitive = typedef_arg
                  else:
                      basetype = getBaseType(typedef)
                      primitive = getProtoType(basetype, ctx)

        return primitive

