"""Y2Cpp plugin"""
import pdb
import re, types
import os
import io
import sys
import logging

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), 'lib')))

import collections
import errno
from pyang import plugin
from string import Template
import time
import optparse
import ordered_set
import ordereddict
from rpc import RpcGenerator
from globals import *

# These fields are not managed objects -- they are standard objects defined in the base class, etc.
NativeFields = [ 'name']

""" 

TODO:
1. add description for each field - leaf
2. add description for rpc
3. add description for notification

"""

def substringInSet(s,theSet):
  for elem in theSet:
    if s in elem: return elem
  return None

def pyang_plugin_init():
    """Called by pyang plugin framework at to initialize the plugin."""

    # Register the plugin
    plugin.register_plugin(Y2CPPPlugin())

class TemplateMgr:
    """ This class caches template files for later use to reduce disk access"""
    def __init__(self):
        self.cache = {}

    def loadPyTemplate(self, fname):
        """Load a template file and return it as a string"""
        if self.cache.has_key(fname): return self.cache[fname]

        f = open(fname, "r")
        s = f.read()
        t = Template(s)
        self.cache[fname] = t
        return t


# The global template manager
templateMgr = TemplateMgr()

def dropToPdb(type, value, tb):
    if hasattr(sys, 'ps1') or not sys.stderr.isatty():
        sys.__excepthook__(type, value, tb)
    else:
        import traceback, pdb
        traceback.print_exception(type, value, tb)
        print
        pdb.pm()


class Y2CPPPlugin(plugin.PyangPlugin):
    def __init__(self):
        self.done = set([])  # Helps avoiding processing modules more than once
        self.namespacePrefix = ""  # TODO: user can jam in a namespace that everything will be generated under

    def pre_load_modules(self,ctx):
        pass

    def add_opts(self, optparser):
        optlist = [
            optparse.make_option(
                "--y2cpp-output",
                dest = 'directory',
                help = 'Generate output to DIRECTORY.'),
            optparse.make_option("--y2cpp-help",
                                 dest = "y2cpp_help",
                                 action = "store_true",
                                 help = "Print help on usage of y2cpp plugin and exit"),
            optparse.make_option(
                "--y2cpp-sdkdir",
                dest = 'sdkdir',
                help = 'SAFplus SDK SRC directory located in.'),

            optparse.make_option(
                "--y2cpp-rpc",
                dest = 'rpc',
                action = "store_true",
                help = 'Generate SAFplus RPC protobuf.'),

            optparse.make_option(
                "--main-module",
                dest = 'mainModule',
                help = 'Specify what module should be the main one, so main file and main function will be generated.'),
            ]

        if hasattr(optparser, 'y2cpp_opts'):
            g = optparser.y2cpp_opts
        else:
            g = optparser.add_option_group("Y2CPP specific options")
            optparser.y2cpp_opts = g

        g.add_options(optlist)
        (o,self.yangSrcFiles) = optparser.parse_args()

    def add_output_format(self, fmts):
        self.multiple_modules = False
        fmts['y2cpp'] = self

    def emit(self, ctx, modules, fd):
        sys.excepthook = dropToPdb
        module_set = set(modules)
        num_modules = 0
        while num_modules != len(module_set):
            num_modules = len(module_set)
            for module in list(module_set):
                imported = map(lambda x: x.arg, search(module, 'import'))
                included = map(lambda x: x.arg, search(module, 'include'))
                for (module_stmt, rev) in self.ctx.modules:
                    if module_stmt in (imported + included):
                        module_set.add(self.ctx.modules[(module_stmt, rev)])

                """ Store prefix module imported """
                if module.arg not in mapping_module_import_prefix.keys():
                    mapping_module_import_prefix[module.arg] = dict()
                import_stmts = search(module, 'import')
                for import_stmt in import_stmts:
                    prefix = search_one(import_stmt, 'prefix')
                    if prefix is None:
                        mapping_module_import_prefix[module.arg][import_stmt.arg] = import_stmt.arg
                    else:
                        mapping_module_import_prefix[module.arg][prefix.arg] = import_stmt.arg

        '''
        Specific RPC and Class hierarchy in separate command
        '''
        if not self.ctx.opts.rpc:
            namespace = set([])
            # Generate files from main modules
            for module in filter(lambda s: s.keyword == 'module', module_set):
                self.generate_from(module)
                namespace.add(camelize(module.arg))
    
            data = {}
            data['include_gen_path'] = " \\\n            ".join(["-I" + self.ctx.opts.directory + "/" + ns for ns in namespace])
            data['libextralink'] = ''
            for ns in namespace:
              # Create Makefile
              templateFile = os.path.abspath(os.path.join(os.path.dirname(__file__), 'lib')) + os.sep + "Makefile.cpp.ts"
              try:
                tmpl = templateMgr.loadPyTemplate(templateFile)
                data['SDK_DIR'] = self.sdkdir
                data['YANG_MODULE'] = ns
    
                s = tmpl.safe_substitute(data)
                write_file(os.sep.join([self.ctx.opts.directory, ns]), "Makefile", s, self.ctx, F_WRITE)
              except AttributeError,e:
                print "Cannot load file %s, error %s.  So unable to generate Makefile" % (templateFile, str(e))

        ''' 
        Generator RPC services
        '''
        if self.ctx.opts.rpc:
            # Generate files from main modules
            for module in filter(lambda s: s.keyword == 'module', module_set):
                RpcGenerator(module, path = self.ctx.opts.directory, ctx=self.ctx)

        '''Generate main.cxx and its Makefile for the app'''
        data={}
        for module in namespace:
            if module==self.ctx.opts.mainModule:
                templateFile = os.path.abspath(os.path.join(os.path.dirname(__file__), 'lib')) + os.sep + "main.cpp.ts"
                try:
                    print 'yang module [%s], path [%s]' %(module,self.ctx.opts.directory)
                    tmpl = templateMgr.loadPyTemplate(templateFile)
                    data['YANG_MODULE'] = module
                    data['UC_YANG_MODULE'] = normalize(module)
                    s = tmpl.safe_substitute(data)
                    write_file(self.ctx.opts.directory, "main.cxx", s, self.ctx, F_WRITE)

                    data={}
                    templateFile = os.path.abspath(os.path.join(os.path.dirname(__file__), 'lib')) + os.sep + "Makefile.main.ts"
                    tmpl = templateMgr.loadPyTemplate(templateFile)
                    data['YANG_MODULE'] = module
                    data['UC_YANG_MODULE'] = module.upper()
                    s = tmpl.safe_substitute(data)
                    write_file(self.ctx.opts.directory, "Makefile", s, self.ctx, F_WRITE)
                except AttributeError,e:
                    print "Cannot load file %s, error %s.  So unable to generate Makefile" % (templateFile, str(e))
        # write yang source file
        if self.yangSrcFiles:
            for yangSrcFile in self.yangSrcFiles:
                fd = io.open(yangSrcFile, "r", encoding="utf-8")
                text = fd.read()
                fd.close()
                idx = yangSrcFile.rfind('/')
                if idx>=0:
                    fileName = yangSrcFile[idx:]
                else:
                    fileName = yangSrcFile
                fd = io.open("%s/%s" % (self.ctx.opts.directory,fileName), "w", encoding="utf-8")
                fd.write(text)
                fd.close()

    def setup_ctx(self, ctx):
        if ctx.opts.y2cpp_help:
            self.print_help()
            sys.exit(0)
        if ctx.opts.format == 'y2cpp':
            if not ctx.opts.directory:
                sys.stderr.write("Please input the output directory path --y2cpp-output <abc>\n")
                sys.exit(0)

            if not ctx.opts.sdkdir:
                sys.stderr.write('\nWARNING: SAFplus SDK does not provide. \nYou should manual modify the Makefile for each module!\n\n')

            if not ctx.opts.mainModule:
                sys.stderr.write('\nWARNING: Main module does not provide. \nYou should create your main file for your app manually\n\n')


        self.ctx = ctx
        self.d = ctx.opts.directory
        self.ctx = ctx
        self.sdkdir = '../..'
        if ctx.opts.sdkdir is not None:
            self.sdkdir = ctx.opts.sdkdir

        logging.basicConfig(stream=sys.stdout, format='%(levelname)s %(pathname)s:%(funcName)s:%(lineno)d %(message)s', level=logging.INFO)

    def setup_fmt(self, ctx):
        ctx.implicit_errors = False

    def print_help():
        print """
        Convert from .yang to C++ implementor
        """

    def generate_from(self, module):
        if module in self.done:
            return
        self.done.add(module)
        pkg = camelize(module.arg)
        generator = ClassGenerator(module,
                                   path = os.sep.join([self.ctx.opts.directory, pkg]), namespace = self.namespacePrefix + "::" + pkg, ctx = self.ctx)

        generator.generate()

""" This generator is used to generate key class for Yang list with multiple keys """
class MultiKeyClassGenerator(object):
    def __init__(self, keyList, name, path = None, module = None, isConfig = True):
        self.key_list = keyList
        self.path = '.'
        self.is_config = isConfig
        if path != None:
            self.path = path
        self.impl = ''
        self.n2 = camelize(name)
        self.class_name = "%sKey" %self.n2

        self.namespace = camelize(module.arg)
        self.rootpkg = self.namespace

        templateFileHeader = os.path.abspath(os.path.join(os.path.dirname(__file__), 'lib')) + os.sep + "Multikey.hxx.ts"
        templateFileImpl = os.path.abspath(os.path.join(os.path.dirname(__file__), 'lib')) + os.sep + "Multikey.cxx.ts"
        try:
          self.tmplHeader = templateMgr.loadPyTemplate(templateFileHeader)
          self.tmplImpl = templateMgr.loadPyTemplate(templateFileImpl)
        except AttributeError,e:
          print "Cannot load file %s, error %s.  So unable to generate Multikey" % (templateFile, str(e))

        # Add root to class_hierarchy dict
        if self.rootpkg not in class_hierarchy:
            class_hierarchy[self.rootpkg] = set([])
        class_hierarchy[self.rootpkg].add(self.class_name)

    def generate(self):
        BUILD_FIELD_TEMPLATE = Template('''
    iter = keyList.find("${key}");
    if(iter != keyList.end())
    {
      std::stringstream ${key}StrStream;
      ${key}StrStream << iter->second;
      ${key}StrStream >> ${key};
    }''')

        data = {}
        data['fields'] = "\n      ".join(map(lambda x: "%s %s;" % (x[1][1],x[0]) if x[1][0] is None else "%s %s;" % (x[1][0],x[0]), self.key_list))
        data['valueOut'] = " <<".join(map(lambda x: "value.%s" %x[0], self.key_list))
        data['toStringOut'] = ' "<<"'.join(map(lambda x: '%s=\\"" << %s << "\\"' %(x[0], x[0]), self.key_list))
        data['strOut'] = " <<".join(map(lambda x: "%s" %x[0], self.key_list))
        data['toXpathOut'] = ' << " and " '.join(map(lambda x: '<< "@%s=\\"" <<%s <<"\\""' %(x[0], x[0]), self.key_list))
        
        data['buildOut'] = "\n".join(map(lambda x: BUILD_FIELD_TEMPLATE.safe_substitute(key="%s" %x[0]), self.key_list))
        conArgs = []
        for x in self.key_list:
            ctype = x[1][1] if x[1][0] is None else x[1][0]
            findStds = filter(lambda s: ctype.find(s) != -1, cstdTypes)
            if len(findStds) > 0:
                conArgs.append("%s %sValue" % (ctype,x[0]))
            else:
                conArgs.append("%s &%sValue" % (ctype,x[0]))

        data['constructorParam'] = ", ".join(conArgs)
        data['constructorImpl'] = "\n    ".join(map(lambda x: "%s = %sValue;" %(x[0], x[0]), self.key_list))

        data['CLASSNAME'] = self.class_name.upper()
        data['className'] = self.class_name
        data['ns'] = self.namespace
        data['include'] = ''

        strHeader = self.tmplHeader.safe_substitute(data)
        strImpl = self.tmplImpl.safe_substitute(data)
        write_file(self.path, self.class_name + '.hxx', strHeader, None, F_OVERWRITE)
        write_file(self.path, self.class_name + '.cxx', strImpl, None, F_OVERWRITE)

class ClassGenerator(object):
    """Used to generate c++ classes from a yang module"""

    def __init__(self, stmt, path = None, namespace = None, ctx = None, parent = None):
        self.stmt = stmt
        self.path = path
        self.ctx = ctx
        self.namespace = namespace

        self.n = normalize(stmt.arg)
        self.n2 = camelize(stmt.arg)

        #print [ (x.keyword,x.arg) for x in self.stmt.i_children]
        #print [ (x.keyword,x.arg) for x in self.stmt.substmts]

        if stmt.arg == 'enumeration':
            self.n = normalize(getParent(stmt).arg)
            self.n2 = camelize(getParent(stmt).arg)

        self.nativeType = search_one(self.stmt, C_EXISTING)
        if self.nativeType:
          self.filename = None
          print "%s is using predefined (not generated) type '%s'" % (self.n, self.nativeType.arg)
        else:
          if stmt.keyword in module_stmts:
            # pdb.set_trace()
            self.filename = "%sCommon.hxx" %  ns2Filename(self.namespace)
            write_file(self.path, self.filename, [CUSTOMIZE_STR_FOR_COMMON_HEADER], ctx, F_WRITE)

          if self.stmt.keyword == 'module' and self.stmt.arg in yangStandardRFC:
            # Create <module>Common.cxx file to support SAFplus plugin architecture
            templateFile = os.path.abspath(os.path.join(os.path.dirname(__file__), 'lib')) + os.sep + "Common.cxx.ts"
            try:
              tmpl = templateMgr.loadPyTemplate(templateFile)
              dataSubsStr = {}
              dataSubsStr['yangModule'] = self.n
              dataSubsStr['ns'] = ns2Filename(self.namespace)
              dataSubsStr['className'] = ns2Filename(self.namespace)

              commonFile = "%sCommon.cxx" % ns2Filename(self.namespace)
              s = tmpl.safe_substitute(dataSubsStr)
              write_file(self.path, commonFile, s, self.ctx, F_WRITE)
            except AttributeError,e:
              print "Cannot load template file %s, error %s.  So unable to generate %s" % (templateFile, str(e), commonFile)

          else:
            self.filename = self.n + '.hxx'

        if parent is not None:
            for attr in ('namespace', 'ctx', 'path'):
                if getattr(self, attr) is None:
                    setattr(self, attr, getattr(parent, attr))

            module = getModule(stmt)
            self.rootpkg = camelize(module.arg)
        else:
            self.rootpkg = namespace

        # Add root to class_hierarchy dict
        if self.rootpkg not in class_hierarchy:
            class_hierarchy[self.rootpkg] = set([])
        class_hierarchy[self.rootpkg].add(self.n)


    def generate(self):
        """Generates class(es) for self.stmt"""

        if self.stmt.keyword in ('module', 'submodule'):
            self.generate_classes()
        else:
            self.generate_class()

    def generate_classes(self):
        assert(self.stmt.keyword == 'module')

        # Namespace and prefix
        ns_arg = search_one(self.stmt, 'namespace').arg
        prefix = search_one(self.stmt, 'prefix')

        # Add root to class_hierarchy dict
        if self.rootpkg not in class_hierarchy:
            class_hierarchy[self.rootpkg] = set([])
        class_hierarchy[self.rootpkg].add(self.n)

        # Add all classes that will be generated to class_hierarchy dict
        def record(stmt, namespace):
            for ch in search(stmt, yang_stmts + ['augment', 'grouping'] ):
                if namespace not in class_hierarchy:
                    class_hierarchy[namespace] = set([])
                class_hierarchy[namespace].add(normalize(ch.arg))
        record(self.stmt, self.rootpkg)

        # Gather typedefs to generate and add to class_hierarchy dict
        typedef_stmts = set([])
        module_stmts = set([self.stmt])
        included = map(lambda x: x.arg, search(self.stmt, 'include'))
        for (module, rev) in self.ctx.modules:
            if module in included:
                module_stmts.add(self.ctx.modules[(module, rev)])
        for module_stmt in module_stmts:
            for stmt in search(module_stmt, ['typedef', 'grouping',]):
                # Check to generate typedef class type or re-use existing C++
                if search_one(stmt, C_EXISTING) is None:
                    typedef_stmts.add(stmt)
                    class_hierarchy[self.rootpkg].add(normalize(stmt.arg))
                    try:
                        while True:
                            type_stmt = search_one(stmt, 'type')
                            if type_stmt.i_typedef is None:
                                if type_stmt.arg in ('enumeration',):
                                    #Put into typedef hashmap
                                    enum_type = normalize(getParent(type_stmt).arg)
                                    enum_manager = "%sManager" %enum_type
                                    mapping_typedef_enum[enum_type] = "::" + self.rootpkg + "::" + enum_type
                                    mapping_typedef_enum[enum_manager] = "::%s::%s" %(self.rootpkg, enum_manager)
                                break
                            stmt = type_stmt.i_typedef
                            # Check to generate typedef class type or re-use existing C++
                            if search_one(stmt, C_EXISTING) is None:
                                class_hierarchy[self.rootpkg].add(normalize(stmt.arg))
                                typedef_stmts.add(type_stmt.i_typedef)
                    except AttributeError:
                        pass

        # Generate the typedef and grouping (containment) classes
        for stmt in typedef_stmts:
            child_generator = ClassGenerator(stmt, path = self.path, namespace = self.namespace, ctx = self.ctx)
            child_generator.generate()

        childClasses = set([])
        # Generate classes for children and keep track of augmented modules
        for stmt in search(self.stmt, yang_stmts + ['augment'] + ['rpc']):
            child_generator = ClassGenerator(stmt, path = self.path, namespace = self.namespace, ctx = self.ctx)
            child_generator.generate()
            childClasses.add(child_generator.n2)

        #Generate top level class name
        if len(childClasses) > 0:

            topModuleCppClass = CppClassGeneric(self.stmt, self.path, namespace = self.namespace, ctx = self.ctx)

            """ Yang object generator """
            yangObject = ObjectGenerator(self.stmt, self.ctx);
    
            """ Constructors """
            for constructor in yangObject.constructors():
                topModuleCppClass.addConstructor(constructor)
    
            """ Method """
            for method in yangObject.methods():
                topModuleCppClass.addMethod(method)
    
            """ Member """
            topModuleCppClass.setMembers(yangObject.memberFields())
    
            """ Include header children """
            if 0: # Already updated from its substmts
              for ch in stmt.substmts:
                if ch.keyword in yang_stmts:
                  nativeType = search_one(ch, C_EXISTING)
                  if nativeType is not None:
                    pass
                  else:
                    ch_arg = normalize(ch.arg)
                    topModuleCppClass.include.add(ch_arg)
    
            """ Destructor """
            for destructor in yangObject.destructors():
                topModuleCppClass.addDestructor(destructor)
    
            """ Done write to file """
            topModuleCppClass.write_file()

    def generate_class(self):
        """Generates a c++ class hierarchy providing an interface to a YANG
        module. Uses mutual recursion with generate_child.
        """
        stmt = self.stmt

        """ Yang object generator """
        yangObject = ObjectGenerator(stmt, self.ctx);

        if yangObject.nativeType:  # If this class is not represented by a "native" type then we must generate it.
          self.cppClass = None
        else:
          self.cppClass = CppClassGeneric(stmt, self.path, namespace = self.namespace, ctx = self.ctx)

          # """ Constructors """
          for constructor in yangObject.constructors():
              self.cppClass.addConstructor(constructor)

          # """ Method """
          for method in yangObject.methods():
              self.cppClass.addMethod(method)

          tmp = yangObject.memberFields()
          #tmp = filter(lambda x: x.name != 'name', tmp) # References to 'name' refer to the existing field in MgtObject so no need to generate it

          self.cppClass.setMembers(tmp)

          for ch in search(stmt, yang_stmts + leaf_stmts + ['input', 'output']):
            if hasattr(ch, "i_uses") == False:
                self.generate_child(ch)

          # """ Destructor """
          for destructor in yangObject.destructors():
            self.cppClass.addDestructor(destructor)

          # """ Generator ENUM typedef """
          self.cppClass.addEnumType(yangObject.enums())

          # """ Done write to file """
          self.cppClass.write_file()

    def generate_child(self, substmt):
        if substmt.keyword in yang_stmts + leaf_stmts:
            if substmt.keyword in yang_stmts:
                child_generator = ClassGenerator(stmt = substmt, path = self.path, namespace = self.namespace, parent = self)
                child_generator.generate()

            """ Find enum type """
            if substmt.keyword in leaf_stmts:
                child_type = search_one(substmt, 'type')
                if child_type.arg in ('enumeration',):
                    #Put into typedef hashmap
                    child_generator = ClassGenerator(child_type, path = self.path, namespace = self.namespace, parent = self)
                    child_generator.generate()

            childGen = ObjectGenerator(substmt, self.ctx)
            for method in childGen.parentMethods():
                method.setClassname(self.n)
                self.cppClass.addMethod(method)

        elif substmt.keyword in ('input', 'output',):
            for param in search(substmt, yang_stmts + leaf_stmts):
                if param.keyword in yang_stmts:
                    child_generator = ClassGenerator(stmt = param, path = self.path, namespace = self.namespace, parent = self)
                    child_generator.generate()

                childGen = ObjectGenerator(param, self.ctx)
                for method in childGen.parentMethods():
                    method.setClassname(self.n)
                    self.cppClass.addMethod(method)

                """ Find enum type """
                if substmt.keyword in leaf_stmts:
                    child_type = search_one(substmt, 'type')
                    if child_type.arg in ('enumeration',):
                        child_generator = ClassGenerator(child_type, path = self.path, namespace = self.namespace, parent = self)
                        child_generator.generate()

class CppClassGeneric(object):
    def __init__(self, stmt, path = None, namespace = None, ctx = None, indent = 4, document = None):
        self.ctx = ctx
        self.path = path
        self.ns = namespace
        self.methods = ordered_set.OrderedSet()
        self.members = []

        self.stmt = stmt
        self.n = normalize(stmt.arg)
        self.n2 = camelize(stmt.arg)

        if stmt.arg == 'enumeration':
            self.n = normalize(getParent(stmt).arg)
            self.n2 = camelize(getParent(stmt).arg)

        if stmt.keyword == 'module':
            # pdb.set_trace()
            self.n += "Module"
            self.n2 += "Module"
            self.xpath = ""  # Modules have no xpath
        else:
            self.xpath = fullPath(stmt)

        #if stmt.keyword in module_stmts:
        #    self.filenameImpl = normalize(search_one(stmt, 'prefix').arg) + '.cxx'
        #    self.filenameHdr = normalize(search_one(stmt, 'prefix').arg) + '.hxx'
        #else:
        self.filenameImpl = self.n + '.cxx'
        self.filenameHdr = self.n + '.hxx'

        self.indent = indent

        self.hdrFileData = {'comment':[], 'begin_defined':[], 'include':[], 'namespace':[], 'body':[], 'end_namespace':[], 'end_defined':[]}
        self.ImplFileData = {'comment':[], 'include':[], 'namespace':[], 'body':[], 'end_namespace':[]}

        self.namespace = set(['std'])
        self.include = set([])

        self.superclass = self.getSuperclass(stmt.keyword)

        """ Update namespace and header for the field """
        self.setNamespaceInclude()

        self.document = None
        if document is not None:
            self.document = document

        # Enumeration type defined
        self.enumtypes = {}

    def setDocument(self, doc):
        self.document = doc

    def getSuperclass(self, string):
        """ TODO for uses statements """
        superclass = set([])

        #Do generator for current module
        cppGenModuleName = getModule(self.stmt).arg
        for stmt_use in self.stmt.substmts:
            #Using derived from uses (but take care with keyword refine)
            if stmt_use.keyword in ('uses',):
                try:
                    #Using external class from imported
                    module_arg, classType = str(stmt_use.arg).split(":")
                    superclass.add(mapping_module_import_prefix[cppGenModuleName][module_arg] + "::" + normalize(classType))
                    self.include.add(normalize(classType))
                except:
                    #Using internal class namespace
                    superclass.add(normalize(stmt_use.arg))
                    self.include.add(normalize(stmt_use.arg))

        if len(superclass) == 0:
            # Now, list has the MgtContainer as supper class
            try:
                inheritString = mapping_stmts_client[string]
                superclass.add(inheritString)
            except KeyError:
                pass

        # To support as SAFplus Plugin Architecture
        if self.stmt.keyword == 'module' and self.stmt.arg in yangStandardRFC:
          superclass.add('SAFplus::ClPlugin')

        return superclass

    """ param constructor is a CppMethod type """
    def addConstructor(self, constructor):
        """Adds a constructor represented as a string"""

        #if self.n == "Date":
        #  pdb.set_trace()

        if len(self.superclass) > 0:
            """ Provide name mgt object if class drived from SAFplus::MgtObject """
            mgtListTemplate = substringInSet('SAFplus::MgtList',self.superclass)
            if 'SAFplus::MgtObject' in self.superclass:
              constructor.addInitFirst("SAFplus::MgtObject(\"%s\")" %self.stmt.arg)
            elif 'SAFplus::MgtContainer' in self.superclass:
              constructor.addInitFirst("SAFplus::MgtContainer(\"%s\")" %self.stmt.arg)
            elif mgtListTemplate: 
              constructor.addInitFirst('%s("%s")' % (mgtListTemplate,self.stmt.arg))
            elif 'SAFplus::MgtProv' in self.superclass:
              constructor.addInitFirst("SAFplus::MgtProv(\"%s\")" %self.stmt.arg)
            elif 'SAFplus::MgtNotify' in self.superclass:
              constructor.addInitFirst("SAFplus::MgtNotify(\"%s\")" %self.stmt.arg)
            elif 'SAFplus::MgtRpc' in self.superclass:
              constructor.addInitFirst("SAFplus::MgtRpc(\"%s\")" %self.stmt.arg)
            elif 'SAFplus::MgtModule' in self.superclass:
              constructor.addInitFirst("SAFplus::MgtModule(\"%s\")" %self.stmt.arg)
            else:
              constructor.addLine("this->tag.assign(\"%s\");" %self.stmt.arg)
#            constructor.addInitFirst("SAFplus::MgtObject(\"%s\")" %self.stmt.arg)

        self.methods.add(constructor)

    """ param method is a CppMethod type """
    def addMethod(self, method):
        self.methods.add(method)
        self.updateNamespaceInclude(method)

    """ param destructor is a CppMethod type """
    def addDestructor(self, destructor):
        self.methods.add(destructor)

    """ Enum defined """ 
    def addEnumType(self, enumtypes):
        self.enumtypes = enumtypes

    """ Member is a CppField type """
    def setMembers(self, members):
        self.members = members
        self.setNamespaceInclude()

    def updateNamespaceInclude(self, obj):
        """ Update namespace and header for the field """
#        print obj.include
#        pdb.set_trace()
        self.namespace = self.namespace | obj.namespace
        self.include = self.include | obj.include

        if 0:
          if getattr(self, 'namespace') is None:
            setattr(self, 'namespace', getattr(obj, 'namespace'))
          else:
            setattr(self, 'namespace', getattr(self, 'namespace') | getattr(obj, 'namespace'))

          if getattr(self, 'include') is None:
            setattr(self, 'include', getattr(obj, 'include'))
          else:
            setattr(self, 'include', getattr(self, 'include') | getattr(obj, 'include'))

    def setNamespaceInclude(self):
        includes = set([])
        namespaces = set([])

        """ Add header for management object factory """
        if len(self.superclass) > 0 and self.stmt.keyword not in ('grouping',):
            includes.add('MgtFactory')

        for derived in self.superclass:
            """ Include super class header """
            try:
                includes.add(mapping_include[derived])
            except KeyError:
                pass

        for member in self.members:
                namespaces.update(member.namespace)
                #print member.include
                includes.update(member.include)

        for method in self.methods:
                namespaces.update(method.namespace) 
                #print method.include
                includes.update(method.include)

        self.namespace.update(namespaces)
        self.include.update(includes)

    """ Output comment """
    def toCommentString(self, fileName):
        comment = []
        comment.append("/* ")
        comment.append(" * File %s" % fileName)
        comment.append(" * This file has been auto-generated by Y2CPP, the")
        comment.append(" * plug-in of pyang.")
        comment.append(" */ ")
        return comment

    """ Output define """
    def beginDefinedString(self):
        defined = []
        defined.append('')
        defined.append("#pragma once\n#ifndef %s_HXX_" % self.n.upper())
        defined.append("#define %s_HXX_" % self.n.upper())
        return defined

    """ Output header include """
    def includeString(self):
        # Ignore duplicate includes
        includes = set([ '#include "%sCommon.hxx"' %  ns2Filename(self.ns)])

        if len(self.include) > 0:  # len(getattr(self, 'include')) > 0:
            includes.add('')
            for include in self.include:  # getattr(self, 'include'):
                include = include.replace("&", "")  # remove any references or pointers because they won't be part of the header file name
                include = include.replace("*", "")
                if isNamespaceStd(include):
                    includes.add("#include <%s>" % include)
                elif include.find('boost') != -1:
                    includes.add("#include <%s>" % include)
                elif type2include.has_key(include):
                  includes.add(type2include[include])
                else:
                    include = getCplusCplusString(include)
                    if include != self.n:
                        #print include, self.n
                        includes.add('#include "%s.hxx"' % include)
        return list(includes)

    """ Output open namespace tag """
    def namespaceString(self, impl = False):
        namespaces = []

        if impl:
            if len(getattr(self, 'namespace')) > 0:
                namespaces.append('')
                for namespace in getattr(self, 'namespace'):
                    #Do used std any more, it is already using full class namespace
                    if namespace != self.ns and namespace != 'std' and namespace:
                        ns = namespace.replace("const","")
                        namespaces.append("using namespace %s;" % ns)

        namespaces.append('')
        # remove preceding :: and other namespaces
        # TODO: nested namespace decl
        t = self.ns.split("::")
        namespaces.append("namespace %s\n  {" % t[-1])
        namespaces.append('')

        return namespaces

    """ Write enumeration defined """
    def enumHdrString(self):
        enums = []
        if len(self.enumtypes.keys()) > 0:
            for (k, v) in self.enumtypes.items():
                enums.append(' ' * self.indent + "enum class %s" % k);
                enums.append(' ' * self.indent + "  {")
                enums.append(' ' * self.indent * 2 + ", ".join(map(lambda x: x[0] if x[1] is None else "%s=%s" % (x[0],x[1]),v)))
                enums.append(' ' * self.indent + "  };")
                enums.append(' ' * self.indent + "std::ostream& operator<<(std::ostream& os, const %s& e);" % k)
                enums.append(' ' * self.indent + "std::istream& operator>>(std::istream& is, %s& e);" % k)
                enums.append(' ' * self.indent + "const char* c_str(const %s& e);" % k)

            enums.append('')

            """ Map init helper on enum """
            document = [' ' * self.indent + '/*']
            document.extend([' ' * self.indent + ' * ' + "This is the class that will handle the conversion for us."])
            document.extend([' ' * self.indent + ' */'])
            enums.extend(document)

            enums.append(' ' * self.indent + "class %sManager : public SAFplus::MgtEnumType<%sManager, %s> {" % (self.enumtypes.keys()[0], self.enumtypes.keys()[0], self.enumtypes.keys()[0]))
            enums.append(' ' * self.indent * 2 + "%sManager();  // private to prevent instantiation" %self.enumtypes.keys()[0])
            enums.append(' ' * self.indent + "public:")
            enums.append(' ' * self.indent * 2 + "static const map_t en2str_map;  // This is the lookup table.")
            enums.append(' ' * self.indent + "};")


        return enums

    def enumImplString(self):
        enums = []
        if len(self.enumtypes.keys()) > 0:
            document = [' ' * self.indent + '/*']
            document.extend([' ' * self.indent + ' * ' + "Provide an implementation of the en2str_map lookup table."])
            document.extend([' ' * self.indent + ' */'])
            enums.extend(document)


            enums.append(' ' * self.indent + "const %sManager::map_t %sManager::en2str_map = {" %(self.enumtypes.keys()[0], self.enumtypes.keys()[0]))
#            for v in self.enumtypes.get(self.enumtypes.keys()[0]):
#                enums.append(' ' * self.indent * 3 + "pair_t(%s::Invalid, \"%s\"" %(self.enumtypes.keys()[0], v))

            enums.append((',\n').join(map(lambda x: " " * self.indent * 3 + "pair_t(%s::%s, \"%s\")" %(self.enumtypes.keys()[0], x[0], x[0]), self.enumtypes.get(self.enumtypes.keys()[0]))))
            enums.append(' ' * self.indent + "}; // uses c++11 initializer lists ")

            enums.append('')
            #Write method implement 
            for method in self.methods:
                """ Add document description for method if available """
                if method.document is not None:
                    document = [' ' * self.indent + '/*']
                    if isinstance(method.document, list):
                        document.extend([' ' * self.indent + ' * ' + doc for doc in method.document])
                    else:
                        document.extend([' ' * self.indent + ' * ' + method.document])
                    document.extend([' ' * self.indent + ' */'])
                    enums.extend(document)

                if method.modifier is None or method.modifier not in ('friend',):
                    enums.extend([' ' * self.indent + line for line in method.toImpl()])
                    enums.append('')

            """ Friend method implementation here """
            for method in self.methods:
                if method.modifier is not None and method.modifier in ('friend',):
                    enums.extend([' ' * self.indent + line for line in method.toImpl()])
                    enums.append('')

        return enums

    """ Output prototype """
    def bodyHdrString(self):
        body = []
        if len(self.members) > 0 or len(self.methods) > 0:
            """ Do specific for enumeration """
            if len(self.enumtypes.keys()) > 0:
                pass
            else:

#                drivedStr = ', public virtual '.join(self.superclass)
                drivedStr = ', public '.join(self.superclass)

                if len(drivedStr) > 0:
                    body.append(' ' * self.indent + "class %s : public %s {" % (self.n, drivedStr))
#                    body.append(' ' * self.indent + "class %s : public virtual %s {" % (self.n, drivedStr))
                else:
                    body.append(' ' * self.indent + "class %s {" % self.n)

                if len(self.superclass) > 0 and self.stmt.keyword not in ('grouping',):
                    """ Register to object factory """
                    body.append('')
                    body.append(' ' * 2 * self.indent + "/* Apply MGT object factory */")
                    body.append(' ' * 2 * self.indent + "MGT_REGISTER(%s);" %self.n )
                    body.append('')

                if len(self.members) > 0:
                    """ Modifier member default is public """
                    body.append(' ' * self.indent + "public:")
                    for member in self.members:
    
                        """ Add document description for field if available """
                        if member.document is not None:
                            document = ['', ' ' * 2 * self.indent + '/*']
                            if isinstance(member.document, list):
                                document.extend([' ' * 2 * self.indent + ' * ' + doc for doc in member.document])
                            else:
                                document.extend([' ' * 2 * self.indent + ' * ' + member.document])
                            document.extend([' ' * 2 * self.indent + ' */'])
                            body.extend(document)
            
                        body.extend([' ' * 2 * self.indent + member.toString() + ";"])
                    body.append('')
        
                """ Modifier method default is public """
                body.append(' ' * self.indent + "public:")
                for method in self.methods:
                    
                    """ Add document description for method if available """
                    if method.document is not None:
                        document = ['', ' ' * 2 * self.indent + '/*']
                        if isinstance(method.document, list):
                            document.extend([' ' * 2 * self.indent + ' * ' + doc for doc in method.document])
                        else:
                            document.extend([' ' * 2 * self.indent + ' * ' + method.document])
                        document.extend([' ' * 2 * self.indent + ' */'])
                        body.extend(document)
        
                    body.extend([' ' * 2 * self.indent + line for line in method.toHdr()])
        
                body.append('')
                body.append(' ' * self.indent + '};')

        return body

    """ Output implementation """
    def bodyImplString(self):
        body = []
        if len(self.members) > 0 or len(self.methods) > 0:
            """ Do specific for enumeration """
            if len(self.enumtypes.keys()) > 0:
                pass
            else:
                if len(self.superclass) > 0 and self.stmt.keyword in ("list",):  # what are the objects that can be dynamically created by the operator? -- I think only those that appear under a list
                    body.append(' ' * self.indent + "/* Apply MGT object factory */")
                    body.append(' ' * self.indent + "MGT_REGISTER_IMPL(%s, /%s)" % (self.n, self.xpath))
                    body.append('')
        
                for method in self.methods:
                    
                    """ Add document description for method if available """
                    if method.document is not None:
                        document = [' ' * self.indent + '/*']
                        if isinstance(method.document, list):
                            document.extend([' ' * self.indent + ' * ' + doc for doc in method.document])
                        else:
                            document.extend([' ' * self.indent + ' * ' + method.document])
                        document.extend([' ' * self.indent + ' */'])
                        body.extend(document)
        
                    if method.modifier is None or method.modifier not in ('friend',):
                        body.extend([' ' * self.indent + line for line in method.toImpl()])
                        body.append('')
        
                """ Friend method implementation here """
                for method in self.methods:
                    if method.modifier is not None and method.modifier in ('friend',):
                        body.extend([' ' * self.indent + line for line in method.toImpl()])
                        body.append('')

        return body

    """ Output close the namespace tag """
    def endNamespaceString(self):
        return ["}", "/* namespace %s */" % self.ns]

    """ Output end define header """
    def endDefinedString(self):
        return ["#endif /* %s_HXX_ */" % self.n.upper()]

    """ All the content header """
    def dataHdrFile(self):
        return self.toCommentString(self.filenameHdr) + self.beginDefinedString() + \
            self.includeString() + self.namespaceString() + self.enumHdrString() + self.bodyHdrString() + \
            self.endNamespaceString() + self.endDefinedString()

    """ All contents implementation """
    def dataImplFile(self):
        return self.toCommentString(self.filenameImpl) + self.includeString() + ["#include \"%s\"" % self.filenameHdr] + self.namespaceString(True) + self.enumImplString() + self.bodyImplString() + self.endNamespaceString()

    def write_file(self):
        """ 
        This is essentially a hidden class, overwrite it is existing.
        Otherwise, Do not touch if file is already existing maybe custom code added
        """
        mode_create_file = F_OVERWRITE
        # We want everything to be overwritten except for the specifix xxxxCommon.hxx file:
        # if 'SAFplus::MgtObject' in self.superclass or 'SAFplus::MgtList' in self.superclass or 'SAFplus::MgtContainer' in self.superclass or not self.superclass:
        #    mode_create_file = F_OVERWRITE
        if "Common.hxx" in self.filenameHdr:
          mode_create_file = F_WRITE
        print "Generate %s mode %s, at %s" % (self.filenameHdr, "Overwrite" if mode_create_file == F_OVERWRITE else "Write",self.path)
        """ Generate header file .hxx """
        write_file(self.path, self.filenameHdr, self.dataHdrFile(), self.ctx, mode_create_file)

        print "Generate %s mode %s" % (self.filenameImpl, "Overwrite" if mode_create_file == F_OVERWRITE else "Write")
        """ Generate implementation file .cxx """
        write_file(self.path, self.filenameImpl, self.dataImplFile(), self.ctx, mode_create_file)


class ObjectGenerator(object):
    """A generator for generic management objects"""

    def __init__(self, stmt, ctx):
        """Sets the attributes of the method generator, depending on stmt"""
        self.stmt = stmt

        self.n = normalize(stmt.arg)
        self.n2 = camelize(stmt.arg)

        self.nativeType = search_one(self.stmt, C_EXISTING)
        if self.nativeType:
          print "%s is using predefined (native) type '%s'" % (self.n, self.nativeType.arg)

        if stmt.arg == 'enumeration':
            self.n = normalize(getParent(stmt).arg)
            self.n2 = camelize(getParent(stmt).arg)

        if stmt.keyword == 'module':
            self.n += "Module"
            self.n2 += "Module"

        self.children = [normalize(s.arg) for s in search(stmt, yang_stmts + leaf_stmts)]

        self.ctx = ctx
        self.module_stmt = getModule(stmt)
        prefix = search_one(self.module_stmt, 'prefix')
        self.root = normalize(prefix.arg)

        self.rootpkg = camelize(self.module_stmt.arg)
        self.namespace = self.rootpkg

        self.is_container = stmt.keyword == 'container'
        self.is_list = stmt.keyword == 'list'
        self.is_typedef = stmt.keyword == 'typedef'
        self.is_leaf = stmt.keyword == 'leaf'
        self.is_leaflist = stmt.keyword == 'leaf-list'
        self.is_rpc = stmt.keyword == 'rpc'
        self.is_notification = stmt.keyword == 'notification'
        self.is_grouping = stmt.keyword == 'grouping'

        #Enum gen
        self.is_enumeration = stmt.arg == 'enumeration'

        self.is_top_level = self.stmt.keyword == 'module'

        assert (self.is_top_level or self.is_container or self.is_list or self.is_typedef
                or self.is_leaf or self.is_leaflist or self.is_notification or self.is_rpc or self.is_enumeration or self.is_grouping)

        self.gen = self

        if type(self) is ObjectGenerator:
            if self.is_typedef:
                self.gen = TypedefObjectGenerator(stmt, ctx)
            elif self.is_container or self.is_top_level or self.is_grouping:
                self.gen = ContainerObjectGenerator(stmt, ctx)
            elif self.is_list:
                self.gen = ListObjectGenerator(stmt, ctx)
            elif self.is_leaf or self.is_leaflist:
                self.gen = LeafObjectGenerator(stmt, ctx)
            elif self.is_notification:
                self.gen = NotifyObjectGenerator(stmt, ctx)
            elif self.is_rpc:
                self.gen = RpcObjectGenerator(stmt, ctx)
            elif self.is_enumeration:
                self.gen = EnumObjectGenerator(stmt, ctx)

    def default_constructor(self):
        assert not self.is_leaf and not self.is_leaflist
        constructor = CppConstructor(classObj=self, classname = self.n, name = self.n)
        #constructor.addLine('this->name = "%s";' % self.n2)
        return constructor

    def default_destructor(self):
        assert not self.is_leaf and not self.is_leaflist
        destructor = CppMethod(classname = self.n, name = "~" + self.n)
        return destructor

    def constructors(self):
        assert self.gen is not self, 'Avoid infinite recursion'
        return self.gen.constructors()

    def destructors(self):
        assert self.gen is not self, 'Avoid infinite recursion'
        return self.gen.destructors()

    def memberFields(self):
        assert self.gen is not self, 'Avoid infinite recursion'
        return self.gen.memberFields()

    def methods(self):
        assert self.gen is not self, 'Avoid infinite recursion'
        return self.gen.methods()

    def parentMethods(self):
        assert self.gen is not self, 'Avoid infinite recursion'
        return self.gen.parentMethods()

    def genAccessorMethods(self,child,child_arg=None,child_type=None):
      xpath = "XPATH: /" + fullPath(child)

      if child_arg is None: child_arg = camelize(child.arg)
      if child_type is None: valuetype = leafType(child,self.ctx)
      else:
        valuetype = getTypes(child_type, self.ctx)
 
      if child.arg in NativeFields:
        getImpl = "return this->%s;" % child.arg
      else:
        getImpl = "return this->%s.value;" % camelize(child.arg)

      if child.arg in NativeFields:
        setImpl = "this->%s = %sValue;" % (child.arg, camelize(child.arg))
      else:
        if child.keyword == 'leaf':
          setImpl = "this->%s.set(%sValue,txn);" % (camelize(child.arg), child_arg)
        else:
          setImpl = "this->%s.value.push_back(%sValue);" % (camelize(child.arg), child_arg)

      #if not hasattr(child, 'nativeType'):
      #  print "WARNING: child [%s] does not have nativeType checking" % str(child.__class__)
      #if hasattr(child, 'nativeType') and child.nativeType:
      #  rt = child.nativeType.arg;

      #nativeType = search_one(child, C_EXISTING)
      #print nativeType
      #if nativeType:
      #  rt = nativeType.arg
      if child.keyword == 'leaf':
        rt = valuetype
        tct = None
      else:
        rt = ["std::vector", "vector"]
        tct = valuetype

      """ getValue method """
      getValueMethod = CppMethod(classname = self.n, name = "get%s" % capitalizeFirst(child_arg), returnType = rt,templateClassType = tct)
      getValueMethod.addLine(getImpl)

      """ Add document for method """
      getValueMethod.setDocument(xpath)

      """ setValue method """
      setValueMethod = CppMethod(classname = self.n, name = "set%s" % capitalizeFirst(child_arg))

      field          = CppField(valueType = valuetype, name = camelize(child.arg) + "Value", isReference=True)
      txnField       = CppField(valueType = ["SAFplus::Transaction","Transaction"], name = "txn",isReference=True,defaultValue="SAFplus::NO_TXN")
      setValueMethod.addField(field)
      setValueMethod.addField(txnField)
      setValueMethod.addLine(setImpl)

      """ Add document for method """
      setValueMethod.setDocument(xpath)

      return [getValueMethod,setValueMethod]

    def addListFields(self):
        ret = []
        # Add yang list type field
        for substmt in search(self.stmt, ['list']):
            # Detect multiple key or single key
            if len(substmt.i_key) > 1: 
                templateType = ("SAFplus::" + normalize(substmt.arg) + "Key",normalize(substmt.arg) + "Key")
                field = CppField(valueType = ["SAFplus::MgtList", "MgtList"], templateClassType = templateType, name = lowerFirst(substmt.arg + "List"), objName = substmt.arg)
                field.addInclude("%s"%normalize(substmt.arg) + "Key")
            elif len(substmt.i_key) == 1:
                listKeyType = substmt.i_key[0]
                templateType = getTypes(listKeyType, self.ctx)
                if templateType[1] != 'string':
                    templateType = ("SAFplus::" + normalize(substmt.arg) + "Key",normalize(substmt.arg) + "Key")
                    field = CppField(valueType = ["SAFplus::MgtList", "MgtList"], templateClassType = templateType, name = lowerFirst(substmt.arg + "List"), objName = substmt.arg)
                    field.addInclude("%s"%normalize(substmt.arg) + "Key")
                else:
                    field = CppField(valueType = ["SAFplus::MgtList", "MgtList"], templateClassType = templateType, name = lowerFirst(substmt.arg + "List"), objName = substmt.arg)

            description = search_one(substmt, 'description')
            if description is not None:
                field.setDocument(description.arg)
            ret.append(field)
        return ret


    def enums(self):
        return self.gen.enums()

class CppMethod():
    def __init__(self, returnType = None, templateClassType = None, classname = None, name = None, params = None, modifier = None, isPointer = False, isReference = False, indent = 4, document = None,isConst = False):
        self.fields = params
        if self.fields is None:
            self.fields = []
        self.classname = classname
        self.name = name
        self.indent = indent

        self.isconstructor = (self.name == self.classname)
        self.isdestructor = self.name.find("~") != -1
        self.extra = ordered_set.OrderedSet()

        self.returnType = [None, "void"]
        self.templateClassType = None

        if returnType is not None:
            self.returnType = getFullNamespace(returnType)

        if templateClassType is not None:
            self.templateClassType = templateClassType

        self.body = []

        self.namespace = set(['std'])
        self.include = set([])
        self.modifier = modifier #friend, static ...

        """ Update namespace and header for the field """
        self.setNamespaceInclude()
        self.isConst = isConst
        self.isPointer = isPointer
        self.isReference = isReference

        self.document = None
        if document is not None:
            self.document = document

    def setDocument(self, doc):
        self.document = doc

    def setReturnType(self, returnType):
        self.returnType = getFullNamespace(returnType)

    def addNamespace(self, namespace):
        self.namespace.add(namespace)

    def addInclude(self, include):
        self.include.add(include)

    def setClassname(self, classname):
        self.classname = classname

    """ Add extra init for parent object in case constructor """
    def addInitFirst(self, extra):
        assert self.isconstructor
        self.extra.add_first(extra)

    """ Add extra init for parent object in case constructor """
    def addInit(self, extra):
        assert self.isconstructor
        self.extra.add(extra)

    def addLine(self, line):
        if isinstance(line, list):
            self.body.extend(line)
        else:
            self.body.append(line)

    def addField(self, field):
        self.fields.append(field)

        """ Update namespace and header for the field """
        if getattr(self, 'namespace') is None:
            setattr(self, 'namespace', getattr(field, 'namespace'))
        else:
            setattr(self, 'namespace', set(getattr(self, 'namespace') | getattr(field, 'namespace')))

        if getattr(self, 'include') is None:
            setattr(self, 'include', getattr(field, 'include'))
        else:
            setattr(self, 'include', set(getattr(self, 'include') | getattr(field, 'include')))

    def fieldString(self, impl = False):
        if len(self.fields) > 0:
            return ', '.join([f.toString(impl) for f in self.fields])
        return None

    def toImpl(self):
        res = []
        method = ''
        if not (self.isconstructor or self.isdestructor):
            if self.templateClassType is not None:
                strTemp = ''
                tpltype = getFullNamespace(self.templateClassType)
                i = 0
                while not isStringTypes(getFullNamespace(tpltype[1])):
                    strTemp += '<' + tpltype[0]
                    tpltype = getFullNamespace(tpltype[1])
                    i += 1

                if tpltype[0] is not None:
                    tplTypeStr = tpltype[0]
                else:
                    tplTypeStr = tpltype[1]
                    if isNamespaceStd(tpltype[1]):
                        tplTypeStr = "std::%s" % tpltype[1]

                strTemp += '<' + tplTypeStr + '>' + ' >' * i

                #Priority get full type i.e [std::vector, vector]
                if isNamespaceStd(self.returnType[1]):
                    method += 'std::%s%s' % (self.returnType[1], strTemp)
                else:
                    if self.returnType[0] is not None:
                        method += '%s%s' % (self.returnType[0], strTemp)
                    else:
                        method += '%s%s' % (self.returnType[1], strTemp)
            else:
                if isNamespaceStd(self.returnType[1]):
                    method += "std::%s" % self.returnType[1]
                else:
                    if self.returnType[0] is not None:
                        method += "%s" % self.returnType[0]
                    else:
                        method += "%s" % self.returnType[1]

        if self.isPointer:
            method += "*"

        if self.isReference:
            method += "&"

        #Add extra space to separate return type and name method (but not constructor/destructor)
        if len(method) > 0:
            method += ' '

        if self.modifier is not None and self.modifier in ('friend',):
            method += "%s(" % self.name
        else:
            method += "%s::%s(" % (self.classname, self.name)

        if self.fieldString(True) != None:
            method += self.fieldString(True)
        method += ")"
        if self.isConst: method += " const"
        if len(self.extra) > 0:
            method += ": %s" % (', '.join(self.extra))

        res.append(method)

        res.append('{')
        res.extend([' ' * self.indent + line for line in self.body])
        res.append("};")
        return res

    def toHdr(self):
        method = ''
        if not (self.isconstructor or self.isdestructor):
            if self.modifier is not None:
                method += self.modifier + " "

            if self.templateClassType is not None:
                strTemp = ''
                tpltype = getFullNamespace(self.templateClassType)
                i = 0
                while not isStringTypes(getFullNamespace(tpltype[1])):
                    strTemp += '<' + tpltype[0]
                    tpltype = getFullNamespace(tpltype[1])
                    i += 1

                if tpltype[0] is not None:
                    tplTypeStr = tpltype[0]
                else:
                    tplTypeStr = tpltype[1]
                    if isNamespaceStd(tpltype[1]):
                        tplTypeStr = "std::%s" % tpltype[1]

                strTemp += '<' + tplTypeStr + '>' + ' >' * i

                #Priority get full type i.e [std::vector, vector]
                if isNamespaceStd(self.returnType[1]):
                    method += 'std::%s%s' % (self.returnType[1], strTemp)
                else:
                    if self.returnType[0] is not None:
                        method += '%s%s' % (self.returnType[0], strTemp)
                    else:
                        method += '%s%s' % (self.returnType[1], strTemp)
            else:
                if isNamespaceStd(self.returnType[1]):
                    method += "std::%s" % self.returnType[1]
                else:
                    if self.returnType[0] is not None:
                        method += "%s" % self.returnType[0]
                    else:
                        method += "%s" % self.returnType[1]

        if self.isPointer:
            method += "*"

        if self.isReference:
            method += "&"

        #Add extra space to separate return type and name method (but not constructor/destructor)
        if len(method) > 0:
            method += ' '

        method += "%s(" % self.name
        if self.fieldString() != None:
            method += self.fieldString()
        method += ")"
        if self.isConst: method += " const"
        method += ";"
        return [method]

    def setNamespaceInclude(self):
        includes = set([])
        namespaces = set([])

        if self.returnType is not None:
            try:
                includes.add(mapping_include[self.returnType[0]])
            except KeyError:
                try:
                    includes.add(mapping_include_boost_lib[self.returnType[0]])
                except KeyError:
                    if self.returnType[0] is not None:
                        namespace, include = nsInc(self.returnType[0])
                        if include: includes.add(include)
                        if namespace: namespaces.add(namespace)

        if self.templateClassType is not None:
            tpltype = self.templateClassType
            while not isStringTypes(tpltype[1]):
                try:
                    includes.add(mapping_include[tpltype[0]])
                except KeyError:
                    try:
                        includes.add(mapping_include_boost_lib[tpltype[0]])
                    except KeyError:
                        if tpltype[0] is not None:                            
                            namespace, include = nsInc(tpltype[0])
                            if include: includes.add(include)   
                            if namespace: namespaces.add(namespace)
                tpltype = tpltype[1]


        if len(self.fields) > 0:
            for f in self.fields:
                namespaces |= getattr(f, 'namespace')
                includes |= getattr(f, 'include')

        if getattr(self, 'namespace') is None:
            setattr(self, 'namespace', namespaces)
        else:
            setattr(self, 'namespace', namespaces | getattr(self, 'namespace'))

        if getattr(self, 'include') is None:
            setattr(self, 'include', includes)
        else:
            setattr(self, 'include', includes | getattr(self, 'include'))

class CppConstructor(CppMethod):
  def __init__(self, classObj=None, returnType = None, templateClassType = None, classname = None, name = None, params = None, modifier = None, isPointer = False, isReference = False, indent = 4, document = None):
    self.classObj = classObj
    CppMethod.__init__(self,returnType , templateClassType , classname , name , params , modifier , isPointer, isReference, indent, document )

  def toImpl(self):
    # we need to wait until the very end to do this in case members are added.
    for field in self.classObj.memberFields():
        if field.name in [x.name for x in self.fields]: # If there is a parameter field that corresponds to the member field then use that value
          self.addLine("this->%s = %s;" % (field.name, field.name))
        elif field.defaultValue is None:
          if field.valueType[1] in cstdTypes or field.valueType[1] in cstdintTypes:  # If this is a "fundamental" C type then assign it to the default for that type.  For nonstandard types, we assume that the ctor properly initialized it.
            self.addLine("%s = %s();" %( field.name,str(field.valueType[0])))
        else:
          self.addLine("%s = %s;" % (field.name, str(field.defaultValue)))

    return CppMethod.toImpl(self)

class CppField():
    def __init__(self, valueType = None, templateClassType = None, name = None, objName = None, isPointer = False, isReference = False, isConst = False, document = None, defaultValue=None, isConfig=True, isInit=True):
        self.name = name
        self.valueType = None
        self.templateClassType = None

        self.namespace = set(['std'])
        self.include = set([])
        self.defaultValue = defaultValue

        self.n2 = objName

        if valueType is not None:
            self.valueType = getFullNamespace(valueType)

        if templateClassType is not None:
            self.templateClassType = templateClassType

        """ Update namespace and header for the field """
        self.setNamespaceInclude()

        self.isPointer = isPointer
        self.isReference = isReference

        ''' check reference type and back to value type '''
        ctype = ''
        try:
            ctype = self.valueType[1]
        except:
            pass

        if self.isReference == True:
            findStds = filter(lambda s: ctype.find(s) != -1, cstdTypes)
            if len(findStds) > 0 or self.isPointer or ctype.find("*") != -1:
                self.isReference = False

        self.isConst = isConst

        self.document = None
        if document is not None:
            self.document = document

        self.isConfig = isConfig
        self.isInit = isInit

    def setDocument(self, doc):
        self.document = doc

    def addNamespace(self, namespace):
        self.namespace.add(namespace)

    def addInclude(self, include):
        self.include.add(include)

    def setNamespaceInclude(self):
        includes = set([])
        namespaces = set([])
        if self.valueType is not None:
            try:
                includes.add(mapping_include[self.valueType[0]])
            except KeyError:
                try:
                    includes.add(mapping_include_boost_lib[self.valueType[0]])
                except KeyError:
                    if self.valueType[0] is not None:
                        namespace, include = nsInc(self.valueType[0])
                        if include: includes.add(include)
                        if namespace: namespaces.add(namespace)

                    # Trying lookup well-known include for C_EXISTING or C_TYPE defined
                    try:
                      tpl = self.valueType[1].split("<") # Search C++ template type
                      include = mapping_include[tpl[0]]
                      includes.add(include)
                    except:
                      pass

        if self.templateClassType is not None:
            tpltype = self.templateClassType
            while not isStringTypes(tpltype[1]):
                try:
                    includes.add(mapping_include[tpltype[0]])
                except KeyError:
                    try:
                        includes.add(mapping_include_boost_lib[tpltype[0]])
                    except KeyError:
                        if tpltype[0] is not None:
                            namespace, include = nsInc(tpltype[0])
                            if include: includes.add(include)
                            if namespace: namespaces.add(namespace)
                tpltype = tpltype[1]

        setattr(self, 'namespace', namespaces)
        setattr(self, 'include', includes)

    def toString(self, impl = False):
        if isNamespaceStd(self.valueType[1]):
            typeStr = "std::%s" % self.valueType[1]
        else:
            if self.valueType[0] is not None:
                typeStr = self.valueType[0]
            else:
                typeStr = self.valueType[1]

        if self.templateClassType is not None:
            strTemp = ''
            tpltype = getFullNamespace(self.templateClassType)

            # Count tag close
            i = 0
            while not isStringTypes(getFullNamespace(tpltype[1])):
                strTemp += '<' + tpltype[0]
                tpltype = getFullNamespace(tpltype[1])
                i += 1

            if isNamespaceStd(tpltype[1]):
                tplTypeStr = "std::%s" % tpltype[1]
            else:
                if tpltype[0] is not None:
                    tplTypeStr = tpltype[0]
                else:
                    tplTypeStr = tpltype[1]

            strTemp += '<' + tplTypeStr + '>' + ' >' * i

            #Add const to prefix of field
            #TODO: reference, pointer ...
            implStr = '%s%s' % (typeStr, strTemp)
            modifier = ''
            if self.isConst:
                implStr = "const " + implStr
            if self.isReference:
                modifier += "&"
            if self.isPointer:
                modifier += "*"

            return implStr + " " + modifier + self.name

        #Add const to prefix of field
        #TODO: reference, pointer ...
        implStr = typeStr
        if self.isConst:
            implStr = "const " + implStr

        modifier = ''
        if self.isReference:
            modifier += "&"
        if self.isPointer:
            modifier += "*"

        if not impl and self.defaultValue:
          defaultStr = "=%s" % str(self.defaultValue)
        else:
          defaultStr = ""
        return implStr + " " + modifier + self.name + defaultStr

class NotifyObjectGenerator(ObjectGenerator):
    def __init__(self, stmt, ctx):
        """Sets the attributes of the method generator, depending on stmt"""
        super(NotifyObjectGenerator, self).__init__(stmt, ctx)

    def constructors(self):
        method = ordered_set.OrderedSet()
        constructor = self.default_constructor()
        for field in self.memberFields():
            if field is not None and field.n2 is not None:
                if field.defaultValue is None:
                  constructor.addInit('%s("%s")' % (field.name, field.n2))
                else:
                  constructor.addInit('%s("%s",%s)' % (field.name, field.n2,str(field.defaultValue)))
                constructor.addLine('this->addChildObject(&%s, "%s");' % (field.name, field.n2))

        method.add(constructor)

        return method

    def destructors(self):
        method = ordered_set.OrderedSet()
        method.add(self.default_destructor())
        return method

    def memberFields(self):
        fields = ordered_set.OrderedSet()

        fields.add(CppField(valueType = ["std::map", "map"], templateClassType = [None, "std::string, SAFplus::MgtObject*"], name = "mChildObject"))

        children = search(self.stmt, leaf_stmts)
        for substmt in children:
            field = None
            key_type = search_one(substmt, 'type')

            """ Find enum type """
            if key_type.arg in ('enumeration',) and normalize(substmt.arg) not in mapping_typedef_enum.keys():
                #Put into typedef hashmap
                child_generator = ClassGenerator(key_type, path = self.rootpkg, parent = self)
                child_generator.generate()

            field = createField(substmt, self.ctx)

            if 0:
              key_override = search_one(substmt, C_TYPE)
              if key_override:
                valuetype = (key_override.arg,key_override.arg)
              else:
                valuetype = getTypes(key_type, self.ctx)

              if substmt.keyword == 'leaf':
                if key_type.arg == 'instance-identifier' and key_override:
                  field = CppField(valueType = ["SAFplus::MgtIdentifier", "MgtIdentifier"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg)
                else:
                  field = CppField(valueType = ["SAFplus::MgtProv", "MgtProv"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg)    
              else:
                if key_type.arg == 'instance-identifier' and key_override:
                  field = CppField(valueType = ["SAFplus::MgtIdentifierList", "MgtIdentifierList"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg)
                else:
                  field = CppField(valueType = ["SAFplus::MgtProvList", "MgtProvList"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg)

            description = search_one(substmt, 'description')
            if description is not None:
                field.setDocument(description.arg)

            if field: fields.add(field)

        return fields

    def methods(self):
        methods = []

        getChildMethod = CppMethod(returnType = [None, "SAFplus::MgtObject*"], templateClassType = None, classname = self.n, name = "getChildObject")
        getChildMethod.addField(CppField(valueType = ["std::string", "string"], name = "childObjectName"))

        getChildMethod.addLine("std::map<std::string, SAFplus::MgtObject* >::iterator mapIndex = mChildObject.find(childObjectName);")

        getChildMethod.addLine("/* Check if MGT module already exists in the database */")
        getChildMethod.addLine("if (mapIndex != mChildObject.end())")
        getChildMethod.addLine("{")
        getChildMethod.addLine(' ' * getChildMethod.indent + "return (*mapIndex).second;");
        getChildMethod.addLine("}")
        getChildMethod.addLine("return NULL;")

        methods.append(getChildMethod)

        addChildMethod = CppMethod(returnType = None, templateClassType = None, classname = self.n, name = "addChildObject")
        addChildMethod.addField(CppField(valueType = [None, "SAFplus::MgtObject"], name = "*childObject"))
        addChildMethod.addField(CppField(valueType = ["std::string", "string"], name = "objectName"))
        addChildMethod.addLine("mChildObject.insert(std::pair<std::string, SAFplus::MgtObject *>(objectName, childObject));")

        methods.append(addChildMethod)

        children = search(self.stmt, yang_stmts + leaf_stmts)
        """ Add getChildNames method """
        if len(children) > 0:
            getChildNamesMethod = CppMethod(classname = self.n, name = "getChildNames", returnType = ["std::vector", "vector"], templateClassType = ["std::string", "string"], isPointer = True)
            childNames = []

            for child in children:
                if child.keyword in ('container', 'list', 'leaf', 'leaf-list'):
                    childNames.append(camelize(child.arg))

            getChildNamesMethod.addLine('std::string childNames[] = { "%s" };' % '", "'.join(childNames))
            getChildNamesMethod.addLine('return new std::vector<std::string> (childNames, childNames + sizeof(childNames) / sizeof(childNames[0]));')

            methods.append(getChildNamesMethod)

            """ Add getValue method for each child """
            for child in children:
                if child.keyword in leaf_stmts:
                    child_arg = camelize(child.arg)
                    child_type = search_one(child, 'type')

                    """ Find and define enum type """
                    if child_type.arg in ('enumeration',):
                        child_generator = ClassGenerator(child_type, path = self.path, namespace = self.namespace, parent = self)
                        child_generator.generate()

                    methods += self.genAccessorMethods(child,child_arg,child_type)

        return methods

    def parentMethods(self):
        return []

    def enums(self):
        return {}

class RpcObjectGenerator(ObjectGenerator):
    def __init__(self, stmt, ctx):
        """Sets the attributes of the method generator, depending on stmt"""
        super(RpcObjectGenerator, self).__init__(stmt, ctx)

    def constructors(self):
        method = ordered_set.OrderedSet()
        constructor = self.default_constructor()

        children = search(self.stmt, ('output', 'input'))
        for substmt in children:
            for child in search(substmt, leaf_stmts):
                if substmt.keyword == 'input':
                    print 'this->mInParams.addChildObject(&%s, "%s")' % (camelize(child.arg), camelize(child.arg))
                    constructor.addLine('this->mInParams.addChildObject(&%s, "%s");' % (camelize(child.arg), camelize(child.arg)))
                else:
                    constructor.addLine('this->mOutParams.addChildObject(&%s, "%s");' % (camelize(child.arg), camelize(child.arg)))

                constructor.addInit('%s("%s")' % (camelize(child.arg), camelize(child.arg)))
            for child in search(substmt, 'container'):
                if substmt.keyword == 'input':
                    print 'this->mInParams.addChildObject(&%s, "%s")' % (camelize(child.arg), camelize(child.arg))
                    constructor.addLine('this->mInParams.addChildObject(&%s, "%s");' % (camelize(child.arg), camelize(child.arg)))
                else:
                    constructor.addLine('this->mOutParams.addChildObject(&%s, "%s");' % (camelize(child.arg), camelize(child.arg)))
        method.add(constructor)
        return method

    def destructors(self):
        method = ordered_set.OrderedSet()
        method.add(self.default_destructor())
        return method

    def memberFields(self):
        fields = ordered_set.OrderedSet()
        children = search(self.stmt, ('output', 'input'))
        for substmt in children:
            for child in search(substmt, 'container'):
                isconfig = is_config(child)
                field = CppField(valueType = cType(child,self.ctx), templateClassType = None, name = camelize(child.arg), objName = camelize(child.arg), isConfig = isconfig, isInit = False)
                if field: fields.add(field)
                print 'add field %s' % camelize(child.arg)

            for child in search(substmt, leaf_stmts):
                field = None
                key_type = search_one(child, 'type')

                """ Find enum type """
                if key_type.arg in ('enumeration',) and normalize(substmt.arg) not in mapping_typedef_enum.keys():
                    #Put into typedef hashmap
                    child_generator = ClassGenerator(key_type, path = self.rootpkg, parent = self)
                    child_generator.generate()

                field = createField(child, self.ctx)

                if 0:
                  key_override = search_one(substmt, C_TYPE)

                  if key_override:
                    valuetype = (key_override.arg,key_override.arg)
                  else:
                    valuetype = getTypes(key_type, self.ctx)
    
                  if substmt.keyword == 'leaf':
                    if key_type.arg == 'instance-identifier' and key_override:
                      field = CppField(valueType = ["SAFplus::MgtIdentifier", "MgtIdentifier"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg)
                    else:
                      field = CppField(valueType = ["SAFplus::MgtProv", "MgtProv"], templateClassType = valuetype, name = camelize(substmt.arg), objName = camelize(substmt.arg))
                  else:
                    if key_type.arg == 'instance-identifier' and key_override:
                      field = CppField(valueType = ["SAFplus::MgtIdentifierList", "MgtIdentifierList"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg)
                    else:
                      field = CppField(valueType = ["SAFplus::MgtProvList", "MgtProvList"], templateClassType = valuetype, name = camelize(substmt.arg), objName = camelize(substmt.arg))

                description = search_one(child, 'description')
                if description is not None:
                    field.setDocument(description.arg)


                if field: fields.add(field)
        return fields

    def methods(self):
        methods = []

        children = search(self.stmt, ('output', 'input'))

        """ Add getValue method for each child """
        for substmt in children:
            childNames = set([])
            for child in search(substmt, leaf_stmts):
                child_arg = camelize(child.arg)
                child_type = search_one(child, 'type')

                """ Find enum type """
                if child_type.arg in ('enumeration',) and normalize(child.arg) not in mapping_typedef_enum.keys():
                    #Put into typedef hashmap
                    child_generator = ClassGenerator(child_type, path = self.rootpkg, parent = self)
                    child_generator.generate()

                valuetype = getTypes(child_type, self.ctx)

                childNames.add(child_arg)

                #methods += self.genAccessorMethods(child,"%s%s" % (capitalizeFirst(substmt.keyword), child_arg),child_type)
                methods += self.genAccessorMethods(child,child_arg,child_type)

            """ Add getChildNames method """
            if len(childNames) > 0:
                getChildNamesMethod = CppMethod(classname = self.n, name = "get%sChildNames" % capitalizeFirst(substmt.keyword) , returnType = ["std::vector", "vector"], templateClassType = ["std::string", "string"], isPointer = True)
                getChildNamesMethod.addLine('std::string childNames[] = { "%s" };' % '", "'.join(childNames))
                getChildNamesMethod.addLine('return new std::vector<std::string> (childNames, childNames + sizeof(childNames) / sizeof(childNames[0]));')

                methods.append(getChildNamesMethod)

        """ prototype RPC class """
        validateMethod = CppMethod(classname = self.n, name = "validate", returnType = [None, 'ClRcT'])
        validateMethod.addLine("/* TODO: Add your implementation here */")
        validateMethod.addLine("return CL_OK;")
        methods.append(validateMethod)

        invokeMethod = CppMethod(classname = self.n, name = "invoke", returnType = [None, 'ClRcT'])
        invokeMethod.addLine("/* TODO: Add your implementation here */")
        invokeMethod.addLine("return CL_OK;")
        methods.append(invokeMethod)

        postReplyMethod = CppMethod(classname = self.n, name = "postReply", returnType = [None, 'ClRcT'])
        postReplyMethod.addLine("/* TODO: Add your implementation here */")
        postReplyMethod.addLine("return CL_OK;")
        methods.append(postReplyMethod)

        return methods

    def parentMethods(self):
        return []

    def enums(self):
        return {}

class ContainerObjectGenerator(ObjectGenerator):
    def __init__(self, stmt, ctx):
        """Sets the attributes of the method generator, depending on stmt"""
        super(ContainerObjectGenerator, self).__init__(stmt, ctx)
        self.stmt = stmt
        self.is_config = is_config(stmt)

    def constructors(self):
        method = ordered_set.OrderedSet()
        constructor = self.default_constructor()

        for field in self.memberFields():
            if field is not None and field.n2 is not None:
                if field.isInit:
                    if field.defaultValue is None:
                      constructor.addInit('%s("%s")' % (field.name, field.n2))
                    else:
                      constructor.addInit('%s("%s",%s)' % (field.name, field.n2,str(field.defaultValue)))
 
                constructor.addLine('this->addChildObject(&%s, "%s");' % (field.name, field.n2))
                if not field.isConfig:
                    constructor.addLine('%s.config = false;' % (field.name))

        # Add setKeyList for list substmt
        liststmts =  search(self.stmt,'list')
        for liststmt in liststmts:
            list_name = liststmt.arg + "List"
            xpath = fullPath(liststmt)
            constructor.addLine('%s.childXpath="/%s";'%(lowerFirst(list_name),xpath))
            for lsub in liststmt.substmts:
                if lsub.keyword == "key":
                    keys = lsub.arg.split(' ')
                    for k in keys:
                        constructor.addLine('%s.setListKey("%s");'%(lowerFirst(list_name),k))
        method.add(constructor)
        return method

    def destructors(self):
        method = ordered_set.OrderedSet()
        method.add(self.default_destructor())
        return method

    def memberFields(self):
        fields = ordered_set.OrderedSet()
        children = search(self.stmt, leaf_stmts)
        for substmt in children:
            #Using derived from uses (but take care with keyword refine)
            if hasattr(substmt, 'i_uses') == False:
                field = None
                key_type = search_one(substmt, 'type')
                #key_override = search_one(substmt, C_TYPE)
                #isconfig = is_config(substmt)

                """ Find enum type """
                if key_type.arg in ('enumeration',) and normalize(substmt.arg) not in mapping_typedef_enum.keys():
                    #Put into typedef hashmap
                    child_generator = ClassGenerator(key_type, path = self.rootpkg, parent = self)
                    child_generator.generate()

                field = createField(substmt, self.ctx)
                #if key_override:
                #    valuetype = (key_override.arg,key_override.arg)
                #else:
                #    valuetype = getTypes(key_type, self.ctx)
    
                #if substmt.keyword == 'leaf':
                #    if key_type.arg == 'instance-identifier' and key_override:
                #      field = CppField(valueType = ["SAFplus::MgtIdentifier", "MgtIdentifier"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg, isConfig = isconfig)
                #    else:
                #      field = CppField(valueType = ["SAFplus::MgtProv", "MgtProv"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg, isConfig = isconfig)
                #else:
                #    if key_type.arg == 'instance-identifier' and key_override:
                #      field = CppField(valueType = ["SAFplus::MgtIdentifierList", "MgtIdentifierList"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg, isConfig = isconfig)
                #    else:
                #      field = CppField(valueType = ["SAFplus::MgtProvList", "MgtProvList"], templateClassType = valuetype, name = substmt.arg, objName = substmt.arg, isConfig = isconfig)
    
                description = search_one(substmt, 'description')
                if description is not None and field:
                    field.setDocument(description.arg)

                if field: fields.add(field)

        children = search(self.stmt, ['container'])
        for substmt in children:
            isconfig = is_config(substmt)
            field = CppField(valueType = cType(substmt,self.ctx), templateClassType = None, name = camelize(substmt.arg), objName = camelize(substmt.arg), isConfig = isconfig, isInit = False)
            if field: fields.add(field)
        children = search(self.stmt, ['rpc'])

        for substmt in children:
            isconfig = is_config(substmt)
            field = CppField(valueType = cType(substmt,self.ctx), templateClassType = None, name = camelize(substmt.arg), objName = camelize(substmt.arg), isConfig = isconfig, isInit = False)
            if field: fields.add(field)


        # Add yang list type field
        fields.update(self.addListFields())

        return fields

    def methods(self):
        methods = []

        """ TODO: Add toString method => pure method at MgtObject """
        """
        toStringMethod = CppMethod(classname = self.n, name = "toString", returnType = [None, "void"])
        toStringMethod.addLine('/* TODO:  */')

        field = CppField(valueType = [None, "std::stringstream"], name = "xmlString", isReference=True)
        toStringMethod.addField(field)
        methods.append(toStringMethod)
        """

        """ TODO 
        removeAllChildrenMethod = CppMethod(classname = self.n, name = "removeAllChildren", returnType = [None, "void"])
        removeAllChildrenMethod.addLine('/* TODO:  */')
        methods.append(removeAllChildrenMethod)
        """

        """ Add getChildNames method """
        children = search(self.stmt, yang_stmts + leaf_stmts)
        if len(children) > 0:
            getChildNamesMethod = CppMethod(classname = self.n, name = "getChildNames", returnType = ["std::vector", "vector"], templateClassType = ["std::string", "string"], isPointer = True)
            childNames = []

            for child in children:
                childNames.append(camelize(child.arg))

            getChildNamesMethod.addLine('std::string childNames[] = { "%s" };' % '", "'.join(childNames))
            getChildNamesMethod.addLine('return new std::vector<std::string> (childNames, childNames + sizeof(childNames) / sizeof(childNames[0]));')

            methods.append(getChildNamesMethod)


            """ Add getValue method for each child """
            for child in children:
                if child.keyword in leaf_stmts and hasattr(child, 'i_uses') == False:
                    #Ignore child from derived class (but take care with keyword refine)
                    # child_arg = capitalizeFirst(camelize(child.arg))
                    #child_type = search_one(child, 'type')
                    #child_override = search_one(child, C_TYPE)
                    #if child_override:
                    #    valuetype = (child_override.arg,child_override.arg)
                    #else:
                    #    valuetype = getTypes(child_type, self.ctx)
                    # theType = leafType(child,self.ctx)
                    #methods += self.genAccessorMethods(child,child_arg,theType)
                    methods += self.genAccessorMethods(child,None,None)

            """ TODO: """
            if self.is_top_level:
                """
                loadDBMethod = CppMethod(classname = self.n, name = "load", returnType = [None, 'void'])
                field = CppField(valueType = [None, 'SAFplus::MgtDatabase'], name = "db", isPointer=True)
                loadDBMethod.addField(field)
                loadDBMethod.addLine('/* TODO: */')
                methods.append(loadDBMethod)
                """

        return methods

    def parentMethods(self):
        methods = []

        xpath = "XPATH: /" + fullPath(self.stmt)

        parent = getParent(self.stmt)

        rpc = parent is not None and parent.keyword in ('input', 'output',)

        if rpc is True:
            """ add method """
            addMethod1 = CppMethod(classname = self.n, name = "add%s" %self.n)
            field = CppField(valueType = [None, self.n], name = "*" + parent.keyword + self.n + "Value")
            if parent.keyword == 'input':
                addMethod1.addLine("this->mInParams.addChildObject(%sValue, \"%s\");" % (parent.keyword + self.n , self.n2))
            else:
                addMethod1.addLine("this->mOutParams.addChildObject(%sValue, \"%s\");" % (parent.keyword + self.n , self.n2))

            addMethod1.addField(field)
            methods.append(addMethod1)

            """ get method """
            getMethod = CppMethod(classname = self.n, name = "get%s" %self.n, returnType = [None, self.n + "*"])

            """ Implemenetation for getMethod """
            if parent.keyword == 'input':
                getMethod.addLine("return (%s*)this->mInParams.getChildObject(\"%s\");" % (self.n , self.n2))
            else:
                getMethod.addLine("return (%s*)this->mOutParams.getChildObject(\"%s\");" % (self.n , self.n2))

            getMethod.addInclude("%s" % self.n)

            """ Add document for addMethod """
            addMethod1.setDocument(xpath)
            getMethod.setDocument(xpath)

            methods.append(getMethod)

        else:
            if self.nativeType:
              theType = self.nativeType.arg
            else:
              theType = self.n

            """ add method """
            addMethod1 = CppMethod(classname = theType, name = "add%s" % self.n)
            field = CppField(valueType = [None, theType], name = "*" + self.n2 + "Value")
            addMethod1.addLine("this->addChildObject(%sValue, \"%s\");" % (self.n2 , self.n2))
            addMethod1.addField(field)

            """ get method """
            
            getMethod = CppMethod(classname = theType, name = "get%s" % self.n, returnType = [None, theType + "*"])

            """ Implementation for getMethod """
            getMethod.addLine("return dynamic_cast<%s*>(this->getChildObject(\"%s\"));" % (theType, self.n2))

            if not self.nativeType:
              getMethod.addInclude("%s" % self.n)

            """ Add document for addMethod """
            addMethod1.setDocument(xpath)
            getMethod.setDocument(xpath)

            methods.append(getMethod)
            methods.append(addMethod1)

        return methods

    def enums(self):
        return {}

class ListObjectGenerator(ObjectGenerator):
    def __init__(self, stmt, ctx):
        """Sets the attributes of the method generator, depending on stmt"""
        super(ListObjectGenerator, self).__init__(stmt, ctx)
        assert self.gen is self
        assert self.is_list, 'Only valid for list stmts'
        self.is_config = is_config(stmt)
        self.stmt = stmt
        self.keys = []
        if True:
            key = search_one(self.stmt, 'key')
            try:
                self.keys = key.arg.split(' ')
            except AttributeError:
                self.is_config = False  # is_config produced wrong value

        findkey = lambda k: search_one(self.stmt, 'leaf', arg = k)
        self.key_stmts = [findkey(k) for k in self.keys]
        self.keyClass = None

    def buildCtorContents(self,ctor):
        if not self.is_config:
            ctor.addLine('this->config = false;');

        """ Add extra init for default constructor """
        for field in self.memberFields():
            if field is not None and field.n2 is not None:
                if field.isInit:
                    if field.defaultValue is None:
                      ctor.addInit('%s("%s")' % (field.name, field.n2))
                    else:
                      ctor.addInit('%s("%s",%s)' % (field.name, field.n2,str(field.defaultValue)))
                """ Add object into tree """
                ctor.addLine('this->addChildObject(&%s, "%s");' % (field.name, field.n2))
                if not field.isConfig:
                    ctor.addLine('%s.config = false;' % (field.name))

        # Add setKeyList for list substmt
        liststmts =  search(self.stmt,'list')
        for liststmt in liststmts:
            list_name = liststmt.arg + "List"
            xpath = fullPath(liststmt)
            ctor.addLine('%s.childXpath="/%s";'%(lowerFirst(list_name),xpath))
            for lsub in liststmt.substmts:
                if lsub.keyword == "key":
                    keys = lsub.arg.split(' ')
                    for k in keys:
                        ctor.addLine('%s.setListKey("%s");'%(lowerFirst(list_name),k))


    def constructors(self):
        method = ordered_set.OrderedSet()

        """ Default constructor """
        defaultConstructor = self.default_constructor()
        self.buildCtorContents(defaultConstructor)

        method.add(defaultConstructor)


        alreadyHasNameCtor = False
        """ Key list for create multikey support """
        key_list = []
        if len(self.key_stmts) > 0:
            """ Constructor with key param init """
            constructor = CppMethod(classname = self.n, name = self.n)
            for key in self.key_stmts:
                key_arg = camelize(key.arg)
                key_type = search_one(key, 'type')
                
                valuetype = getTypes(key_type, self.ctx)
                key_list.append([key_arg,valuetype])
                if (not valuetype[1] in cstdintTypes and not valuetype[1] in cstdTypes) or  valuetype[1]=="string":
                  cvt = ( "const " + valuetype[0] + "&", "const " + valuetype[1] + "&")
                else: cvt = valuetype
                field = CppField(valueType = cvt, name = key_arg + "Value", isReference=True)
                constructor.addField(field)
                constructor.addLine("this->%s.value =  %s;" % (key_arg, key_arg + "Value"))

                """ Init keys mapping for clMgtObject """
                """ TODO: 
                defaultConstructor.addLine('this->addKey("%s");' % key_arg)
                constructor.addLine('this->addKey("%s");' % key_arg)
                """

            if not self.is_config:
                constructor.addLine('this->config = false;');

            """ Add extra init for constructor """
            for field in self.memberFields():
                if field is not None and field.n2 is not None:
                    if field.isInit:
                      if field.defaultValue is None:
                        constructor.addInit('%s("%s")' % (field.name, field.n2))
                      else:
                        constructor.addInit('%s("%s",%s)' % (field.name, field.n2,str(field.defaultValue)))

                    """ Add object into tree """
                    constructor.addLine('this->addChildObject(&%s, "%s");' % (field.name, field.n2))
                    if not field.isConfig:
                        constructor.addLine('%s.config = false;' % (field.name))

            # Add setKeyList for list substmt
            liststmts =  search(self.stmt,'list')
            for liststmt in liststmts:
                list_name = liststmt.arg + "List"
                xpath = fullPath(liststmt)
                constructor.addLine('%s.childXpath="/%s";'%(lowerFirst(list_name),xpath))
                for lsub in liststmt.substmts:
                    if lsub.keyword == "key":
                        keys = lsub.arg.split(' ')
                        for k in keys:
                            constructor.addLine('%s.setListKey("%s");'%(lowerFirst(list_name),k))

            if len(constructor.fields) == 1 and "string" in constructor.fields[0].valueType[0]:
              alreadyHasNameCtor = True
            method.add(constructor)
            """ Generate multiple key class support """
            if len(self.key_stmts) > 1 or key_list[0][1][1] != 'string':
                multikey_gen = MultiKeyClassGenerator(key_list, self.n, path=self.rootpkg, module=getModule(self.stmt), isConfig=self.is_config)
                multikey_gen.generate()

                ''' TODO: Auto gen KeyType '''
                self.keyClass = multikey_gen.class_name

        if not alreadyHasNameCtor:
          # Add a constructor that takes the object's name
          name_constructor = CppConstructor(classObj=self, classname = self.n, name = self.n)
          name_constructor.addField(CppField(valueType = ("const std::string&", "const string&"), name = "nameParam", isReference=True))
          name_constructor.addLine("this->tag = nameParam;")
          self.buildCtorContents(name_constructor)
          method.add(name_constructor)

        return method

    def destructors(self):
        method = ordered_set.OrderedSet()
        method.add(self.default_destructor())
        return method

    def memberFields(self):
        fields = ordered_set.OrderedSet()
        children = search(self.stmt, leaf_stmts)
        for substmt in children:
            field = None
            #Using derived from uses (but take care with keyword refine)
            if hasattr(substmt, 'i_uses') == False:
                key_type = search_one(substmt, 'type')    
                """ Find enum type """
                if key_type.arg in ('enumeration',) and normalize(substmt.arg) not in mapping_typedef_enum.keys():
                    #Put into typedef hashmap
                    child_generator = ClassGenerator(key_type, path = self.rootpkg, parent = self)
                    child_generator.generate()

                field = createField(substmt,self.ctx)

                description = search_one(substmt, 'description')
                if description is not None and field:
                    field.setDocument(description.arg)
    
                if field: fields.add(field)

        children = search(self.stmt, ['container'])
        for substmt in children:
            typ = cType(substmt, self.ctx)
            isconfig = is_config(substmt)
            namespace = getNamespace(substmt,self.ctx)
            field = CppField(valueType = typ, templateClassType = None, name = camelize(substmt.arg), objName = camelize(substmt.arg), isConfig = isconfig, isInit = False)
            if field: fields.add(field)

        fields.update(self.addListFields())

        return fields

    def methods(self):
        methods = []

        children = search(self.stmt, yang_stmts + leaf_stmts)

        """ Add toString method => pure method at MgtObject """
        """
        toStringMethod = CppMethod(classname = self.n, name = "toString", returnType = [None, "void"])
        toStringMethod.addLine('/* TODO:  */')

        field = CppField(valueType = [None, "std::stringstream"], name = "xmlString", isReference=True)
        toStringMethod.addField(field)
        methods.append(toStringMethod)
        """

        """ TODO 
        removeAllChildrenMethod = CppMethod(classname = self.n, name = "removeAllChildren", returnType = [None, "void"])
        removeAllChildrenMethod.addLine('/* TODO:  */')
        methods.append(removeAllChildrenMethod)
        """

        """ Add getKeys method """
        if len(self.key_stmts) > 0:
            keyNames = []
            for key in self.key_stmts:
                keyNames.append(key.arg)

            getKeysMethod = CppMethod(classname = self.n, name = "getKeys", returnType = ["std::vector", "vector"], templateClassType = ["std::string", "string"])
            getKeysMethod.addLine('std::string keyNames[] = { "%s" };' % '", "'.join(keyNames))
            getKeysMethod.addLine('return std::vector<std::string> (keyNames, keyNames + sizeof(keyNames) / sizeof(keyNames[0]));')
            methods.append(getKeysMethod)

        """ Add getChildNames method """
        if len(children) > 0:
            getChildNamesMethod = CppMethod(classname = self.n, name = "getChildNames", returnType = ["std::vector", "vector"], templateClassType = ["std::string", "string"], isPointer = True)
            childNames = []

            for child in children:
                if child.keyword in ('container', 'list', 'leaf', 'leaf-list'):
                    childNames.append(camelize(child.arg))

            getChildNamesMethod.addLine('std::string childNames[] = { "%s" };' % '", "'.join(childNames))
            getChildNamesMethod.addLine('return new std::vector<std::string> (childNames, childNames + sizeof(childNames) / sizeof(childNames[0]));')

            methods.append(getChildNamesMethod)

            """ Add getValue method for each child """
            for child in children:
                if child.keyword in leaf_stmts:
                    child_arg = capitalizeFirst(camelize(child.arg))
                    child_type = search_one(child, 'type')
                    child_override = search_one(child, C_TYPE)
                    if child_override:
                        valuetype = (child_override.arg,child_override.arg)
                    else:
                        valuetype = getTypes(child_type, self.ctx)

                    xpath = "XPATH: /" + fullPath(child)

                    if child.keyword == 'leaf':
                        #Using derived from uses (but take care with keyword refine)
                        if hasattr(child, 'i_uses') == False:
                            """ getValue method """
                            getValueMethod = CppMethod(classname = self.n, name = "get%s" % child_arg, returnType = valuetype)
                            getValueMethod.addLine("return this->%s.value;" % camelize(child.arg))
    
                            """ Add document for method """
                            getValueMethod.setDocument(xpath)
                            methods.append(getValueMethod)
    
                            """ setValue method """
                            setValueMethod = CppMethod(classname = self.n, name = "set%s" % child_arg)
                            field = CppField(valueType = valuetype, name = camelize(child.arg) + "Value", isReference=True)
                            txnField = CppField(valueType = ["SAFplus::Transaction","Transaction"], name = "t",isReference=True,defaultValue="SAFplus::NO_TXN")
                            setValueMethod.addField(field)
                            setValueMethod.addField(txnField)
                            """ Header for Transaction """
                            setValueMethod.addInclude("clTransaction")
                            """ Implement for methodSet which supports Transaction """
                            setValueMethod.addLine("if(&t == &SAFplus::NO_TXN) this->%s.value = %sValue;" % (camelize(child.arg), camelize(child.arg)))
                            setValueMethod.addLine("else")
                            setValueMethod.addLine("{")
                            if field.valueType[0] == None:
                                setValueMethod.addLine(" " * setValueMethod.indent + "SAFplus::SimpleTxnOperation<%s> *opt = new SAFplus::SimpleTxnOperation<%s>(&(%s.value),%sValue);" % (field.valueType[1],field.valueType[1],camelize(child.arg), camelize(child.arg)))
                            else:
                                setValueMethod.addLine(" " * setValueMethod.indent + "SAFplus::SimpleTxnOperation<%s> *opt = new SAFplus::SimpleTxnOperation<%s>(&(%s.value),%sValue);" % (field.valueType[0],field.valueType[0],camelize(child.arg), camelize(child.arg)))
                            setValueMethod.addLine(" " * setValueMethod.indent + "t.addOperation(opt);")
                            setValueMethod.addLine("}")                        
    
                            """ Add document for method """
                            setValueMethod.setDocument(xpath)
                            methods.append(setValueMethod)

                    else:
                        """ getValue method """
                        getValueMethod = CppMethod(classname = self.n, name = "get%s" % child_arg, returnType = ["std::vector", "vector"], templateClassType = valuetype)
                        getValueMethod.addLine("return this->%s.value;" % camelize(child.arg))

                        """ Add document for method """
                        getValueMethod.setDocument(xpath)
                        methods.append(getValueMethod)

                        """ setValue method : MgtProvList - list-leaf """
                        setValueMethod = CppMethod(classname = self.n, name = "set%s" % child_arg)
                        field = CppField(valueType = valuetype, name = camelize(child.arg) + "Value", isReference=True)
                        setValueMethod.addField(field)
                        setValueMethod.addLine("this->%s.value.push_back(%sValue);" % (camelize(child.arg), camelize(child.arg)))

                        """ Add document for method """
                        setValueMethod.setDocument(xpath)
                        methods.append(setValueMethod)

        return methods

    def parentMethods(self):
        methods = []

        parent = getParent(self.stmt)
        rpc = parent is not None and parent.keyword in ('input', 'output',)

        xpath = "XPATH: /" + fullPath(self.stmt)

        #Namely field of list
        memberListName = lowerFirst(self.stmt.arg + 'List')

        if rpc is True:
            """ rpc specific """
            """ add method """
            addMethod1 = CppMethod(classname = self.n, name = "add%s%s" % (capitalizeFirst(parent.keyword), self.n))
            field = CppField(valueType = [None, self.n], name = "*" + parent.keyword + self.n + "Value")
            if parent.keyword == 'input':
                addMethod1.addLine("this->mInParams.addChildObject(%sValue, \"%s\");" % (parent.keyword + self.n , self.n2))
            else:
                addMethod1.addLine("this->mOutParams.addChildObject(%sValue, \"%s\");" % (parent.keyword + self.n , self.n2))

            addMethod1.addField(field)

            """ addMethod2 """
            addMethod2 = CppMethod(classname = self.n, name = "add%s%s" % (capitalizeFirst(parent.keyword), self.n))

            """ Implementation for addMethod2 """
            addMethod2.addLine("%s *%s = new %s(%s);" % (self.n, self.n2, self.n, ", ".join([field.name for field in addMethod2.fields])))
            addMethod2.addLine("this->mOutParams.addChildObject(%s, \"%s\");" % (self.n2 , self.n2))
            addMethod2.addNamespace("std")
            addMethod2.addInclude("vector")
            addMethod2.addInclude("map")
            addMethod2.addInclude("string")
            addMethod2.addInclude("%s" % self.n)

            """ Add document for addMethod """
            addMethod1.setDocument(xpath)
            addMethod2.setDocument(xpath)

            methods.append(addMethod1)
            methods.append(addMethod2)
            pass
        else:
            pass
            ''' TODO: we dont know how to use key in LIST, so just keep it
            """ add method """
            addMethod1 = CppMethod(classname = self.n, name = "add%s" % self.n)
            field = CppField(valueType = [None, self.n], name = "*" + self.n2 + "Value")
            addMethod1.addLine("this->%s.addChildObject(%sValue);" %(memberListName , self.n2))
            addMethod1.addField(field)

            """ get method """
            getMethod = CppMethod(classname = self.n, name = "get%s" % self.n, returnType = [None, self.n + "*"])

            objectChildCompare = []
            for key in self.key_stmts:
                key_arg = capitalizeFirst(camelize(key.arg))
                key_type = search_one(key, 'type')
                valuetype = getTypes(key_type, self.ctx)
                field = CppField(valueType = valuetype, name = key.arg + "Value")
                getMethod.addField(field)
                objectChildCompare.append("childObject->get%s() == %sValue" % (key_arg, key.arg))

            assert(objectChildCompare>0)

            """ Implemenetation for getMethod """
            getMethod.addLine("for (unsigned int i = 0; i < this->%s.getEntrySize(); i++)" %memberListName)
            getMethod.addLine("{")
            getMethod.addLine(' ' * getMethod.indent + "%s* childObject = (%s*)this->%s.getEntry(i);" %(self.n, self.n, memberListName))

            getMethod.addLine(' ' * getMethod.indent + "if (" + " and ".join(objectChildCompare) + ")")
            getMethod.addLine(' ' * getMethod.indent + "{")
            getMethod.addLine(' ' * 2 * getMethod.indent + "return childObject;")
            getMethod.addLine(' ' * getMethod.indent + "}")
            getMethod.addLine("}")
            getMethod.addLine("return NULL;")

            """ Add document for addMethod """
            addMethod1.setDocument(xpath)
            getMethod.setDocument(xpath)

            methods.append(getMethod)
            methods.append(addMethod1)

            '''

        return methods

    def enums(self):
        return {}

class LeafObjectGenerator(ObjectGenerator):
    def __init__(self, stmt, ctx):
        """Sets the attributes of the method generator, depending on stmt"""
        super(LeafObjectGenerator, self).__init__(stmt, ctx)

        self.type = search_one(stmt, 'type')
        self.namespace, self.primitive = getTypes(stmt, ctx)
        self.base_type = getBaseType(stmt) if self.type else None
        self.base_primitive = None
        if self.base_type:
            self.base_primitive = getTypes(self.base_type, ctx)[1]

    def methods(self):
        return []

    def memberFields(self):
      return ordered_set.OrderedSet()

    def parentMethods(self):
      methods = ordered_set.OrderedSet()
      return methods

    def enums(self):
      return {}

class TypedefObjectGenerator(ObjectGenerator):
    def __init__(self, stmt, ctx):
        """Sets the attributes of the method generator, depending on stmt"""
        super(TypedefObjectGenerator, self).__init__(stmt, ctx)
        assert self.gen is self
        assert self.is_typedef, 'This class is only valid for typedef stmts'

        self.is_num = False
        self.type = search_one(stmt, 'type')

        #C++ typedef enum
        if self.type and self.type.arg in ('enumeration',):
            self.is_num = True
            self.enum_type = normalize(self.stmt.arg)
            self.enum_manager = "%sManager" %self.enum_type
            self.enum_param = lowerFirst(self.enum_type)

            #Put into typedef hashmap
            mapping_typedef_enum[self.enum_type] = "::" + self.rootpkg + "::" + self.enum_type
            mapping_typedef_enum[self.enum_manager] = "::%s::%s" %(self.rootpkg, self.enum_manager)

        self.namespace, self.primitive = getTypes(stmt, ctx)
        self.base_type = getBaseType(stmt) if self.type else None
        self.base_primitive = None
        if self.base_type:
            self.base_primitive = getTypes(self.base_type, ctx)

    def constructors(self):
        method = ordered_set.OrderedSet()
        if self.is_num == False:
            method.add(self.default_constructor())
        fields = self.memberFields()

        if len(fields) == 1:  # Create a constructor that just takes one of these if there is one item in this typedef.  This helps yang default values
          for field in fields:
            method.add(CppConstructor(classObj=self, classname = self.n, name = self.n, params=[field]))
        return method

    def destructors(self):
        method = ordered_set.OrderedSet()
        if self.is_num == False:
            method.add(self.default_destructor())
        return method

    def methods(self):
        if self.is_num == False:
            methodGet = CppMethod(returnType = self.base_primitive, classname = self.n, name = "getValue")
            methodGet.addLine("return this->value;")
    
            methodSet = CppMethod(classname = self.n, name = "setValue")
            field = CppField(valueType = self.base_primitive, name = "value", isReference=True)
            txnField = CppField(valueType = ["SAFplus::Transaction","Transaction"], name = "t",isReference=True,defaultValue="SAFplus::NO_TXN")
            methodSet.addField(field)
            methodSet.addField(txnField)
            """ Include clTransaction.hxx """
            txnField.addInclude("clTransaction")
            """ Implement for methodSet which supports Transaction """
            methodSet.addLine("if(&t == &SAFplus::NO_TXN) this->value = value;")
            methodSet.addLine("else")
            methodSet.addLine("{")
            if field.valueType[0] == None:
                methodSet.addLine(" " * methodSet.indent + "SAFplus::SimpleTxnOperation<%s> *opt = new SAFplus::SimpleTxnOperation<%s>(&this->value,value);" % (field.valueType[1],field.valueType[1]))
            else:
                methodSet.addLine(" " * methodSet.indent + "SAFplus::SimpleTxnOperation<%s> *opt = new SAFplus::SimpleTxnOperation<%s>(&this->value,value);" % (field.valueType[0],field.valueType[0]))
            methodSet.addLine(" " * methodSet.indent + "t.addOperation(opt);")
            methodSet.addLine("}")
    
            """ operator= """
            operator1 = CppMethod(returnType = [None, self.n], classname = self.n, name = "operator=", isReference=True)
            field = CppField(valueType = [None, self.n], name = self.n2, isReference=True, isConst=True)
            operator1.addField(field)
            operator1.addLine("value = %s.value;" % self.n2)
            operator1.addLine("return *this;")
    
            """ ostream """
            operator2 = CppMethod(returnType = [None, "ostream"], classname = None, name = "operator<<", modifier = "friend", isReference=True)
            operator2.addField(CppField(valueType = [None, "ostream"], name = "os", isReference=True))
            operator2.addField(CppField(valueType = [None, self.n], name = self.n2, isConst=True, isReference=True))
            operator2.addLine("return os << %s.value;" % self.n2)
            operator2.addInclude("iostream")
    
            """ istream """
            operator3 = CppMethod(returnType = [None, "istream"], classname = None, name = "operator>>", modifier = "friend", isReference=True)
            operator3.addField(CppField(valueType = [None, "istream"], name = "is", isReference=True))
            operator3.addField(CppField(valueType = [None, self.n], name = self.n2, isReference=True))
            operator3.addLine("return is >> %s.value;" % self.n2)

            # operator==
            operator4 = CppMethod(returnType = [None, "bool"], classname = self.n, name = "operator==", isReference=False, isConst=True)
            field = CppField(valueType = [None, self.n], name = self.n2, isReference=True, isConst=True)
            operator4.addField(field)
            operator4.addLine("return (value == %s.value);" % self.n2)
    
            return [methodGet, methodSet, operator1, operator4, operator2, operator3]
        else:
            """ c_str """
            operator1 = CppMethod(returnType=[None, "const char"], name="c_str", modifier="friend", isPointer=True);
            operator1.addField(CppField(valueType=[None, self.enum_type], name=self.enum_param, isConst=True, isReference=True))
            operator1.addLine("return %s::c_str(%s);" % (self.enum_manager, self.enum_param))

            """ ostream """
            operator2 = CppMethod(returnType=[None, "ostream"], classname=None, name="operator<<", modifier="friend", isReference=True)
            operator2.addField(CppField(valueType=[None, "ostream"], name="os", isReference=True))
            operator2.addField(CppField(valueType=[None, self.enum_type], name=self.enum_param, isConst=True, isReference=True))
            operator2.addLine("return os << %s::toString(%s);" % (self.enum_manager, self.enum_param))
    
            """ istream """
            operator3 = CppMethod(returnType = [None, "istream"], classname = None, name = "operator>>", modifier = "friend", isReference=True)
            operator3.addField(CppField(valueType = [None, "istream"], name = "is", isReference=True))
            operator3.addField(CppField(valueType = [None, self.enum_type], name = self.enum_param, isReference=True))
            operator3.addLine("std::string buf;")
            operator3.addLine("is >> buf;");
            operator3.addLine("%s = %s::toEnum(buf);" %(self.enum_param, self.enum_manager))
            operator3.addLine("return is;")
    
            operator2.addInclude("MgtEnumType")
            operator2.addInclude("iostream")

            return [operator1, operator2, operator3]

    def memberFields(self):
        fields = ordered_set.OrderedSet()
        if self.is_num == False:
            field = CppField(valueType = self.base_primitive, name = "value")
            fields.add(field)
        return fields

    def parentMethods(self):
        return []

    def enums(self):
        if self.is_num:
            return {self.enum_type: map(enumize, search(self.type, 'enum'))}
        return {}

class EnumObjectGenerator(ObjectGenerator):
    def __init__(self, stmt, ctx):
        """Sets the attributes of the method generator, depending on stmt"""
        super(EnumObjectGenerator, self).__init__(stmt, ctx)
        assert self.gen is self

        self.type = stmt

        #Statement with leaf name as enumeration 
        self.stmt = getParent(stmt)

        self.enum_type = normalize(self.stmt.arg)
        self.enum_manager = "%sManager" %normalize(self.stmt.arg)
        self.enum_param = lowerFirst(self.enum_type)

        #Put into typedef hashmap
        mapping_typedef_enum[self.enum_type] = "::" + self.rootpkg + "::" + self.enum_type
        mapping_typedef_enum[self.enum_manager] = "::%s::%s" %(self.rootpkg, self.enum_manager)

    def constructors(self):
        method = ordered_set.OrderedSet()
        return method

    def destructors(self):
        method = ordered_set.OrderedSet()
        return method

    def methods(self):
        """ c_str """
        operator1 = CppMethod(returnType=[None, "const char"], name="c_str", modifier="friend", isPointer=True);
        operator1.addField(CppField(valueType=[None, self.enum_type], name=self.enum_param, isConst=True, isReference=True))
        operator1.addLine("return %s::c_str(%s);" % (self.enum_manager, self.enum_param))

        """ ostream """
        operator2 = CppMethod(returnType=[None, "ostream"], classname=None, name="operator<<", modifier="friend", isReference=True)
        operator2.addField(CppField(valueType=[None, "ostream"], name="os", isReference=True))
        operator2.addField(CppField(valueType=[None, self.enum_type], name=self.enum_param, isConst=True, isReference=True))
        operator2.addLine("return os << %s::toString(%s);" % (self.enum_manager, self.enum_param))

        """ istream """
        operator3 = CppMethod(returnType = [None, "istream"], classname = None, name = "operator>>", modifier = "friend", isReference=True)
        operator3.addField(CppField(valueType = [None, "istream"], name = "is", isReference=True))
        operator3.addField(CppField(valueType = [None, self.enum_type], name = self.enum_param, isReference=True))
        operator3.addLine("std::string buf;")
        operator3.addLine("is >> buf;");
        operator3.addLine("%s = %s::toEnum(buf);" %(self.enum_param, self.enum_manager))
        operator3.addLine("return is;")

        operator2.addInclude("MgtEnumType")
        operator2.addInclude("iostream")

        return [operator1, operator2, operator3]

    def memberFields(self):
        fields = ordered_set.OrderedSet()
        return fields

    def parentMethods(self):
        return []

    def enums(self):
        return {self.enum_type: map(enumize, search(self.type, 'enum'))}


def createField(substmt,ctx):
  """Create a field object from a substatement"""
  isconfig = is_config(substmt)
  nativeType = search_one(substmt, C_EXISTING)
  # pdb.set_trace()
  if nativeType:
    theType = (None,nativeType.arg)
    templateType = None
  else:
    key_type = search_one(substmt, 'type')
    key_override = search_one(substmt, C_TYPE)
    if key_override:
      templateType = (key_override.arg,key_override.arg)
    else:
      templateType = getTypes(key_type, ctx)

    if substmt.keyword == 'leaf':
      if key_type.arg == 'instance-identifier' and key_override:
        theType = ["SAFplus::MgtIdentifier", "MgtIdentifier"]
      else:
        theType = ["SAFplus::MgtProv", "MgtProv"]
    else:
      if key_type.arg == 'instance-identifier' and key_override:
        theType = ["SAFplus::MgtIdentifierList", "MgtIdentifierList"]
      else:
        theType = ["SAFplus::MgtProvList", "MgtProvList"]

  # Figure out what to put as the default value based on the passed value and the type of the object
  defaultVal = None
  if hasattr(substmt,"i_default_str") and substmt.i_default_str: 
    t = substmt.i_default_str
    if isNumber(t): 
      if templateType[1] in cstdTypes or templateType[1] in cstdintTypes:
        defaultVal = t
      else: # If its a complex type, construct the type with the default arg as the parameter  TODO generate constructors for typedefs
        defaultVal = templateType[1] + "(" + t + ")"

    elif t=="true": defaultVal = "true"
    elif t=="false": defaultVal = "false"
    else: 
      if "string" in templateType[1]:
        defaultVal = 'std::string("' + t + '")'
      else:  # Its an enum type
        defaultVal = templateType[1] + "::" + t
 
  field = CppField(valueType = theType, templateClassType = templateType, name = camelize(substmt.arg), objName = camelize(substmt.arg), isConfig = isconfig, defaultValue=defaultVal)
  return field
