###############################################################################
#
# Copyright (C) 2002-2009 by OpenClovis Inc. All  Rights Reserved.
# 
# The source code for  this program is not published  or otherwise 
# divested of  its trade secrets, irrespective  of  what  has been 
# deposited with the U.S. Copyright office.
# 
# This program is  free software; you can redistribute it and / or
# modify  it under  the  terms  of  the GNU General Public License
# version 2 as published by the Free Software Foundation.
# 
# This program is distributed in the  hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied  warranty  of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.
# 
# You  should  have  received  a  copy of  the  GNU General Public
# License along  with  this program. If  not,  write  to  the 
# Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#
###############################################################################

import pdb
import sys, datetime
import getpass

from mgtcommon import *
from ncclient import manager
from ncclient.transport import SessionListener
from ncclient.operations import RPC
from ncclient.operations.errors import TimeoutExpiredError
from ncclient.transport.errors import TransportError
from ncclient.xml_ import *
import xml.dom.minidom as minidom
import microdom
from types import *

from ncclient.devices.default import DefaultDeviceHandler
import mgtcrypto
import mgtobject
import notification

class SessionEventHandler(SessionListener):

    def __init__(self, init_cb, error_cb):
        self._init_cb = init_cb
        self._error_cb = error_cb

    def callback(self, root, raw):
        self._init_cb(root, raw)

    def errback(self, err):
        self._error_cb(err)

def getDomName(dom):
    try:
        return dom.nodeName.split(':')[1]
    except Exception, ex:
        return dom.nodeName


class MgtServerObject(mgtobject.MgtObject):

    def __init__(self, parent, attributes,children,data, modules):
        self.config = None
        self.manager = None
        self.modules = modules
        self.keymapping = modules.mapKeyList
        self.frozen = False
        self.lastUpdated = None
        attributes["tag_"] = "networkElement"
        mgtobject.MgtObject.__init__(self, parent, attributes,children,data)
        self.isNetworkElement = True
        self.names.append(attributes["listkey"])

    def miniDom2MgtObjects(self,dom,path,parent):
        """Convert a minidom tree to a set of management objects"""
        # pdb.set_trace()
        if type(dom) is ListType:
            return [self.miniDom2MgtObjects(d,path,self) for d in dom]

        childlist = []
        dataLst=[]

        name = getDomName(dom)

        attrlist = { "tag_": name}

        if dom.childNodes:
            for child in dom.childNodes:
                if child.nodeName == '#text' or child.nodeName == '#cdata-section':  # Raw data
                    if not child.nodeValue.startswith('\n '):
                        childlist.append(str(child.nodeValue))
                        dataLst.append(str(child.nodeValue))
                else:
                    pathc = path + "/" + name
                    childlist.append(self.miniDom2MgtObjects(child,pathc,self))

        for (key,val) in dom._attrs.items():
            attrlist[key] = val.value

        try: 
            data = dom.data
        except AttributeError:
            data = "".join(dataLst) 

        try:
            data = int(data)
            goodAsInt = True
        except ValueError:
            try:
                data = float(data)
            except ValueError:
                data = str(data)
                if data in microdom.Truths: data = True
                if data in microdom.Falses: data = False

        #print path + "/" + name
        #if "/safplusAmf" in path: 
        #  pdb.set_trace()
        #if name == "limit":
        #  pdb.set_trace()
        keyPath = name
        if len(path) > 0:
            keyPath = path[1:] + "/" + name

        listkey = self.keymapping.get(keyPath, None)
        if listkey:
          theObj = mgtobject.MgtObject(parent,attrlist, childlist, data,listkey)
        #  theObj = mgtlist.MgtList(attrlist,childlist,data,listkey)
        else:
          theObj = mgtobject.MgtObject(parent,attrlist, childlist, data)
        return theObj


    def resolvePath(self, path):
      self.fetchData() # Reload the child object tree
      return mgtobject.MgtObject.resolvePath(self,path)

    def getNetworkElement(self):
        """Get network element that is the upper level of the current object -- THAT's ME!"""
        return self


    def updateData(self, name, host, port, user, password):
        self.attributes_["listkey"] = name
        self.host = host
        self.port = port
        self.user = user
        self.passwordEncoded = mgtcrypto.encode(password)

    def dump(self, recurse=True, fullPathOnce=False):
      """Override the dump so we can update live data"""
      self.attributes_["status"] = "connected" if self.connected else "disconnected"
      self.attributes_["fqdn"] = self.host
      return mgtobject.MgtObject.dump(self,recurse,fullPathOnce)

    def connect(self, cli = None):
        try:
            password = mgtcrypto.decode(self.passwordEncoded)

            def allowUnknownHosts(host,fingerprint):
              return True

            print "Connecting to %s" % self.tag_

            try:
                self.manager = manager.connect(host=self.host, port=self.port, username=self.user, password=password,unknown_host_cb=allowUnknownHosts,timeout=5)
            except Exception, e:
                print e

            self.cli = cli

            if self.connected:
                def msg_cb(root, raw):
                    pass
                def err_cb(err):
                    self.children_   = []
                    self.child_      = {}
                    self.data_       = []
                    print '\nError: %s - Server: %s' %(str(err), self.tag_)

                listener = SessionEventHandler(msg_cb, err_cb)
                self.manager._session.add_listener(listener)
                self.manager._session.add_listener(notification.NotificationListener())
        except Exception, e:
            self.connected = False
            print "Connection error"
            print e
            print ""

    def disconnect(self):
        # I don't think I should wipe the data just because I disconnected
        # self.children_   = []
        # self.child_      = {}
        # self.data_       = []
        self.manager.close_session()

    def add_config(self, config):
        self.config = config

    @property
    def connected(self):
        "Whether currently connected to the NETCONF server."
        if self.manager == None:
            return False
        return self.manager.connected

    def fetchData(self):
        """ Retrieve all the configuration information from this network element"""
        if not self.connected:
          self.connect()
        if self.frozen: return  # If the data is "frozen" it means don't fetch it...
        if not self.manager: return  # self.connect should have connected if possible
   
        self.lastUpdated = datetime.datetime.now()

        try:
            self.children_= []

            conf_xml = self.manager.get_config('running').data_xml
            conf_doms = minidom.parseString(conf_xml)
            if conf_xml.find("nc:data") != -1:
                conf_elements = conf_doms.getElementsByTagName("nc:data")
            else:
                conf_elements = conf_doms.getElementsByTagName("data")

            config_data = []
            if conf_elements[0].childNodes:
                for element in conf_elements[0].childNodes:
                    if not (element.nodeName == '#text' and element.nodeValue.startswith('\n ')):
                        config_data.append(str(element.nodeName))

            data_xml = self.manager.get().data_xml
            doms = minidom.parseString(data_xml)
            if data_xml.find("nc:data") != -1:
                elements = doms.getElementsByTagName("nc:data")
            else:
                elements = doms.getElementsByTagName("data")

            if elements[0].childNodes:
                for element in elements[0].childNodes:
                    if not (element.nodeName == '#text' and element.nodeValue.startswith('\n ')):
                        if str(element.nodeName) in config_data:
                            child_obj = self.miniDom2MgtObjects(element,"",self)
                            if child_obj:
                                self.addChild(child_obj)

            self.children_ = sorted(self.children_, key=lambda obj: obj.tag_)

            for child in self.children_:
                if not child.isleaf:
                    child.getKeys()
        except TimeoutExpiredError,e:
            self.connected = False
        except TransportError, e:
            self.connected = False
        except Exception, e:            
            print e
            if not ignoreExceptions: raise

    def cmd_get(self):
        try:
            data_xml = self.manager.get().data_xml
            print data_xml
        except Exception, e:
            print e

    def cmd_get_config(self):
        try:
            conf_xml = self.manager.get_config('running').data_xml
            print conf_xml
        except Exception, e:
            print e

    # Standard NETCONF command
    def cmd_commit(self):
      try:
        with self.manager.locked(target='candidate'):
          self.manager.commit()
          return "<ok/>"
      except Exception, e:
        raise e

    # Standard NETCONF command
    def cmd_copy_config(self, source='running', target='candidate'):
      try:
        self.manager.copy_config(source,target)
        return "<ok/>"
      except Exception, e:
        raise e

    # Standard NETCONF command
    def cmd_get_config(self, xpath):
      try:
        # Using <candidate> database 
        output = self.manager.get_config(source="candidate", filter=("xpath", xpath))

        result = ''
        doms = minidom.parseString(str(output))
        elements = doms.getElementsByTagName("data")
        if elements != [] and elements[0].childNodes:
            for element in elements[0].childNodes:
                result += element.toxml()
        return result
      except Exception, e:
        raise e
