#!/usr/bin/env python
###############################################################################
#
# Copyright (C) 2002-2009 by OpenClovis Inc. All  Rights Reserved.
# 
# The source code for  this program is not published  or otherwise 
# divested of  its trade secrets, irrespective  of  what  has been 
# deposited with the U.S. Copyright office.
# 
# This program is  free software; you can redistribute it and / or
# modify  it under  the  terms  of  the GNU General Public License
# version 2 as published by the Free Software Foundation.
# 
# This program is distributed in the  hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied  warranty  of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
# General Public License for more details.
# 
# You  should  have  received  a  copy of  the  GNU General Public
# License along  with  this program. If  not,  write  to  the 
# Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#
###############################################################################
import re
from threading import Event, Lock
import time
import os
import xml.dom

from threading import Event

from ncclient.xml_ import *
from ncclient import manager
from ncclient.operations import RPC
from ncclient.transport import session
from ncclient.devices.default import DefaultDeviceHandler
from collections import OrderedDict

NOTIFICATION_CAPACITY = 100
NOTIFICATION_LOG = ".notification.log"

class Notification(object):
    """Represents an *notification*.
    """
    _instance = None
    def __new__(self, *args, **kwargs):
        if not Notification._instance:
            Notification._instance = super(Notification, self).__new__(
                                self, *args, **kwargs)
            self._instance._raw = ""
            self._instance._parsestring = ""
            
            self._instance._parsed = False
            self._instance._on = True # Default True: Enable to receive all notifications. Otherwise, all notifications is disable
            self._instance._root = None
            self._instance._print_notify = False
            
            self._instance._notify_log = NOTIFICATION_LOG
            self._instance._capacity = NOTIFICATION_CAPACITY #Capacity of notification messages
            self._instance._errors = []
            self._instance._filter = [] # All notifications in this list will be stopped the output
            self._instance._notifications = OrderedDict() #Dict of notifications with key is DateTime - value is list of notifications
            self._instance._loadNotifications()
                
        return self._instance

    def __init__(self, raw = ""):
        self._raw = raw
        self._parsed = False
        self._root = None
        self._errors = []
        self._print_notify = True

    def __repr__(self):
        return self._raw
    
    def parseChildNode(self, childNodes, space):
        """Parse nodeList and their children"""
        if (childNodes is not None):
            for n in childNodes:
                if (n.nodeType == n.TEXT_NODE):
                    if not n.nodeValue.isspace():
                        self._parsestring += "  " * space + "%s\n" % str(n.nodeValue)
                elif (n.nodeType == n.ELEMENT_NODE):
                    self._parsestring += "  " * space + "%s {\n" % n.nodeName
                    space += 1
                    self.parseChildNode(n.childNodes, space)
                    space -= 1
                    self._parsestring += "  " * space + "}\n"
                 
    def parse(self):
        """Parses the *notification* for easier view"""
        if not self._on:
            return ""
        tag, attrs = self._root
        m = re.search(r'.*notification$',tag)
        ret = ""
        if (tag == "notification" or m):
            space = 0
            self._parsestring = ""
            dom = xml.dom.minidom.parseString(self._raw)
            try:
                self.parseChildNode(dom.childNodes, space)
            except Exception as e:
                self._error_cb(e)
            if (self._print_notify):
                print self._parsestring
            
            self._insertNotification(self._parsestring)
            #TODO flushNotication should be done when CLI close 
            self._flushNotification()
            ret = "<text>" + self._parsestring + "</text>"    
        return ret
      
    def _error_cb(self, e):
        ""
        self._errors.append(e)
        print "Error :"
        print e
       
    def _parsing_hook(self, root):
        "No-op by default. Gets passed the *root* element for the reply."
        pass

  
            
    def _insertNotification(self, parse_string):
        """Insert a notification to buffer when it comes """
        if (len(self._notifications) >= self._capacity):
            #remove the last one
            last_list = self._notifications.values()[0]
            last_list.pop()
            if (len(last_list) <= 0):
                self._notifications.pop(self._notifications.keys()[0])
                
        date_str = time.strftime("%Y-%m-%d")
        if date_str in self._notifications:
            cur_list = self._notifications[date_str]
            cur_list.append(parse_string)
        else:
            self._notifications[date_str] = [parse_string]
            
    def _loadNotifications(self):
        """Load all the saved notifications in _notify_log"""
        #Create file if not exist
        if not os.path.exists(self._notify_log):
             open(self._notify_log, "a")
        #Load all notifications log
        with open(self._notify_log, "r") as f:
            key = ""
            notify = ""
            for line in f:
                m1 = re.match(r'^Date:\s+(\d{4}-\d{2}-\d{2})$', line)
                m2 = re.match(r'^\}$', line) #End of notification
                if m1:
                    #Found a key    
                    key = m1.group(1)
                    
                elif m2:
                    notify += "}"
                    if not key.isspace():
                        if key in self._notifications:
                          list = self._notifications[key]
                          list.append(notify)
                        else:
                            self._notifications[key] = [notify]
                    notify = ""
                else:
                    if not line.isspace():
                        notify += line
          
    def _flushNotification(self):
        """ Flush notification into _notify_log """
        with open(self._notify_log, "w") as file:
            str = ""
            for key, list in self._notifications.items():
                str += "Date: %s\n" % key
                for notify in list:
                    str += "%s\n" % notify
            file.write(str)        
    
    def ShowNotifications(self, val):
        """Show notifications from newest to oldest"""
        ret = ""
        showDict = OrderedDict()
        items = self._notifications.items()
        items.reverse()
        reverseDict = OrderedDict(items)
        if val is None:
            return ""
            
        #Show N recently notifications
        elif val.isdigit():
            i = 0
            last = False
            for key, value in reverseDict.items():
                if last: break
                showDict[key] = []
                list = value[:]
                list.reverse() #reverse to get the newest one
                for notify in list:
                    i += 1
                    if (i >= int(val)): last = True
                    if (i > int(val)): break   
                    showDict[key].append(notify)
                    
        #Show notification from date
        else:
            m = re.match(r'^\d{4}-\d{2}-\d{2}$', val)
            if m:
                #Check if the date is greater than the oldest date
                if (val >= self._notifications.keys()[0]):
                    for key, value in reverseDict.items():
                        if (val <= key):
                            showDict[key] = []
                            list = value[:]
                            list.reverse() #reverse to get the newest one
                            for notify in list:
                                showDict[key].append(notify)
                        else:
                            break
            else:
                ret = "<error>Wrong format to show notification. Try 'help' for details</error>" 
        
        ret += "<top>"
        for key, list in showDict.items():
          ret += "<text>%s:</text>" % key
          for notify in list:
              ret += "<text>%s</text>" % notify
        ret += "</top>"
        return ret
    
    @property
    def xml(self):
        "*notification* element as returned."
        return self._raw

    @property
    def ok(self):
        "Boolean value indicating if there were no errors."
        return not self.errors # empty list => false

class NotificationListener(session.SessionListener):
    creation_lock = Lock()

    def __init__(self):
        self._init_cb = self.callback
        self._error_cb = self.errback
        self._notify = Notification()

    def callback(self, root, raw):
        self._notify._root = root
        self._notify._raw = raw
        self._notify.parse()
      
    def errback(self, err):
        self._error_cb(err)
               
def request(manager, stream = "NETCONF", filter = "", start = "", stop = ""):
    try:
        rpc = RPC(manager.manager._session, device_handler=DefaultDeviceHandler(), async=manager.manager._async_mode, timeout=manager.manager._timeout, raise_mode=manager.manager._raise_mode)
        xml = """
            <create-subscription xmlns=\"urn:ietf:params:xml:ns:netconf:notification:1.0\">\n
            </create-subscription>\n
            """
        str = etree.fromstring(xml)
        
        print str
        result = rpc._request(str)
        return "%s" % result
    except Exception, e:
        print e
        return "" 