#include <mgtclient.hxx>

static boost::local_time::time_zone_ptr const utc_time_zone(new boost::local_time::posix_time_zone("GMT"));
/*constructor*/
MgtClient::MgtClient(const std::string& puserName, const std::string& ppassWord, const std::string& pipAddress, const int& pport, const bool& pisRunning)
{
  isConnected = false;
  isRunning = pisRunning;
  timeOut = 100;
  isDataAvailable = false;
  isSubscriber = false;
  isFinishSubscriber = false;
  strUserName = puserName;
  strPassWord = ppassWord;
  strIpAddress = pipAddress;
  iPort = pport;
  messageId = 0;
  currentTransactionId = 0;
  if (isRunning) strDatabase = "running";
  else strDatabase = "candidate";
}
/*Get isConnect*/
bool MgtClient::GetIsConnect()
{
  return isConnected;
}
/*Get isRuning*/
bool MgtClient::GetIsRunning()
{
  return isRunning;
}
/*Set timeout for entire connnection*/
void MgtClient::SetTimeOut(const int& ptimeOut)
{
  timeOut = ptimeOut;
}
/*Get timeout*/
int MgtClient::GetTimeout()
{
  return timeOut;
}
/*Connect to server*/
ClRcT MgtClient::Connect()
{
  int rcRet = Connect(session_data, channel_data);
  if (CL_OK == rcRet) isConnected = true;
  return rcRet;
}
/*Connect to Server*/
ClRcT MgtClient::Connect(ssh_session& psession, ssh_channel& pchannel)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  int nbytes;
  ClRcT rc, rcRet;
  char bufferIn[MAXBUFF];
  rcRet = CL_ERR_UNSPECIFIED;
  if (strUserName.length() == 0 || strPassWord.length() == 0 || strIpAddress.length() == 0 || iPort == 0)
  {
    logError("MGT", "NC", "Can not connect with parameter not set!");
    return rcRet;
  }
  psession = ssh_new();
  if (psession == NULL) return (-1);
  ssh_options_set(psession, SSH_OPTIONS_HOST, strIpAddress.c_str());
  ssh_options_set(psession, SSH_OPTIONS_LOG_VERBOSITY, &verbosity);
  ssh_options_set(psession, SSH_OPTIONS_PORT, &iPort);
  ssh_options_set(psession, SSH_OPTIONS_USER, strUserName.c_str());
  rc = ssh_connect(psession);
  if (rc != SSH_OK)
  {
    logError("MGT", "NC", "Error connecting to localhost: %s\n", ssh_get_error(psession));
    return CL_ERR_UNSPECIFIED;
  }
  rc = ssh_userauth_password(psession, NULL, strPassWord.c_str());
  if (rc != SSH_AUTH_SUCCESS)
  {
    logError("MGT", "NC", "Error authenticating with password: %s\n", ssh_get_error(psession));
    rcRet = CL_ERR_OP_NOT_PERMITTED;
    goto freesession;
  }
  pchannel = ssh_channel_new(psession);
  if (pchannel == NULL)
  {
    logError("MGT", "NC", "Error channel_data is null\n");
    rcRet = CL_ERR_UNSPECIFIED;
    goto freesession;
  }
  rc = ssh_channel_open_session(pchannel);
  if (rc < 0)
  {
    logError("MGT", "NC", "Open session failed with rc:%d\n", rc);
    rcRet = CL_ERR_UNSPECIFIED;
    goto freechannel;
  }
  rc = ssh_channel_request_subsystem(pchannel, "netconf");
  if (rc < 0)
  {
    logError("MGT", "NC", "Execute netconf failed with rc:%d\n", rc);
    rcRet = CL_ERR_UNSPECIFIED;
    goto freechannel;
  }
  if (ssh_channel_is_open(pchannel) && !ssh_channel_is_eof(pchannel))
  {
    while (1)
    {
      memset(bufferIn, 0, MAXBUFF);
      nbytes = ssh_channel_read_timeout(pchannel, bufferIn, MAXBUFF, 0, timeOut);
      if (nbytes < 0) return CL_ERR_UNSPECIFIED;
      if (nbytes > 0)
      {
        logDebug("MGT", "NC", "receive Hello OK");
      } else
      {
        break;
      }
    }
    SendHelloMessage(pchannel);
  } else goto freechannel;
  return CL_OK;

  freesession: ssh_disconnect(psession);
  ssh_free(psession);
  return rcRet;
  freechannel: ssh_channel_close(pchannel);
  ssh_channel_free(pchannel);
  ssh_disconnect(psession);
  ssh_free(psession);
  return rcRet;
}
/*Transaction operation*/
ClRcT MgtClient::TransStart(uint64_t& transId)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  ClRcT rc;
  int nbytes, datalength;
  char buffer[MAXBUFF];
  char bufferIn[MAXBUFF];
  char bufferOut[MAXBUFF];
  std::string strResult;
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strGetTrans.c_str(), GetMessageId().c_str(), strDatabase.c_str());
  rc = ExecRpc(buffer, strResult);
  if (CL_OK != rc) return rc;
  size_t posEraseBegin = strResult.find("<data>");
  if (std::string::npos == posEraseBegin) return CL_ERR_UNSPECIFIED;
  strResult.erase(0, posEraseBegin + 6);
  size_t posEraseEnd = strResult.find("</data>");
  if (std::string::npos == posEraseEnd) return CL_ERR_UNSPECIFIED;
  strResult.erase(posEraseEnd, strResult.length() - posEraseEnd);
  xmlParser lxmlParser(strResult);
  try
  {
    transId = lxmlParser.getValue < uint64_t > ("trans-id");
    currentTransactionId = transId;
  }
  catch (std::exception const& ex)
  {
    return CL_ERR_UNSPECIFIED;
  }
  return CL_OK;
}
/*Get Transaction Id*/
uint64_t MgtClient::GetTransId()
{
  return currentTransactionId;
}
/*Transaction roll back*/
ClRcT MgtClient::TransRollback(const uint64_t& transId)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strTransRollBack.c_str(), GetMessageId().c_str(), transId);
  return ExecRpc(buffer);
}
/*Transaction commit*/
ClRcT MgtClient::TransCommit(const uint64_t& transId)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strTransCommit.c_str(), GetMessageId().c_str(), transId);
  return ExecRpc(buffer);
}
/*Create subscriber
 *strPath: string of path subscriber
 *startTime: start of time
 *pcbNotification: callback function
 *return : CL_OK is success,else failed
 */
ClRcT MgtClient::Subscriber(const std::string& strPath, std::function<void(void*)> pcbNotification)
{
  return Subscriber(strPath, "", "", pcbNotification);
}
/*Create subscriber
 *strPath: string of path subscriber
 *startTime: start of time
 *stopTime: stop of time
 *pcbNotification: callback function
 *return : CL_OK is success,else failed
 */
ClRcT MgtClient::Subscriber(const std::string& strPath, boost::posix_time::ptime startTime, std::function<void(void*)> pcbNotification)
{
  boost::local_time::local_time_facet* facet = new boost::local_time::local_time_facet("%Y-%m-%dT%T%Q");
  boost::local_time::local_date_time lstartTime(startTime, utc_time_zone);
  std::ostringstream startdate_osstr;
  startdate_osstr.imbue(std::locale(startdate_osstr.getloc(), facet));
  startdate_osstr << lstartTime;
  return Subscriber(strPath, startdate_osstr.str(), "", pcbNotification);
}
/*Create subscriber
 *strPath: string of path subscriber
 *startTime: start of time
 *stopTime: stop of time
 *pcbNotification: callback function
 *return : CL_OK is success,else failed
 */
ClRcT MgtClient::Subscriber(const std::string& strPath, boost::posix_time::ptime startTime, boost::posix_time::ptime stopTime,
    std::function<void(void*)> pcbNotification)
{
  if (startTime >= stopTime)
  {
    logError("MGT", "NC", "stopTime must greater than startTime!");
    return CL_ERR_UNSPECIFIED;
  }
  boost::local_time::local_time_facet* facet = new boost::local_time::local_time_facet("%Y-%m-%dT%T%Q");
  boost::local_time::local_date_time lstartTime(startTime, utc_time_zone);
  boost::local_time::local_date_time lstopTime(stopTime, utc_time_zone);
  std::ostringstream startdate_osstr;
  startdate_osstr.imbue(std::locale(startdate_osstr.getloc(), facet));
  startdate_osstr << lstartTime;
  std::ostringstream stopdate_osstr;
  stopdate_osstr.imbue(std::locale(stopdate_osstr.getloc(), facet));
  stopdate_osstr << lstopTime;
  return Subscriber(strPath, startdate_osstr.str(), stopdate_osstr.str(), pcbNotification);
}
/*Create subscriber
 *strPath: string of path subscriber
 *startTime: string start of time
 *stopTime: string stop of time
 *pcbNotification: callback function
 *return : CL_OK is success,else failed
 */
ClRcT MgtClient::Subscriber(const std::string& strPath, const std::string& strStartTime, const std::string& strStopTime,
    std::function<void(void*)> pcbNotification)
{
  int nbytes, datalength;
  char buffer[MAXBUFF];
  char bufferFilter[MAXBUFF];
  char bufferOut[MAXBUFF];
  if (!isSubscriber)
  {
    if (CL_OK != Connect(session_subscriber, channel_subscriber))
    {
      logError("MGT", "NC", "Connect notification channel_data error!");
      return CL_ERR_UNSPECIFIED;
    }
    if (ssh_channel_is_open(channel_subscriber) && !ssh_channel_is_eof(channel_subscriber))
    {
      std::stringstream ssTime;
      if (strStartTime.length() > 0)
      {
        ssTime << "<startTime>" << strStartTime << "</startTime>";
      } else ssTime << "";
      if (strStopTime.length() > 0)
      {
        ssTime << "<stopTime>" << strStopTime << "</stopTime>";
      } else ssTime << "";
      memset(buffer, 0, MAXBUFF);
      if (strPath.length() > 0)
      {
        std::string strXmlContent = path2xml(strPath);
        memset(bufferFilter, 0, MAXBUFF);
        sprintf(bufferFilter, "<filter type=\"subtree\">%s</filter>", strXmlContent.c_str());
        sprintf(buffer, strCreateSubscriber.c_str(), GetMessageId().c_str(), ssTime.str().c_str(), bufferFilter);
      } else
      {
        sprintf(buffer, strCreateSubscriber.c_str(), GetMessageId().c_str(), ssTime.str().c_str(), (char*) " ");
      }
      datalength = strlen(buffer);
      memset(bufferOut, 0, MAXBUFF);
      sprintf(bufferOut, "\n#%d\n%s\n##\n", datalength, buffer);
      datalength = strlen(bufferOut);
      nbytes = ssh_channel_write(channel_subscriber, bufferOut, datalength);
      if (nbytes < 0) return CL_ERR_UNSPECIFIED;
      cbNotification = pcbNotification;
      threadSubscriber = std::thread(&MgtClient::funcSubscriber, this);
      int count = 0;
      while (!isFinishSubscriber)
      {
        std::this_thread::sleep_for(std::chrono::microseconds(1000));
        if (count > MAXCOUNTWAIT) break;
        count++;
      }//while
      if (isSubscriber)
      {
        return CL_OK;
      }
    }
  }
  return CL_ERR_UNSPECIFIED;
}
/*UnSubscribe*/
ClRcT MgtClient::UnSubscriber()
{
  if (isSubscriber)
  {
    isSubscriber = false;
    threadSubscriber.join();
    Close(session_subscriber, channel_subscriber);
    logDebug("MGT", "NC", "UnSubscribe!\n");
  }
  return CL_OK;
}
/*GetConfig
 *strFilter: string of path filter
 *outValue: result return
 *return : CL_OK is success,else failed
 */
ClRcT MgtClient::GetConfig(const std::string& strFilter, std::string& outValue, const bool& isXpath)
{
  return GetConfig(strFilter, strDatabase, outValue, isXpath);
}
/*GetConfig
 *strFilter: string of path filter
 *databaseName: candidate/running
 *outValue: result return
 *return : CL_OK is success,else failed
 */
ClRcT MgtClient::GetConfig(const std::string& strFilter, const std::string& databaseName, std::string& outValue, const bool& isXpath)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  if (strFilter.length() > 0)
  {
    char bufferFilter[MAXBUFF];
    memset(bufferFilter, 0, MAXBUFF);
    if (isXpath)
    {
      std::string strXmlPath = path2xml(strFilter);
      sprintf(bufferFilter, "<nc:filter type=\"subtree\">%s</nc:filter>", strXmlPath.c_str());
    }else
    {
      sprintf(bufferFilter, "<nc:filter type=\"subtree\">%s</nc:filter>", strFilter.c_str());
    }
    sprintf(buffer, strGetConfig.c_str(), GetMessageId().c_str(), databaseName.c_str(), bufferFilter);
  } else
  {
    sprintf(buffer, strGetConfig.c_str(), GetMessageId().c_str(), databaseName.c_str(), " ");//empty
  }
  ClRcT rc = ExecRpc(buffer, outValue);
  if (CL_OK == rc)
  {
    if (outValue.length() > 0)
    {
      size_t posEraseBegin = outValue.find("<data>");
      if (std::string::npos == posEraseBegin) return CL_ERR_UNSPECIFIED;
      outValue.erase(0, posEraseBegin + 6);
      size_t posEraseEnd = outValue.find("</data>");
      if (std::string::npos == posEraseEnd) return CL_ERR_UNSPECIFIED;
      outValue.erase(posEraseEnd, outValue.length() - posEraseEnd);
      if (outValue.length() == 0) return CL_ERR_UNSPECIFIED;
      xmlObj.setContent(outValue);
      isDataAvailable = true;
      return CL_OK;
    } else return CL_ERR_UNSPECIFIED;
  } else return CL_ERR_UNSPECIFIED;
}
/*Checks existense of an object.
 *strPath: path to element
 *This function checks whether a node exists in database, return true if it exists, false if not
 */
bool MgtClient::IsExist(const std::string& strPath)
{
  return IsExist(strPath, strDatabase);
}
/*Checks existense of an object.
 *strPath: path to element
 *databaseName: candidate/running
 *This function checks whether a node exists in database, return true if it exists, false if not
 */
bool MgtClient::IsExist(const std::string& strPath, const std::string& databaseName)
{
  std::string strRet;
  if (strPath.length() == 0)
  {
    logError("MGT", "NC", "Path is empty");
    return false;
  }
  ClRcT rc = GetConfig(strPath, databaseName, strRet);
  if (CL_OK == rc)
  {
    return true;
  } else
  {
    return false;
  }
}
/*Returns all the values in a container or list entry
 *strPath: path to element
 *tagOutValue: result return
 */
ClRcT MgtClient::GetObject(const std::string& strPath, std::string& tagOutValue)
{
  return GetObject(strPath, strDatabase, tagOutValue);
}
/*Returns all the values in a container or list entry
 *strPath: path to element
 *databaseName: candidate/running
 *tagOutValue: result return
 */
ClRcT MgtClient::GetObject(const std::string& strPath, const std::string& databaseName, std::string& tagOutValue)
{
  return GetConfig(strPath, databaseName, tagOutValue);
}
/*Returns the values for the leafs that have the "value" 'not_found' in the Values list. This can be used to read an
 *arbitrary set of sub-elements of a container or list entry. The return value is a list of the same length as Values,
 *arbitrary  i.e. the requested leafs are in the same position in the returned list as in the Values argument. The elements in the returned list are always
 *arbitrary "canonical" though, i.e. of the form {[Ns|Tag], value() | start | stop | leaf}
 *Write an entire object, i.e. YANG list entry or container
 *strPath: path
 *tagInValue: xml format attribute
 *transId: transaction id
 */
ClRcT MgtClient::SetObject(const std::string& strPath, const std::string& tagInValue, const uint64_t& transId)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  std::string strXmlContent = path2xml < std::string > (strPath, tagInValue, false);
  sprintf(buffer, strSetConfig.c_str(), GetMessageId().c_str(), transId, strDatabase.c_str(), strXmlContent.c_str());
  return ExecRpc(buffer);
}
/*Set namespace for next rpc*/
void MgtClient::SetNameSpace(const std::string& pstrNameSpace)
{
  strNameSpace = pstrNameSpace;
}
/*Do RPC
 *rpcName: string rpc name
 *tagInValue: xml format value
 *strOutput: result output
 *return CL_OK: if success,else: failed
 */
ClRcT MgtClient::DoRpc(const std::string& rpcName, const std::string& tagInValue, std::string& strOutput)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  if (rpcName.length() == 0)
  {
    logError("MGT", "NC", "RPC Name is empty");
    return CL_ERR_UNSPECIFIED;
  }
  if (strNameSpace.length() == 0)
  {
    logError("MGT", "NC", "NameSpace is empty");
    return CL_ERR_UNSPECIFIED;
  }
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strRPC.c_str(), GetMessageId().c_str(), rpcName.c_str(), strNameSpace.c_str(), tagInValue.c_str(), rpcName.c_str());
  return ExecRpc(buffer, strOutput);
}
/*Lock database
 *strTargetDatabase: database name: candidate,running
 *return CL_OK if success, else: failed
 */
ClRcT MgtClient::Lock(const std::string& strTargetDatabase)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  if (strTargetDatabase.length() == 0)
  {
    logError("MGT", "NC", "Target Database is empty");
    return CL_ERR_UNSPECIFIED;
  }
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strLock.c_str(), GetMessageId().c_str(), strTargetDatabase.c_str());
  return ExecRpc(buffer);
}
/*UnLock database
 *strTargetDatabase: database name: candidate,running
 *return CL_OK if success, else: failed
 */
ClRcT MgtClient::UnLock(const std::string& strTargetDatabase)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  if (strTargetDatabase.length() == 0)
  {
    logError("MGT", "NC", "Target Database is empty");
    return CL_ERR_UNSPECIFIED;
  }
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strUnLock.c_str(), GetMessageId().c_str(), strTargetDatabase.c_str());
  return ExecRpc(buffer);
}
/*Discard change
 *return CL_OK if success, else: failed
 */
ClRcT MgtClient::DiscardChange()
{
  std::lock_guard <std::mutex> lock(mutex_data);
  if (isRunning)
  {
    logError("MGT", "NC", "running database can not discard change");
    return CL_ERR_UNSPECIFIED;
  }
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strDiscardChange.c_str(), GetMessageId().c_str());
  return ExecRpc(buffer);
}
/*SetLogLevel
 *logLevel: log level
 *return CL_OK if success, else: failed
 */
ClRcT MgtClient::SetLogLevel(const SAFplus::LogSeverity& logLevel)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  std::string strLevel = SAFplus::logSeverityStrGet(logLevel);
  sprintf(buffer, strSetLogLevel.c_str(), GetMessageId().c_str(), boost::algorithm::to_lower_copy(strLevel).c_str());
  return ExecRpc(buffer);
}
/*Copy config
 *srcDatabase: source database name: candidate,running
 *destDatabase: destination database name: candidate,running
 *return CL_OK if success, else: failed
 */
ClRcT MgtClient::CopyConfig(const std::string& srcDatabase, const std::string& destDatabase)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  if (isRunning)
  {
    logError("MGT", "NC", "running database can not copy");
    return CL_ERR_UNSPECIFIED;
  }
  if (srcDatabase.length() == 0)
  {
    logError("MGT", "NC", "Source Database is empty");
    return CL_ERR_UNSPECIFIED;
  }
  if (destDatabase.length() == 0)
  {
    logError("MGT", "NC", "Destination Database is empty");
    return CL_ERR_UNSPECIFIED;
  }
  if (srcDatabase.compare(destDatabase) == 0)
  {
    logError("MGT", "NC", "Both database is same");
    return CL_ERR_UNSPECIFIED;
  }
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strCopyConfig.c_str(), GetMessageId().c_str(), srcDatabase.c_str(), destDatabase.c_str());
  return ExecRpc(buffer);
}
/*Commit RPC
 *return CL_OK if success, else: failed
 */
ClRcT MgtClient::Commit()
{
  std::lock_guard <std::mutex> lock(mutex_data);
  if (isRunning)
  {
    logError("MGT", "NC", "running database can not commit");
    return CL_ERR_UNSPECIFIED;
  }
  char buffer[MAXBUFF];
  memset(buffer, 0, MAXBUFF);
  sprintf(buffer, strCommit.c_str(), GetMessageId().c_str());
  return ExecRpc(buffer);
}
/* Close Connection*/
ClRcT MgtClient::Close(ssh_session & psession, ssh_channel & pchannel)
{
  std::lock_guard <std::mutex> lock(mutex_data);
  if (ssh_channel_is_open(pchannel))
  {
    if (ssh_channel_is_eof(pchannel) == 0)
    {
      if (SSH_OK != ssh_channel_send_eof(pchannel))
      {
        logError("MGT", "NC", "Error send end of file!");
      }
    }
    if (SSH_OK != ssh_channel_close(pchannel))
    {
      logError("MGT", "NC", "Error close channel_data!\n");
    }
    ssh_channel_free(pchannel);
    pchannel = nullptr;
    ssh_disconnect(psession);
    ssh_free(psession);
    psession = nullptr;
  }
  return CL_OK;
}
ClRcT MgtClient::Close()
{
  isConnected = false;
  return Close(session_data, channel_data);
}
/* Destructor*/
MgtClient::~MgtClient()
{
  Close();
  UnSubscriber();
}
/*Generate new messageId
 *return: new message id
 */
std::string MgtClient::GenerateMessageId()
{
  return std::to_string(++messageId);
}
/*Get messageId
 *return: message id
 */
std::string MgtClient::GetMessageId()
{
  return std::to_string(messageId);
}
/*function listener subscriber*/
void MgtClient::funcSubscriber()
{
  int nbytes;
  char bufferIn[MAXBUFF];
  do
  {
    if (ssh_channel_is_open(channel_subscriber))
    {
      memset(bufferIn, 0, MAXBUFF);
      nbytes = ssh_channel_read_timeout(channel_subscriber, bufferIn, MAXBUFF, 0, timeOut);//remove data from buffer
      if (nbytes > 0)
      {
        if (!isFinishSubscriber)
        {
          if (strstr(bufferIn, "<ok/>") != nullptr)
          {
            isSubscriber = true;
          }
          isFinishSubscriber = true;
        }
        if (isSubscriber)
        {
          if (IsNotification(bufferIn))
          {
            if (cbNotification != nullptr) cbNotification(bufferIn);
          }
        }
      }
    }
    std::this_thread::sleep_for(std::chrono::microseconds(1000));
  } while (isSubscriber);
}
ClRcT MgtClient::ExecRpc(const std::string& tagInValue)
{
  std::string strResult;
  ClRcT rc = ExecRpc(tagInValue, strResult);
  if (CL_OK == rc) return rc;
  if ((CL_OK == rc) && (strstr(strResult.c_str(), "<ok/>") != nullptr))
  {
    return CL_OK;
  } else return CL_ERR_UNSPECIFIED;
}
ClRcT MgtClient::ExecRpc(const std::string& tagInValue, std::string& strOutput)
{
  if (!isConnected)
  {
    logError("MGT", "NC", "Not connected!");
    return CL_ERR_UNSPECIFIED;
  }
  if (ssh_channel_is_open(channel_data) && !ssh_channel_is_eof(channel_data))
  {
    int nbytes, datalength;
    char bufferIn[MAXBUFF];
    char bufferOut[MAXBUFF];
    datalength = tagInValue.length();
    memset(bufferOut, 0, MAXBUFF);
    sprintf(bufferOut, "\n#%d\n%s\n##\n", datalength, tagInValue.c_str());
    datalength = strlen(bufferOut);
    nbytes = ssh_channel_write(channel_data, bufferOut, datalength);
    if (nbytes != datalength) return CL_ERR_UNSPECIFIED;
    std::stringstream ss;
    do
    {
      memset(bufferIn, 0, MAXBUFF);
      nbytes = ssh_channel_read_timeout(channel_data, bufferIn, sizeof(bufferIn), 0, timeOut);
      if (nbytes <= 0) break;
      else
      {
        logDebug("MGT", "NC", "Receive result from server:\n%s\n", bufferIn);
        std::string strTemp(bufferIn);
        int poshead = 0;
        int postail = 0;
        while (true)
        {
          poshead = strTemp.find("\n#");
          if (poshead >= 0)
          {
            postail = strTemp.find("\n", poshead+1);
            if (postail > poshead)
            {
              strTemp.erase(poshead, postail - poshead + 1);//delete head
            }else
            {
              break;
            }
          } else
          {
            break;
          }
        }//while
        //delete tail
        poshead = strTemp.find("\n##\n");
        if (poshead >= 0)
        {
          strTemp.erase(poshead, 4);//delete \n##\n
        }
        ss << strTemp;
      }
    } while (true);
    strOutput = ss.str();
    if (strOutput.length() > 0)
    {
      if (strstr(strOutput.c_str(), "<rpc-error>") != nullptr)
      {
        return CL_ERR_UNSPECIFIED;
      } else
      {
        return CL_OK;
      }
    } else return CL_ERR_UNSPECIFIED;
  } else
  {
    logError("MGT", "NC", "ExecuteRpc error ssh channel_data!");
    isConnected = false;
    return CL_ERR_UNSPECIFIED;
  }
}
bool MgtClient::IsNotification(void* data)
{
  if (data == nullptr) return false;
  if (strstr((char*) data, "<notification") != nullptr) return true;
  else return false;
}
void MgtClient::SendHelloMessage(ssh_channel& pchannel)
{
  if (ssh_channel_is_open(pchannel) && !ssh_channel_is_eof(pchannel))
  {
    ssh_channel_write(pchannel, strHello.c_str(), strlen(strHello.c_str()));
  } else
  {
    logError("MGT", "NC", "Cannot send message hello");
  }
}
