#include <iostream>
#include <clDbalBase.hxx>
#include <clTestApi.hxx>
#include <mgtclient.hxx>

using namespace SAFplus;
#define clprintf(sev,...) SAFplus::logMsgWrite(SAFplus::APP_LOG, sev, 0, "APP", "MAIN", __FILE__, __LINE__, __VA_ARGS__)

const int NUMELEMENTS = 10;
void callbackfunction(void*data)
{
  std::cout << "callbackfunction exec:\n*******************************************************************\n" << (char*) data << std::endl;
  std::cout << "*******************************************************************\n";
}
void testCreateList();
void testDeleteList();
void testGetLists();
void testGetList();
void testExist();
void testGetConfig();
void testGetElement();
void testSetConfig();
void testSubscriber();
void testSetObject();
void testGetObject();
void testCreateLeafList();
void testDeleteLeafList();
void testGetLeafList();
void testRPC();
void testTransCommit();
void testTransRollback();
void testGetObject();
void testLock_Unlock();
void testDiscard();
void testSetLogLevel();
void testCopyConfig();
bool isRunning = true;
MgtClient client("root", "root", "127.0.0.1", 830, isRunning);
int main(void)
{
  SAFplus::logSeverity = SAFplus::LOG_SEV_DEBUG;
  clTestGroupInitialize(("MGT-MGTCLIENT: unit test for mgtclient"));
  int port, i;
  std::string strResult, strResult1, strResult2, strResult3;
  uint64_t transId;
  ClRcT rc = CL_OK;
  rc = client.Connect();
  clTest(("Opening database is "), rc == CL_OK, (("rc is %x"), rc));
  client.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client.GenerateMessageId();
  ////////////////////////////////////////////////////////////
  testCreateList();
  sleep(1);
  testGetLists();
  testGetList();
  testGetElement();
  testSubscriber();
  sleep(1);
  testSetConfig();
  testSetObject();
  testGetObject();
  testCreateLeafList();
  testGetLeafList();
  testRPC();
  testTransCommit();
  testTransRollback();
  testLock_Unlock();
  testDiscard();
  testSetLogLevel();
  testCopyConfig();
  testExist();
  testGetConfig();
  testDeleteLeafList();
  testDeleteList();
  sleep(2);
  ///////////////////////////////////////////////////////////
  client.UnSubscriber();
  client.Close();
  clTestGroupFinalize();
  return 0;
}
void testCreateList()
{
  for (int i = 0; i < NUMELEMENTS; i++)
  {
    char buffer[100];
    char buffer1[100];
    memset(buffer, 0, 100);
    memset(buffer1, 0, 100);
    sprintf(buffer, "<name>host%d</name>", i);
    sprintf(buffer1, "[name=host%d]", i);
    std::string strPath = "myService\\subscribers\\subscriber";
    ClRcT rc = client.CreateElement<std::string>(strPath, buffer);
    clTest(("CreateElement %s is ", buffer), rc == CL_OK, ("rc is %x", rc));
    logInfo("DBAL", "TEST", "MessageId: %s", client.GenerateMessageId().c_str());
    std::string strResult;
    strPath.append(buffer1);
    if (!isRunning)
    {
      sleep(1);
      rc = client.Commit();
      clTest(("Commit is "), rc == CL_OK, ("rc is %x", rc));
    }
    bool bresult = client.IsExist(strPath);
    clTest(("GetObject is "), bresult, ("rc is %x", rc));
  }
}
void testDeleteList()
{
  for (int i = 0; i < NUMELEMENTS; i++)
  {
    char buffer[100];
    char buffer1[100];
    memset(buffer, 0, 100);
    memset(buffer, 0, 100);
    sprintf(buffer, "<name>host%d</name>", i);
    sprintf(buffer1, "[name=host%d]", i);
    std::string strPath = "myService\\subscribers\\subscriber";
    ClRcT rc = client.DeleteElement<std::string>(strPath, buffer);
    strPath.append(buffer1);
    clTest(("Delete %s is", buffer), rc == CL_OK, ("rc is %x", rc));
    bool isexist = client.IsExist(strPath);
    clTest(("Still exist %s is", strPath.c_str()), !isexist, ("bool is %x", isexist));
    if (!isRunning)
    {
      sleep(1);
      rc = client.Commit();
      clTest(("Commit is"), rc == CL_OK, ("rc is %x", rc));
    }
  }
}
void testGetLists()
{
  std::string strResult;
  std::string strPath = "myService\\subscribers\\subscriber";
  ClRcT rc = client.GetObject(strPath, strResult);
  clTest(("Still exist %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
}
void testGetList()
{
  std::string strResult;
  std::string filter = "myService\\subscribers\\subscriber[name=host0]";
  ClRcT rc = client.GetObject(filter, strResult);
  clTest(("xpath exist %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
  filter = "<myService><subscribers><subscriber><name>host0</name></subscriber></subscribers></myService>";
  rc = client.GetConfig(filter, strResult, false);
  clTest(("subtree exist %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
}
void testExist()
{
  std::string strResult;
  std::string filter = "myService\\subscribers\\subscriber[name=host0]";
  bool rcBool = client.IsExist(filter);
  clTest(("xpath exist %s is", filter.c_str()), rcBool, ("rcBool is %x", rcBool));
}
void testGetConfig()
{
  std::string strResult;
  std::string strPath = "myService";
  ClRcT rc = client.GetConfig(strPath, strResult);
  clTest(("GetConfig data %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
}
void testGetElement()
{
  int port;
  ClRcT rc = client.GetElement<int>("myService\\config\\port", port);
  clTest(("GetElement data %d is", port), CL_OK == rc, ("rc is %x", rc));
}
void testSetConfig()
{
  int port = 32;
  ClRcT rc;
  if (CL_OK == client.SetElement<int>("myService\\config\\port", port)) ;
  clTest(("SetElement data %d is", port), CL_OK == rc, ("rc is %x", rc));
  if (!isRunning)
  {
    rc = client.Commit();
    clTest(("Commit data %d is", port), CL_OK == rc, ("rc is %x", rc));
  }
  rc = client.GetElement<int>("myService\\config\\port", port);
  clTest(("GetElement data %d is", port), CL_OK == rc, ("rc is %x", rc));
  clTest(("GetElement data %d is", port), 32 == port, ("port is %d", port));
  sleep(1);
  port = 33;
  if (CL_OK == client.SetElement<int>("myService\\config\\port", port)) ;
  clTest(("SetElement data %d is", port), CL_OK == rc, ("rc is %x", rc));
  if (!isRunning)
  {
    rc = client.Commit();
    clTest(("Commit data %d is", port), CL_OK == rc, ("rc is %x", rc));
  }
  rc = client.GetElement<int>("myService\\config\\port", port);
  clTest(("GetElement data %d is", port), CL_OK == rc, ("rc is %x", rc));
  clTest(("GetElement data %d is", port), 33 == port, ("port is %d", port));
}
void testSubscriber()
{
  ClRcT rc = client.Subscriber("myService\\config\\port", boost::posix_time::second_clock::local_time(), &callbackfunction);
  clTest(("Subscriber is"), CL_OK == rc, ("rc is %x", rc));
}
void testSetObject()
{
  ClRcT rc = client.SetObject("myService\\computer",
      "<harddrive>4</harddrive><floppydrive>4</floppydrive><mainboard><cpu>intel core i4</cpu><pan>cooler master4</pan></mainboard>");
  clTest(("SetObject is"), CL_OK == rc, ("rc is %x", rc));
  if (!isRunning)
  {
    rc = client.Commit();
    clTest(("SetObject is"), CL_OK == rc, ("rc is %x", rc));
  }
}
void testGetObject()
{
  std::string strResult;
  ClRcT rc = client.GetObject("myService\\computer", strResult);
  clTest(("GetObject data %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
}
void testRPC()
{
  std::string strResult;
  ClRcT rc = client.DoRpc("sumContainer", "<var0>1</var0><configurationInteger><var1>5</var1><var2>4</var2></configurationInteger>", strResult);
  clTest(("DoRpc data %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
  rc = client.DoRpc("sumContainerReturnContainer", "<var0>1</var0><configurationInteger><var1>2</var1><var2>4</var2></configurationInteger>", strResult);
  clTest(("DoRpc data %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
  rc =
      client.DoRpc("sumMultipleInlineContainer",
          "<var0>1</var0><configurationIntegerLevel1><var1>1</var1><var2>1</var2><configurationIntegerLevel2><var1>1</var1><var2>1</var2><configurationIntegerLevel3><var1>1</var1><var2>1</var2><configurationIntegerLevel4><var1>1</var1><var2>1</var2></configurationIntegerLevel4></configurationIntegerLevel3></configurationIntegerLevel2></configurationIntegerLevel1>",
          strResult);
  clTest(("DoRpc data %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
  rc =
      client.DoRpc("mergeStringContainer",
          "<str0>Merge</str0><configurationString><str1>string</str1><str2>all</str2><childConfigurationString><str3>Container</str3><str4>member</str4><str5>!</str5></childConfigurationString></configurationString>",
          strResult);
  clTest(("DoRpc data %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
}
void testCreateLeafList()
{
  std::string strPath = "myService\\authority\\allowuser";
  ClRcT rc = client.CreateElement<std::string>(strPath, "user8");
  clTest(("CreateElement for list data %s is", strPath.c_str()), CL_OK == rc, ("rc is %x", rc));
  rc = client.CreateElement<std::string>(strPath, "user9");
  clTest(("CreateElement for list data %s is", strPath.c_str()), CL_OK == rc, ("rc is %x", rc));
  if (!isRunning)
  {
    rc = client.Commit();
    clTest(("Commit data %s is", strPath.c_str()), CL_OK == rc, ("rc is %x", rc));
  }
}
void testDeleteLeafList()
{
  std::string strPath = "myService\\authority\\allowuser";
  ClRcT rc = client.DeleteElement<std::string>(strPath, "user8");
  clTest(("DeleteLeafList for list data %s is", strPath.c_str()), CL_OK == rc, ("rc is %x", rc));
  rc = client.DeleteElement<std::string>(strPath, "user9");
  clTest(("DeleteLeafList for list data %s is", strPath.c_str()), CL_OK == rc, ("rc is %x", rc));
  if (!isRunning)
  {
    rc = client.Commit();
    clTest(("commit DeleteLeafList for list data %s is", strPath.c_str()), CL_OK == rc, ("rc is %x", rc));
  }
}
void testGetLeafList()
{
  std::string strResult;
  ClRcT rc = client.GetObject("myService\\authority\\allowuser", strResult);
  clTest(("GetLeafList for list data %s is", strResult.c_str()), CL_OK == rc, ("rc is %x", rc));
}
void testTransCommit()
{
  uint64_t transId;
  ClRcT rc = client.TransStart(transId);
  clTest(("TransStart for transId %ld is", transId), CL_OK == rc, ("rc is %x", rc));
  rc = client.TransCommit(transId);
  clTest(("TransCommit for transId %ld is", transId), CL_OK == rc, ("rc is %x", rc));
}
void testTransRollback()
{
  uint64_t transId;
  ClRcT rc = client.TransStart(transId);
  clTest(("TransStart for transId %ld is", transId), CL_OK == rc, ("rc is %x", rc));
  rc = client.TransRollback(transId);
  clTest(("TransRollback for transId %ld is", transId), CL_OK == rc, ("rc is %x", rc));
}
void testLock_Unlock()
{
  if (!client.GetIsRunning())
  {
    ClRcT rc = client.Lock("candidate");
    clTest(("Lock for candidate is "), CL_OK == rc, ("rc is %x", rc));
    rc = client.UnLock("candidate");
    clTest(("UnLock for candidate is "), CL_OK == rc, ("rc is %x", rc));
  }
}
void testDiscard()
{
  if (!client.GetIsRunning())
  {
    ClRcT rc = client.DiscardChange();
    clTest(("Discard is "), CL_OK == rc, ("rc is %x", rc));
  }
}
void testSetLogLevel()
{
  ClRcT rc = client.SetLogLevel(SAFplus::LogSeverity::LOG_SEV_DEBUG);
  clTest(("SetLogLevel is "), CL_OK == rc, ("rc is %x", rc));
  rc = client.SetLogLevel(SAFplus::LogSeverity::LOG_SEV_ERROR);
  clTest(("SetLogLevel is "), CL_OK == rc, ("rc is %x", rc));
}
void testCopyConfig()
{
  if (!client.GetIsRunning())
  {
    ClRcT rc = client.CopyConfig("running", "candidate");
    clTest(("CopyConfig is "), CL_OK == rc, ("rc is %x", rc));
  }
}
