/******Multi Subscriber******/

#include <iostream>
#include <clDbalBase.hxx>
#include <clTestApi.hxx>
#include <mgtclient.hxx>

using namespace SAFplus;
#define clprintf(sev,...) SAFplus::logMsgWrite(SAFplus::APP_LOG, sev, 0, "APP", "MAIN", __FILE__, __LINE__, __VA_ARGS__)

void callbackfunction(void*data)
{
  std::cout << "callbackfunction exec:\n*******************************************************************\n" << (char*) data << std::endl;
  std::cout << "*******************************************************************\n";
}

void testSetConfig();
void testSubscriber();

bool isRunning = true;
MgtClient client1("root", "root", "127.0.0.1", 830, isRunning);
MgtClient client2("root", "root", "127.0.0.1", 830, isRunning);
MgtClient client3("root", "root", "127.0.0.1", 830, isRunning);
int main(void)
{
  SAFplus::logSeverity = SAFplus::LOG_SEV_DEBUG;
  clTestGroupInitialize(("MGT-MGTCLIENT: Subscription"));
  ClRcT rc = CL_OK;

  // Create 3 client
  rc = client1.Connect();
  rc = client2.Connect();
  rc = client3.Connect();

  clTest(("Opening database is "), rc == CL_OK, (("rc is %x"), rc));
  client1.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client1.GenerateMessageId();

  client2.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client2.GenerateMessageId();

  client3.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client3.GenerateMessageId();
  ////////////////////////////////////////////////////////////

  testSubscriber();
  sleep(2);
  testSetConfig();
  sleep(10);
  ///////////////////////////////////////////////////////////

  client1.UnSubscriber();
  client2.UnSubscriber();
  clTest(("client1 UnSubscriber success!"), client1.UnSubscriber() == CL_OK, ("client1 UnSubscriber fail!"));
  clTest(("client2 UnSubscriber success!"), client2.UnSubscriber() == CL_OK, ("client2 UnSubscriber fail!"));

  client1.Close();
  client2.Close();
  client3.Close();
  clTestGroupFinalize();

  return 0;
}

void testSubscriber()
{
  ClRcT rc = CL_OK;
  std::string strPath1 = "myService\\config\\port";
  std::string strPath2 = "myService";

  //Client1 subscription
  rc = client1.Subscriber(strPath1, boost::posix_time::second_clock::local_time(), &callbackfunction);
  clTest(("client1 Subscriber success!"), rc == CL_OK, ("client1 Subscriber fail!"));

  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: client1 Subscriber success!\n");
  } else
  {
    logInfo("DBAL","TEST","Result: client1 Subscriber fail!\n");
  }

  //Client2 subscription
  rc = client2.Subscriber(strPath1, boost::posix_time::second_clock::local_time(), &callbackfunction);
  clTest(("client2 Subscriber success!"), rc == CL_OK, ("client2 Subscriber fail!"));

  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: client2 Subscriber success!\n");
  } else
  {
    logInfo("DBAL","TEST","Result: client2 Subscriber fail!\n");
  }
}

void testSetConfig()
{
  int port = 8080;
  std::string strPath = "myService\\config\\port";
  ClRcT rc = client3.SetElement<int>(strPath, port);
  clTest(("SetConfig success!"), rc == CL_OK, ("SetConfig fail!"));
  if (CL_OK == rc)
  {
    std::cout << "Setting with new port:" << port << std::endl;
    logInfo("DBAL","TEST","Result: SetConfig is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: SetConfig is fail\n");
  }
}


