/*****Execute edit config fail while another client is locking database******/

#include <iostream>
#include <clDbalBase.hxx>
#include <clTestApi.hxx>
#include <mgtclient.hxx>

using namespace SAFplus;
#define clprintf(sev,...) SAFplus::logMsgWrite(SAFplus::APP_LOG, sev, 0, "APP", "MAIN", __FILE__, __LINE__, __VA_ARGS__)


void callbackfunction(void*data)
{
  std::cout << "callbackfunction exec:\n*******************************************************************\n" << (char*) data << std::endl;
  std::cout << "*******************************************************************\n";
}

void testLockDatabase();
bool isRunning = true;
MgtClient client1("root", "root", "127.0.0.1", 830, isRunning);
MgtClient client2("root", "root", "127.0.0.1", 830, isRunning);
int main(void)
{
  SAFplus::logSeverity = SAFplus::LOG_SEV_DEBUG;
  clTestGroupInitialize(("MGT-MGTCLIENT: Locking database"));
  ClRcT rc = CL_OK;

  // Create 2 client
  rc = client1.Connect();
  rc = client2.Connect();
  clTest(("Opening database is "), rc == CL_OK, (("rc is %x"), rc));
  client1.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client1.GenerateMessageId();
  client2.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client2.GenerateMessageId();
  ////////////////////////////////////////////////////////////

  testLockDatabase();

  sleep(5);
  ///////////////////////////////////////////////////////////
  client1.Close();
  client2.Close();
  clTestGroupFinalize();
  return 0;
}

void testLockDatabase()
{
  ClRcT rc = CL_OK;
  //Client 1 lock database
  rc = client1.Lock("running");
  clTest(("Lock success!"), rc == CL_OK, ("Lock fail!"));
  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: Lock database is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: Lock database is fail!");
  }

  // Client2 create host in subscriber
  std::string strPath = "myService\\subscribers\\subscriber";
  std::string strElement = "<name>host2</name>";
  rc = client2.CreateElement<std::string>(strPath, strElement);
  clTest(("CreateElement fail!"), rc == CL_ERR_UNSPECIFIED, ("CreateElement success!"));
  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: CreateElement is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: CreateElement is failed\n");
  }

  // Client2 delete host in subscriber
  strPath = "myService\\subscribers\\subscriber";
  strElement = "<name>host2</name>";
  rc = client2.DeleteElement<std::string>(strPath, strElement);
  clTest(("DeleteElement fail!"), rc == CL_ERR_UNSPECIFIED, ("DeleteElement success!"));
  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: DeleteElement is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: DeleteElement is failed\n");
  }

  // Client2 set config
  int port = 8080;
  strPath = "myService\\config\\port";
  rc = client2.SetElement<int>(strPath, port);
  clTest(("SetConfig fail!"), rc == CL_ERR_UNSPECIFIED, ("SetConfig success!"));
  if (CL_OK == rc)
  {
    std::cout << "Setting with new port:" << port << std::endl;
    logInfo("DBAL","TEST","Result: SetConfig is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: SetConfig is fail\n");
  }
}
