/*
 * 1. Create element successfully when locking database from the same client
 * 2. Unlock fail when database locking by another client
 */

#include <iostream>
#include <clDbalBase.hxx>
#include <clTestApi.hxx>
#include <mgtclient.hxx>

using namespace SAFplus;
#define clprintf(sev,...) SAFplus::logMsgWrite(SAFplus::APP_LOG, sev, 0, "APP", "MAIN", __FILE__, __LINE__, __VA_ARGS__)

void callbackfunction(void*data)
{
  std::cout << "callbackfunction exec:\n*******************************************************************\n" << (char*) data << std::endl;
  std::cout << "*******************************************************************\n";
}

void testlockDatabase();
bool isRunning = true;
MgtClient client1("root", "root", "127.0.0.1", 830, isRunning);
MgtClient client2("root", "root", "127.0.0.1", 830, isRunning);
int main(void)
{
  SAFplus::logSeverity = SAFplus::LOG_SEV_DEBUG;
  clTestGroupInitialize(("MGT-MGTCLIENT: Lock_Unlock Database"));
  ClRcT rc = CL_OK;

  // Create 2 client
  rc = client1.Connect();
  rc = client2.Connect();
  clTest(("Opening database is "), rc == CL_OK, (("rc is %x"), rc));
  client1.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client1.GenerateMessageId();
  client2.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client2.GenerateMessageId();
  ////////////////////////////////////////////////////////////

  testlockDatabase();

  sleep(5);
  ///////////////////////////////////////////////////////////
  client1.Close();
  client2.Close();
  clTestGroupFinalize();
  return 0;
}

void testlockDatabase()
{
  ClRcT rc = CL_OK;

  // Client1 lock database
  rc = client1.Lock("running");
  clTest(("Lock success!"), rc == CL_OK, ("Lock fail!"));
  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: Lock database is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: Lock database is fail!");
  }

  // Client 1 create element
  std::string strPath = "myService\\subscribers\\subscriber";
  std::string strElement = "<name>host5</name>";
  rc = client1.CreateElement<std::string>(strPath, strElement);
  clTest(("CreateElement success!"), rc == CL_OK, ("CreateElement fail!"));
  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: CreateElement is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: CreateElement is failed\n");
  }

  // Client2 unlock database
  rc = client2.UnLock("running");
  clTest(("UnLock fail!"), rc == CL_ERR_UNSPECIFIED, ("UnLock success!"));
  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: UnLock database is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: UnLock database is fail!");
  }
}
