/*********Create element fail when another client is setting config in transaction (don't commit)*********/

#include <iostream>
#include <clDbalBase.hxx>
#include <clTestApi.hxx>
#include <mgtclient.hxx>

using namespace SAFplus;
#define clprintf(sev,...) SAFplus::logMsgWrite(SAFplus::APP_LOG, sev, 0, "APP", "MAIN", __FILE__, __LINE__, __VA_ARGS__)

void callbackfunction(void*data)
{
  std::cout << "callbackfunction exec:\n*******************************************************************\n" << (char*) data << std::endl;
  std::cout << "*******************************************************************\n";
}

void testSetConfig();

bool isRunning = true;
MgtClient client1("root", "root", "127.0.0.1", 830, isRunning);
MgtClient client2("root", "root", "127.0.0.1", 830, isRunning);
int main(void)
{
  SAFplus::logSeverity = SAFplus::LOG_SEV_DEBUG;
  clTestGroupInitialize(("MGT-MGTCLIENT: Transaction"));
  ClRcT rc = CL_OK;

  // Create 2 client
  rc = client1.Connect();
  rc = client2.Connect();
  clTest(("Opening database is "), rc == CL_OK, (("rc is %x"), rc));
  client1.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client1.GenerateMessageId();
  client2.SetNameSpace("xmlns=\"http://www.myCompany.com/ns/myService\"");
  client2.GenerateMessageId();
  ////////////////////////////////////////////////////////////

  testSetConfig();

  sleep(5);
  ///////////////////////////////////////////////////////////
  client1.Close();
  client2.Close();
  clTestGroupFinalize();
  return 0;
}

void testSetConfig()
{
  int port = 32;
  ClRcT rc = CL_OK;
  uint64_t transId;

  // Start transaction
  rc = client1.TransStart(transId);
  clTest(("Transaction start success!"), rc == CL_OK, ("Transaction start fail!"));
  if (CL_OK == rc)
  {
    std::cout << "Transid from server success:" << transId << std::endl;
    logInfo("DBAL","TEST","Result: Transaction start is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: Transaction start is fail!\n");
    return;
  }

  // Client1 set config in transaction created.
  std::string strPath = "myService\\config\\port";
  rc = client1.SetElement<int>(strPath, port, transId);
  clTest(("SetConfig success!"), rc == CL_OK, ("SetConfig fail!"));
  if (CL_OK == rc)
  {
    std::cout << "Setting with new port:" << port << std::endl;
    logInfo("DBAL","TEST","Result: SetConfig in transaction is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: SetConfig in transaction is fail\n");
  }

  // Client2 create host in subscriber
  strPath = "myService\\subscribers\\subscriber";
  std::string strElement = "<name>host6</name>";
  rc = client2.CreateElement<std::string>(strPath, strElement);
  clTest(("CreateElement fail!"), rc == CL_ERR_UNSPECIFIED, ("CreateElement success!"));
  if (CL_OK == rc)
  {
    logInfo("DBAL","TEST","Result: CreateElement is success!");
  } else
  {
    logInfo("DBAL","TEST","Result: CreateElement is failed\n");
  }
}
