/*
 * Copyright (C) 2002-2015 OpenClovis Solutions Inc.  All Rights Reserved.
 *
 * This file is available  under  a  commercial  license  from  the
 * copyright  holder or the GNU General Public License Version 2.0.
 *
 * The source code for  this program is not published  or otherwise
 * divested of  its trade secrets, irrespective  of  what  has been
 * deposited with the U.S. Copyright office.
 *
 * This program is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * For more  information, see  the file  COPYING provided with this
 * material.
 */
#pragma once

#ifndef _CLMGTCOMMON_HXX_
#define _CLMGTCOMMON_HXX_

#include <string>
#include <map>
#include <sys/time.h>

namespace MgtServer
{
  class MgtCacheOMData
  {
    public:
      std::string XMLdata;
      struct timeval timeCache;

      MgtCacheOMData(const std::string &XMLdata)
              : XMLdata(XMLdata)
      {
        gettimeofday(&timeCache, NULL);
      }
      ;

      MgtCacheOMData(const MgtCacheOMData &rhs)
              : XMLdata(rhs.XMLdata)
      {
        timeCache.tv_sec = rhs.timeCache.tv_sec;
        timeCache.tv_usec = rhs.timeCache.tv_usec;
      }

      MgtCacheOMData & operator=(const MgtCacheOMData &rhs)
      {
        XMLdata = rhs.XMLdata;
        timeCache.tv_sec = rhs.timeCache.tv_sec;
        timeCache.tv_usec = rhs.timeCache.tv_usec;
        return *this;
      }

  };

  // Stored OMs (NETCONF & SNMP) within 30s
  extern std::map<std::string, MgtCacheOMData*> cacheDataOMs;

  void findCacheData(const std::string &route, std::string &xmlOutput);
  void updateCacheData(const std::string &route, const std::string &xml);

  template<class T>
    void destroyMap(std::map<std::string, T *> &map)
    {
      for (typename std::map<std::string, T *>::iterator it = map.begin(); it != map.end(); ++it)
      {
        delete (*it).second;
      }
      map.clear();
    }
}
#endif /* _CLMGTCOMMON_HXX_ */
