/*
 * Copyright (C) 2002-2012 OpenClovis Solutions Inc.  All Rights Reserved.
 *
 * This file is available  under  a  commercial  license  from  the
 * copyright  holder or the GNU General Public License Version 2.0.
 * 
 * The source code for  this program is not published  or otherwise 
 * divested of  its trade secrets, irrespective  of  what  has been 
 * deposited with the U.S. Copyright office.
 * 
 * This program is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied  warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 * 
 * For more  information, see  the file  COPYING provided with this
 * material.
 *
 */

#ifndef MGTNETCONFHANDLER_HXX_
#define MGTNETCONFHANDLER_HXX_

#include "clMgtNetconfOM.hxx"
#include <clCkptApi.hxx> // to use checkpoint

#ifdef __cplusplus
extern "C"
{
#include "ncx.h"
#include "agt.h"
#ifndef _H_xml_util
#include "xml_util.h"
#endif
#include "ncxmod.h"
#include "agt_util.h"
#include "agt_not.h"
#include "tk.h"
#include "agt_val_parse.h"
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <xml_util.h>
#include "agt_ses.h"
#include "agt_xml.h"
#include "ses_msg.h"
#include "xml_wr.h"
}
#endif
namespace MgtServer
{

  class MgtNetconfHandler
  {
  private:
    MgtNetconfHandler();
    ~MgtNetconfHandler()
    {
      MgtRootOM::DestroyInstance();
    };
    static MgtNetconfHandler* instance;

  public:
    static MgtNetconfHandler *getInstance();
    static void DestroyInstance();

    ClRcT netconfInitialize(int argc, ClCharT *argv[]);
    /*
     * Deprecated
     */
    void mgtMsgBindHandle(SAFplus::Handle from, void* msg);

    /*
     * Function for handling notification data change
     */
    void mgtMsgNotifyHandle(SAFplus::Handle from, void* msg);

    /*
     * Callback for getting data
     */
    void mgtNetconfGet(const xmlChar *moduleName, val_value_t *virval, val_value_t *dstval);

    /*
     * Callback for setting data
     */
    status_t mgtNetconfSet(op_editop_t editop, val_value_t *newval);
    status_t mgtNetconfRpc(Mgt::Msg::MsgRpc_MgtRpcType rpcType,ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);
    /*
     * Function to parse xml data to val_value_t
     */
    void mgtNetconfParserXmlData(const xmlChar *moduleName, val_value_t *virval, val_value_t *dstval, xmlChar *outVal);

    /*
     * Function to parse xml data to NETCONF notification
     */
    void mgtNetconfNotifParserXmlData(const xmlChar* moduleName, agt_not_msg_t *notifMsg, obj_template_t *notiobj, const xmlChar *outVal);

    /*
     * helper function to pre-format xml before parsing to netconf
     */
    std::string preformatXML(val_value_t *dstval, const char *outVal);

    /*
     * Get keys list of an obj template
     */
      std::string getKeysObj(const obj_template_t *obj);
      
      /*
       * Utils function to debug val_value_t
       */
      static std::string dumpValStr(val_value_t *val, ncx_instfmt_t fmt = NCX_IFMT_XPATH1);

  };

}
#endif /* MGTNETCONFHANDLER_HXX_ */
