/*
 * Copyright (C) 2002-2015 OpenClovis Solutions Inc.  All Rights Reserved.
 *
 * This file is available  under  a  commercial  license  from  the
 * copyright  holder or the GNU General Public License Version 2.0.
 *
 * The source code for  this program is not published  or otherwise
 * divested of  its trade secrets, irrespective  of  what  has been
 * deposited with the U.S. Copyright office.
 *
 * This program is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * For more  information, see  the file  COPYING provided with this
 * material.
 *
 */

#ifndef MGTYUMA_HXX_
#define MGTYUMA_HXX_

#ifdef __cplusplus
extern "C"
{
#endif
#include "status.h"
#include <libxml/xmlstring.h>
#include <libxml/parser.h>
#ifdef __cplusplus
} /* end extern 'C' */
#endif

//Yuma includes
#include  "procdefs.h"
#include "agt.h"
#include "agt_util.h"
#include "agt_cap.h"
#include "agt_cb.h"
#include "agt_cfg.h"
#include "agt_cli.h"
#include "agt_ncx.h"
#include "agt_rpc.h"
#include "agt_rpcerr.h"
#include "agt_ses.h"
#include "agt_sys.h"
#include "agt_state.h"
#include "agt_time_filter.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cap.h"
#include "cfg.h"
#include "ncxmod.h"
#include "obj.h"
#include "op.h"
#include "rpc.h"
#include "rpc_err.h"
#include "ses.h"
#include "status.h"
#include "tstamp.h"
#include "val.h"
#include "xml_wr.h"
#include "yangconst.h"
#include "xpath.h"
#include "yang_parse.h"

namespace MgtServer
{
  class MgtYuma
  {
  public:
    /**
     * Parser for RPC
     */
    static xmlNodePtr yumaRpcParseAttributeNode(const xmlNodePtr node, const xmlChar *keyword);
    static std::string yumaRpcParseAttribute(ses_cb_t *scb, xml_node_t *methnode);

    /**
     * Netconf callback for setting data
     */
    static status_t yumaDataSetCallback(ses_cb_t *scb, rpc_msg_t *msg, agt_cbtyp_t cbtyp, op_editop_t editop, val_value_t *newval, val_value_t *curval);
    static status_t yumaDataGetCallback(ses_cb_t *scb, getcb_mode_t cbmode, val_value_t *virval, val_value_t *dstval);

    //? Netconf callback for validate rpc
    static status_t yumaRpcValidate(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);
    //? Netconf callback for invoke rpc
    static status_t yumaRpcInvoke(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);
    //? Netconf callback for postreply rpc
    static status_t yumaRpcPostReply(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);

    //? Netconf callback for validate rpc
    static status_t yumaLeafValidate(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);
    //? Netconf callback for invoke rpc
    static status_t yumaLeafInvoke(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);
    //? Netconf callback for postreply rpc
    static status_t yumaLeafPostReply(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);

    //? Netconf callback for validate rpc
    static status_t yumaListValidate(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);
    //? Netconf callback for invoke rpc
    static status_t yumaListInvoke(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);
    //? Netconf callback for postreply rpc
    static status_t yumaListPostReply(ses_cb_t *scb, rpc_msg_t *msg, xml_node_t *methnode);


    /**
     * RPC module register/unregister
     */
    static void register_rpc(ncx_module_t *mod, dlq_hdr_t *objnameQ);
    static void register_callbacks(ncx_module_t *mod);
    static void unregister_rpc(ncx_module_t *mod);
    /**
     * Load/unload module
     */
    static status_t yumaLoadModule(const xmlChar* moduleName, bool loadSIL);
    static status_t yumaUnloadModule(const xmlChar* moduleName);

    /**
     * Parse XPATH
     */
    static std::string yumaXPathParse(xmlChar *valXPath);
    /**
     * Register callback for setting
     */
    static status_t yumaRegisterDataSetCallback(const xmlChar* moduleName, const xmlChar* route);

    /**
     * 
     */
    static ClRcT yumaCheckAndLoadModule(std::vector<std::string> &nodes, const std::vector<std::string> &paths, std::map<std::string,bool> *mapLoadFailed);

    /*
     * Nodes is already register to yuma callback
     */
    static std::vector<std::string> nodeRegistered;
  };

}

#endif /* CLMGTYUMA_HXX_ */
