/*
 * Copyright (C) 2002-2012 OpenClovis Solutions Inc.  All Rights Reserved.
 *
 * This file is available  under  a  commercial  license  from  the
 * copyright  holder or the GNU General Public License Version 2.0.
 * 
 * The source code for  this program is not published  or otherwise 
 * divested of  its trade secrets, irrespective  of  what  has been 
 * deposited with the U.S. Copyright office.
 * 
 * This program is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied  warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 * 
 * For more  information, see  the file  COPYING provided with this
 * material.
 *
 */

#ifndef CLMGTMEDDEFS_HXX_
#define CLMGTMEDDEFS_HXX_

#ifdef __cplusplus
extern "C"
{
/*
 * net-snmp
 */
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
/*
 * libsmi
 */
#include <smi.h>
/*
 * libxml2
 */
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xmlreader.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>
}
#endif

#define CL_SNMP_MAX_STR_LEN 255
#define CL_SNMP_MAX_OID_LEN         128
#define CL_SNMP_MAX_ATTR_FOR_INDEX  8
#define CL_SNMP_MAX_MOID_STR_LEN  1024
#define CL_SNMP_DISPLAY_STRING_SIZE 255


/**
 * Error macro definitions for SNMP subagent.
 */
#define CL_SNMP_RC(ERROR_CODE)  CL_RC(CL_CID_SNMP, (ERROR_CODE))

/*
 * Error codes (the value of the field error-status in PDUs)
 */
/*
 * in SNMPv1, SNMPsec, SNMPv2p, SNMPv2c, SNMPv2u, SNMPv2*, and SNMPv3 PDUs
 */
#define CL_SNMP_ERR_NOERROR                (0)     /* XXX  Used only for PDUs? */
#define CL_SNMP_ERR_TOOBIG                  (1)
#define CL_SNMP_ERR_NOSUCHNAME             (2)
#define CL_SNMP_ERR_BADVALUE               (3)
#define CL_SNMP_ERR_READONLY               (4)
#define CL_SNMP_ERR_GENERR                  (5)

/*
 * in SNMPv2p, SNMPv2c, SNMPv2u, SNMPv2*, and SNMPv3 PDUs
 */
#define CL_SNMP_ERR_NOACCESS        (6)
#define CL_SNMP_ERR_WRONGTYPE       (7)
#define CL_SNMP_ERR_WRONGLENGTH     (8)
#define CL_SNMP_ERR_WRONGENCODING       (9)
#define CL_SNMP_ERR_WRONGVALUE      (10)
#define CL_SNMP_ERR_NOCREATION      (11)
#define CL_SNMP_ERR_INCONSISTENTVALUE   (12)
#define CL_SNMP_ERR_RESOURCEUNAVAILABLE (13)
#define CL_SNMP_ERR_COMMITFAILED        (14)
#define CL_SNMP_ERR_UNDOFAILED      (15)
#define CL_SNMP_ERR_AUTHORIZATIONERROR  (16)
#define CL_SNMP_ERR_NOTWRITABLE     (17)

/*
 * in SNMPv2c, SNMPv2u, SNMPv2*, and SNMPv3 PDUs
 */
#define CL_SNMP_ERR_INCONSISTENTNAME    (18)

/**
 * The structure ClMedAgentId contains the client representation of the attribute(s).
 *
 */
struct ClMedAgentId
{

/**
 * Identifier in agent language.
 * SNMP OID kind of method.
 */
    ClUint8T      *id;

/**
 *  Length of the identifier.
 */
    ClUint32T      len;
};


typedef struct ClMedAgentId ClMedAgentIdT;

/**
 * The type of an identifier for a COR attribute.
 */
typedef ClInt32T   ClCorAttrIdT;

/**
 * Structure representing the parameters.
 *
 */
typedef struct
{

/**
 * Attribute ID in client language
 */
    ClMedAgentIdT    attrId;

/**
 *  Actual Value.
 */
    void           *pVal;

/**
 * Length of the value.
 */
    ClUint32T     len;

/**
 * Result buffer.
 */
    void           *outBuf;

/**
 * Length of the output buffer.
 */
    ClUint32T     outBufLen;

/**
 *   Data related to instance.
 */
void           *pInst;

/**
 *  Instance Data Length.
 */
    ClUint32T     instLen;

/**
 *  Error Id return value.
 */
    ClUint32T     errId;

/**
 * Private information to be stored in the agent.
 */
    void           *cookie;

/**
 * Index for array/containment/associated attributes. Set -1 if not an array.
 */
    ClUint32T index;

/**
 * Attribute Id of containment or association attribute.
 */
    ClCorAttrIdT    ContAttrId;

}ClMedVarBindT;


/**
 * Structure to represent Operation information.
 */
typedef struct
{

/**
 *  Operation Code: Language - Client.
 */
    ClUint32T       opCode;

/**
 *  Number of variables for this operation.
 */
    ClUint32T       varCount;

/**
 *  Pointer to an array of operands.
 */
    ClMedVarBindT     *varInfo;

}ClMedOpT;

/**
 * The structure ClMedErrorIdT represents the error returned from COR
 */
typedef struct
{
/**
 * Error id
 */
  ClRcT errId;
/**
 *  Oid info
 */
   ClMedAgentIdT oidInfo;

}ClMedErrorIdT;

/**
 * The structure ClMedErrorListT represents list of error returned from COR
 */
typedef struct
{
    ClUint32T   count;
/**
 *  List of errId with oid
 */
    ClMedErrorIdT  *pErrorList;

}ClMedErrorListT;

#endif /* CLMGT_HXX_ */
