/*
 * Copyright (C) 2002-2015 OpenClovis Solutions Inc.  All Rights Reserved.
 *
 * This file is available  under  a  commercial  license  from  the
 * copyright  holder or the GNU General Public License Version 2.0.
 * 
 * The source code for  this program is not published  or otherwise 
 * divested of  its trade secrets, irrespective  of  what  has been 
 * deposited with the U.S. Copyright office.
 * 
 * This program is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied  warranty  of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 * 
 * For more  information, see  the file  COPYING provided with this
 * material.
 *
 */

#ifndef MGT_SNMP_OM_HXX_
#define MGT_SNMP_OM_HXX_

#include <string>
#include <map>
#include <list>

#include <clMgtApi.hxx>
#include <clMgtDatabase.hxx>
#include <clMgtSnmpDefs.hxx>

using namespace std;

#ifdef __cplusplus
extern "C"
{
#endif
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/types.h>
#ifdef __cplusplus
}
#endif

namespace MgtServer
{
  class MgtSnmpBaseInfo
  {
  public:
    std::string module;
    std::string name;
    std::string xpath;
    ClUint32T oid[CL_SNMP_MAX_OID_LEN];
    ClUint32T len;

  public:
    MgtSnmpBaseInfo();
    MgtSnmpBaseInfo(const std::string &module, const std::string &name, const std::string &xpathStr);
    virtual ~MgtSnmpBaseInfo();
  };

  /*
   *
   */
  class MgtSnmpScalarInfo : public MgtSnmpBaseInfo
  {
  public:
    ClUint8T type;
  public:
    MgtSnmpScalarInfo();
    MgtSnmpScalarInfo(const std::string &module, const std::string &name, const std::string &xpathStr);
    virtual ~MgtSnmpScalarInfo();
  };

  class MgtSnmpTableInfo : public MgtSnmpBaseInfo
  {
  public:
    std::vector<MgtSnmpScalarInfo*> columns;
    std::map<std::string, MgtSnmpScalarInfo*> mColumns;
    std::vector<std::string> indexes;

  public:
    MgtSnmpTableInfo();
    MgtSnmpTableInfo(const std::string &module, const std::string &name, const std::string &xpathStr);
    virtual ~MgtSnmpTableInfo();
  };

  class MgtSnmpNotificationInfo : public MgtSnmpBaseInfo
  {
  public:
    std::vector<MgtSnmpScalarInfo*> columns;
    std::map<std::string, MgtSnmpScalarInfo*> mColumns;

  public:
    MgtSnmpNotificationInfo();
    MgtSnmpNotificationInfo(const std::string &module, const std::string &name, const std::string &xpathStr);
    virtual ~MgtSnmpNotificationInfo();
  };

  /*
   * This class is used to store the data binding of a component.
   * Example:
   *      componentA -> bind -> new clMgtOM() -> insert into hash map
   *
   */
  class MgtSnmpOM
  {
  public:
      Mgt::Msg::MsgBind bindData;
      ClUint32T rev;
      ClUint32T oid[CL_SNMP_MAX_OID_LEN];
      ClUint32T len;

      MgtSnmpOM(Mgt::Msg::MsgBind *bindDataComp);

      /**
       * Write cache data to database
       */
      ClRcT write(std::string &xml, SAFplus::MgtDatabase *db = NULL);
      /**
       * Read cache data from database
       */
      ClRcT read(std::string &value, SAFplus::MgtDatabase *db = NULL);

      /*
       * To string
       */
      const void toKey(std::string &key);
      MgtSnmpOM();
      ~MgtSnmpOM();
  };

  class MgtMibModuleOM
  {
  public:
      MgtMibModuleOM();
      ~MgtMibModuleOM();

      map<string, MgtSnmpOM *> mOMList;
      /*
       * HashMap stored notification
       */
      map<string, MgtSnmpNotificationInfo *> mNotifOMList;

      /*
       * HashMap stored table/scalar
       */
      map<string, MgtSnmpBaseInfo *> mSnmpInfos;
  };

}
#endif /* CL_MGT_SNMP_OM_HXX_ */
