/*
 * Copyright (C) 2002-2015 OpenClovis Solutions Inc.  All Rights Reserved.
 *
 * This file is available  under  a  commercial  license  from  the
 * copyright  holder or the GNU General Public License Version 2.0.
 *
 * The source code for  this program is not published  or otherwise
 * divested of  its trade secrets, irrespective  of  what  has been
 * deposited with the U.S. Copyright office.
 *
 * This program is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * For more  information, see  the file  COPYING provided with this
 * material.
 */
#include <clCommon.hxx>
#include <clMgtCommon.hxx>
#include <map>
#include <string>

using namespace std;

namespace MgtServer
{
  std::map<std::string, MgtCacheOMData*> cacheDataOMs;

  ClUint64T getTimeStamp(timeval t2, timeval t1)
  {
    return ((t2).tv_sec * 1000000 + (t2).tv_usec) - ((t1).tv_sec * 1000000 + (t1).tv_usec);
  }

  void findCacheData(const string &route, std::string &xmlOutput)
  {
    ClUint64T diff;
    struct timeval timeNow;
    map<string, MgtCacheOMData*>::iterator cacheIter = cacheDataOMs.find(route);
    if (cacheIter != cacheDataOMs.end())
    {
      MgtCacheOMData *cacheOM = (*cacheIter).second;

      // Check expire
      gettimeofday(&timeNow, NULL);
      diff = getTimeStamp(timeNow, cacheOM->timeCache);

      /*
       * Cache OM data within 30s
       */
      if (diff < (5 * 1000000))
      {
        xmlOutput.assign(cacheOM->XMLdata);
        return;
      }
      else
      {
        delete (MgtCacheOMData*)((*cacheIter).second);
        cacheDataOMs.erase(cacheIter);
      }
    }
    xmlOutput.assign("");
  }

  void updateCacheData(const std::string &route, const std::string &xml)
  {
    map<string, MgtCacheOMData*>::iterator cacheIter = cacheDataOMs.find(route);
    if (cacheIter != cacheDataOMs.end())
    {
      delete (MgtCacheOMData*)((*cacheIter).second);
      cacheDataOMs.erase(route);
    }
    cacheDataOMs.insert(pair<string, MgtCacheOMData*> (route, new MgtCacheOMData(xml)));
  }

}
