/******************************************************************************
 * clCompAppMain.C
 *
 * This file is autogenerated by OpenClovis IDE, Copyright (C) 2012 by 
 * OpenClovis. All rights reserved.
 */

#include <cltypes.h> /* This header fix the PRIx compilation errors */
// POSIX includes.
#include <assert.h>
#include <errno.h>
#include <stdlib.h>
#include <signal.h>
#include <ncxtypes.h>

#ifdef __cplusplus
extern "C" {
#endif
#include <agt.h>
#include <agt_ncx.h>
#include <agt_ncxserver.h>
#ifdef __cplusplus
} /* end extern 'C' */
#endif

#include <algorithm>
#include <boost/algorithm/string.hpp>
#include <boost/filesystem.hpp>
#include <boost/program_options.hpp>

// SAFplus includes.
#include <clMsgApi.hxx>
#include <clMgtApi.hxx>
#include <clMsgPortsAndTypes.hxx>
#include <clLogApi.hxx>
#include <clGroupIpi.hxx>
#include <clGlobals.hxx>
#include <clMgtDatabase.hxx>
#include <clSafplusMsgServer.hxx>

#include "clMgtCustomization.hxx"
#include "clMgtMsgHandler.hxx"
#include "clMgtNetconfYuma.hxx"
#include "clMgtNetconfHandler.hxx"
#include "clMgtSnmpHandler.hxx"

SAFplus::MgtDatabase mgtSrvDb;
#define MGT_SERVER_DB_NAME "MgtServerDatabase"
#define MGT_SERVER_KEY_SIZE 999
#define MGT_SERVER_REC_SIZE  999
#define AGENT_CONFIG "subagent"

#define NETCONF "NETCONF"
#define SNMP "SNMP"
#define MGT "MGT"

using namespace std;
using namespace MgtServer;
using namespace SAFplus;
using namespace SAFplusI;
namespace po = boost::program_options;

extern void* NetconfServerThunk(void* params);
void stopNetconfServer();
void stopSnmpServer();
int  errorExit(SaAisErrorT rc);

SAFplusI::GroupServer gs;  // WARN: will not be initialized if safplus_group or safplus_amf is running

#define clprintf(sev,...) SAFplus::logMsgWrite(SAFplus::APP_LOG, sev, 0, "MGT", "MAIN", __FILE__, __LINE__, __VA_ARGS__)

// This is the application name to use if the AMF did not start up this component
#define DEFAULT_APP_NAME "mgt"

/* This process's SAF name */
SaNameT      appName = {0};
/* Access to the SAF AMF framework occurs through this handle */
SaAmfHandleT amfHandle;

/* set when SAFplus tells this process to quit */
bool quitting = false;
/* set when SAFplus assigns active/standby work */
bool running = true;
bool keep_snmpAgent_running;

int nodeRepresentative = true;

extern SAFplus::Checkpoint *SAFplus::mgtCheckpoint;

// TODO: replace with checkpoint notification and just get updated entry
boost::thread threadMgtCkptPolling;

// Latest CKPT handle checking
vector<string> ckptHdlLast;

bool isCkptUpdate(vector<string> &ckptNew, vector<string> &ckptDeleted)
{
  bool isUpdate = false;
  vector<string> ckptHdCurrent;

  // Get xpath from Mgt Checkpoint and get new CKPT entry
  for (Checkpoint::Iterator it = mgtCheckpoint->begin(); it != mgtCheckpoint->end(); ++it)
  {
    string xpath = (*it).first->data;
    string xpathNode = xpath.substr(xpath.find('/') + 1);
    if (!xpathNode.empty())
      {
        ckptHdCurrent.push_back(xpathNode);
        if (std::find(ckptHdlLast.begin(), ckptHdlLast.end(), xpathNode) == ckptHdlLast.end())
        {
          ckptNew.push_back(xpathNode);
          // Update new ckpt into last CKPT checking
          ckptHdlLast.push_back(xpathNode);
          isUpdate = true;
        }
      }
  }

  /*
   * Compare ckptHdCurrent with ckptHdlLast to get deleted CKPT entry
   */
  for (vector<string>::iterator it = ckptHdlLast.begin(); it != ckptHdlLast.end(); ++it)
  {
    if (std::find(ckptHdCurrent.begin(), ckptHdCurrent.end(), *it) == ckptHdCurrent.end())
    {
      ckptDeleted.push_back(*it);
      isUpdate = true;
    }
  }

  return isUpdate;
}

void mgtCkptPolling(void)
{
  int delay = 2;
  bool isUpd = false;
  // Iterate thru SAFplus specific directories, loading all the YANG modules
  // Export environment SAFPLUS_MODULES_PATH=path1:path2:path3... for yang modules auto loading
  char *env = getenv("SAFPLUS_MODULES_PATH");
  std::vector<std::string> pathsSearch;
  std::vector<std::string>::iterator iterCkpt;
  std::map<string,bool> mapLoadFile;
  if (env != NULL)
  {
    boost::split(pathsSearch, env, boost::is_any_of(":"), boost::token_compress_on);
  }
  else
    {
      logCritical("NETCONF","SRV","Environment variable SAFPLUS_MODULES_PATH must be assigned to indicate the location of SAFplus-enabled NETCONF modules");
      assert(!"Environment variable SAFPLUS_MODULES_PATH must be assigned to indicate the location of SAFplus-enabled NETCONF modules");
    }
  bool loaded = false;
  while(1)
  {
    vector<string> ckptNew, ckptDeleted;
    isUpd = isCkptUpdate(ckptNew, ckptDeleted);
    if (isUpd && !ckptNew.empty())
    {
      // Filter unique ckpt items
      iterCkpt = std::unique(ckptNew.begin(), ckptNew.end());
      ckptNew.resize(std::distance(ckptNew.begin(), iterCkpt));

      /*
       * Load YANG module
       */
      MgtYuma::yumaCheckAndLoadModule(ckptNew, pathsSearch, &mapLoadFile);
      /*
       * Load MIB module
       */
      //MgtSnmpHandler::CheckAndLoadMibModule(ckptNew, pathsSearch, &mapLoadFailed);
      for(std::map<string, bool>::iterator iterCkptFile = mapLoadFile.begin(); iterCkptFile != mapLoadFile.end(); ++iterCkptFile)
      {
        iterCkpt = std::find(ckptHdlLast.begin(), ckptHdlLast.end(), (*iterCkptFile).first);
        if (iterCkpt != ckptHdlLast.end() && (*iterCkptFile).second)
        {
          /* Should recheck in the next polling */
          ckptHdlLast.erase(iterCkpt);
        }
      }
    }

    if (isUpd && !ckptDeleted.empty())
    {
      // Filter unique ckpt items
      iterCkpt = std::unique(ckptDeleted.begin(), ckptDeleted.end());
      ckptDeleted.resize(std::distance(ckptDeleted.begin(), iterCkpt));

      /*
       * TODO: Unregister handle and remove it out of NETCONF/SNMP
       */
    }

    boost::this_thread::sleep(boost::posix_time::seconds(delay));
  }
}

void stopdaemon(int a)
{
  /* Shutdown netconf */
  agt_request_shutdown(NCX_SHUT_EXIT);
  /* Cleanup the Netconf Server Library */
  agt_cleanup();

  logDebug("NETCONF","SRV","NetConf daemon stopped");
  /* Main thread should stop */
  running = false;

  // Remove socket
  int r = system("rm -f /tmp/ncxserver.sock");
  (void)r;

}

void stopNetconfServer()
{
  stopdaemon(1);
}

/**
 * Signal handler. Stop snmp when received any signal
 */
RETSIGTYPE stopsubagent(ClInt32T a)
{
  keep_snmpAgent_running = false;
}

void stopSnmpServer()
{
  stopsubagent(1);
}

/*
 * Function to hook snmp_log and convert to SAFplus log API level
 */
int snmpFallbackLog(int majorID, int minorID, void *msg, void *cookies)
{
    snmp_log_message *logMsg = (snmp_log_message *)msg;
    if (logMsg)
    {
        /* Convert to SAFplus priorities */
        int pri = logMsg->priority + 1;
        appLog(SAFplus::APP_LOG, (SAFplus::LogSeverity)pri, 0, MGT, SNMP, logMsg->msg);
    }
    return 0;
}

void* NetconfServerThunk(void* ths)
{
    uint_t i = 0;
    uint_t retries = 3;

    signal(SIGTERM, stopdaemon);
    signal(SIGINT, stopdaemon);
    while (i++ <= retries)
    {
        /* try to connect to the NCX server */
        status_t res = agt_ncxserver_run();
        if (res != NO_ERR)
        {
            if (i > retries)
            {
              logDebug("NETCONF","SRV","Exceed retry times. Exit");
              break;
            }
            boost::this_thread::sleep(boost::posix_time::milliseconds(50));
        }
    }
    return NULL;
}

/* 
 * This is sanity checking agent configuration file (<AGENT_CONFIG>.conf) existing or not:
 *   If found, using this path for reading.
 *   Otherwise, using default setting.
 *
 * Go through subagent configuration path to lookup <AGENT_CONFIG>.conf.
 */
void initAgentxConfig()
{
    std::vector<std::string> pathsSearch;
    const char* agentxDefaultSocket = "tcp:localhost:3456";

    /* Subagent configuration path: $SNMPCONFPATH:<$CURDIR>/share/snmp:$ASP_CONFIG */
    char *snmpConfig = NULL;
    string curDir = boost::filesystem::current_path().string();
    string snmpConfDir(curDir);

    if (SAFplus::ASP_CONFIG[0]!=0)
    {
      snmpConfDir.append(":").append(SAFplus::ASP_CONFIG);
    }

    // ENV SNMPCONFPATH if exporting
    if ((snmpConfig = SAFplus::clParseEnvStr("SNMPCONFPATH")) != NULL)
    {
      snmpConfDir.append(":").append(snmpConfig);
    }

    if (1)
    {
      snmpConfDir.append(":").append(curDir).append("/share/snmp");
      snmpConfDir.append(":").append(curDir).append("/../share/snmp");
      snmpConfDir.append(":").append(curDir).append("/etc");
      snmpConfDir.append(":").append(curDir).append("/../etc");
    }

    if (SAFplus::ASP_BINDIR[0]!=0)
    {
      snmpConfDir.append(":").append(SAFplus::ASP_BINDIR).append("/../share/snmp");
    }

    /* Go through snmpConfDir to find configuration file */
    boost::split(pathsSearch, snmpConfDir, boost::is_any_of(":"), boost::token_compress_on);
    for (std::vector<std::string>::const_iterator iter = pathsSearch.begin(); iter != pathsSearch.end(); ++iter)
    {
      boost::filesystem::path p(*iter + "/" + AGENT_CONFIG + ".conf");
      if (boost::filesystem::exists(p))
      {
        logDebug("SNM", "SRV", "Subagent configuration path: %s", (*iter).c_str());
        netsnmp_ds_set_string(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_CONFIGURATION_DIR, (*iter).c_str());
        return;
      }
    }
    logDebug("SNM", "SRV", "Agentx deault socket: %s", agentxDefaultSocket);
    netsnmp_ds_set_string(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_X_SOCKET, agentxDefaultSocket);
}

static void *SnmpAgentThunk(void* params)
{
  /* Disable all log handlers */
  snmp_disable_log();
  /* Enable log handler callback */
  snmp_enable_calllog();

  /* Sanity check subagent configuration file or using default setting */
  initAgentxConfig();

  netsnmp_ds_set_boolean(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_ROLE, 1);
  netsnmp_ds_set_boolean(NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_AGENT_NO_ROOT_ACCESS, 1);
  netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_DISABLE_PERSISTENT_LOAD, 1);

  /* MIB search path */
  netsnmp_ds_set_string(NETSNMP_DS_LIBRARY_ID, NETSNMP_DS_LIB_MIBDIRS, smiGetPath());

  /*
   * Register SNMP log handler callback
   */
  snmp_register_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_LOGGING, snmpFallbackLog, NULL);

  /* Initialize the agent library */
  init_agent(AGENT_CONFIG);

  /* Initialize snmp subagent */
  init_snmp(AGENT_CONFIG);

  /* In case we recevie a request to stop (kill -TERM or kill -INT) */
  signal(SIGTERM, stopsubagent);
  signal(SIGINT, stopsubagent);

  logInfo("SNM", "SRV", "SNMP subagent started");

  /* SNMP subagent main loop, unblocking mode */
  while (keep_snmpAgent_running)
    {
      agent_check_and_process(0); /* 0 == don't block */

      /* Don't know why packet asserted failure - workaround */
      usleep(500);
    }

  snmp_unregister_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_LOGGING, snmpFallbackLog, NULL, 0);
  /* Signal received, stop snmp */
  snmp_shutdown(AGENT_CONFIG);
  shutdown_agent();
  SOCK_CLEANUP;
  logDebug("SNM", "SRV", "SNMP subagent stopped");
  running = false;
  return NULL;
}

/**
 * Initialize SAFplus libraries
 */
void initSafLibraries()
{
  ClRcT rc = CL_OK;
  //SafplusInitializationConfiguration sic;

  // Partial initialize
  safplusInitialize(LibSet::LOG | LibSet::UTILS | LibSet::OSAL | LibSet::HEAP | LibSet::BUFFER | LibSet::TIMER | LibSet::IOC | LibSet::GRP | LibSet::FAULT);

  //SafplusMsgServer amf(SAFplusI::AMF_IOC_PORT,50,1);
  try
    {
    msgServerInitialize(SAFplusI::AMF_IOC_PORT, 50,1);
    nodeRepresentative = true;
    }
  catch (Error &e)
    {
      if (e.osError == EADDRINUSE)
        {
        msgServerInitialize(SAFplusI::MGT_IOC_PORT, 25,1);
        nodeRepresentative = false;
        }
      else throw;
    }

#if 0
  if (nodeRepresentative)  // If I'm the node rep, I need to use the AMF port because other entities expect to communicate with servers there.  TODO: fix by putting the node rep's port in shared mem for each service.
    {
    sic.iocPort     = SAFplusI::AMF_IOC_PORT;
    sic.msgQueueLen = 50;
    sic.msgThreads  = 1;
    }
  else
    {
    sic.iocPort     = SAFplusI::MGT_IOC_PORT;
    sic.msgQueueLen = 25;
    sic.msgThreads  = 1;
    }
#endif

  safplusInitialize(SAFplus::LibDep::CKPT | LibDep::NAME ); // , sic);
  logEchoToFd = 1;  // echo logs to stdout for debugging
  logCompName = "MGT";

  if (nodeRepresentative)
    {
      try
        {
          gs.init();
        }
      catch(SAFplus::Error& e)
        {
          if (e.clError == SAFplus::Error::EXISTS)
            {
              logAlert("NCF","INI", "Group server is not being run from within the AMF because it already exists.");
              nodeRepresentative = false;
            }
          else
            {
              throw;
            }
        }

    }

  if (SAFplus::mgtAccessInitialize) SAFplus::mgtAccessInitialize();
  mgtSrvDb.initialize(MGT_SERVER_DB_NAME, "", MGT_SERVER_KEY_SIZE, MGT_SERVER_REC_SIZE);
  DbalPlugin* plugin = mgtSrvDb.getPlugin();
  logInfo("MGTSRV","DB","Opening database file [%s] using plugin [%s]", MGT_SERVER_DB_NAME, plugin->type);
}

/**
 * Initialize message handler for communication with mgt objects
 */

void initMsgServer(MgtServer::MgtMsgHandler *msghandle)
{
  SAFplus::SafplusMsgServer* mgtIocInstance = &safplusMsgServer;
  mgtIocInstance->registerHandler(SAFplusI::CL_MGT_MSG_TYPE,msghandle,NULL);
}

/**
 * Start netconfd and snmpsubagent
 */
ClRcT startServer(int argc,char* argv[])
{
  ClRcT rc = CL_OK;
  logDebug("NETCONF","SRV","NETCONF server daemon debug 6");
#ifdef SAFPLUS_MGT_NETCONF

  // Remove socket
  logDebug("NETCONF","SRV","NETCONF server daemon debug 7");
  int r = system("rm -f /tmp/ncxserver.sock");
  (void)r;
  logDebug("NETCONF","SRV","NETCONF server daemon debug 8");
  /* Init netconf server */
  rc = MgtServer::MgtNetconfHandler::getInstance()->netconfInitialize(argc,argv);
  if(rc != CL_OK)
  {
    logError("MGTSRV","START","Initialize netconf failed [%x]",rc);
    return rc;
  }
  /* Start netconf server */
  pthread_t thid;
  pthread_create(&thid, NULL, NetconfServerThunk, NULL);
  pthread_detach(thid);
#endif
  logDebug("NETCONF","SRV","NETCONF server daemon debug 9");
#ifdef SAFPLUS_MGT_SNMP
  /* Init snmp subagent */
  rc = MgtServer::MgtSnmpHandler::getInstance()->snmpInitialize(argc, argv);
  if (rc != CL_OK)
  {
    throw MgtError("Initialization failed");
  }
  /* Start snmp subagent */
  logDebug("NETCONF","SRV","NETCONF server daemon debug 10");
  pthread_create(&thid, NULL, SnmpAgentThunk, NULL);
  pthread_detach(thid);
#endif
  logDebug("NETCONF","SRV","NETCONF server daemon debug 11");
  return rc;
}



/* This simple helper function just prints an error and quits */
int errorExit(SaAisErrorT rc)
{
    logCritical("APP","INI", "Component [%.*s] : PID [%d]. Initialization error [0x%x]\n", appName.length, appName.value, SAFplus::pid, rc);
    exit(-1);
    return -1;
}

/**
 * Stop netconfd and snmpagent
 */
void stopServer()
{
#ifdef SAFPLUS_MGT_NETCONF
  stopNetconfServer();
#endif
#ifdef SAFPLUS_MGT_SNMP
  stopSnmpServer();
#endif

  SAFplus::SafplusMsgServer* mgtIocInstance = &safplusMsgServer;
  mgtIocInstance->Stop();
  mgtIocInstance->RemoveHandler(SAFplusI::CL_MGT_MSG_TYPE);
}

/**
 * Notify client to bind data to the server
 */
void sendBindRequest()
{
  Mgt::Msg::MsgMgt mgtMsgReq;
  string strBind("");
  mgtMsgReq.set_type(Mgt::Msg::MsgMgt::CL_MGT_MSG_BIND_REQUEST);
  mgtMsgReq.set_bind(strBind);

  SAFplus::SafplusMsgServer* mgtIocInstance = &safplusMsgServer;

  try
  {
    string output;
    mgtMsgReq.SerializeToString(&output);
    int size = output.size();

    /*TODO: Allow user configure the list of port that safplus_mgt communicate with */
    for (int i = SAFplusI::START_IOC_PORT; i <= SAFplusI::END_IOC_PORT; i++)
    {
      mgtIocInstance->SendMsg(SAFplus::getProcessHandle(i,Handle::AllNodes), (void *)output.c_str(), size, SAFplusI::CL_MGT_MSG_TYPE);
    }
  }
  catch (Error &e)
  {
    assert(0);
  }

}



void safTerminate(SaInvocationT invocation, const SaNameT *compName)
  {
  SaAisErrorT rc = SA_AIS_OK;

  clprintf(SAFplus::LOG_SEV_INFO, "Component [%.*s] : PID [%d]. Terminating\n", compName->length, compName->value, SAFplus::pid);

  /*
   * Unregister with AMF and respond to AMF saying whether the
   * termination was successful or not.
   */
  if ( (rc = saAmfComponentUnregister(amfHandle, compName, NULL)) != SA_AIS_OK)
    {
    clprintf (SAFplus::LOG_SEV_ERROR, "Component [%.*s] : PID [%d]. Unregister failed with error [0x%x]\n", compName->length, compName->value, SAFplus::pid, rc);
    return;
    }

  /* Ok tell SAFplus that we handled it properly */
  saAmfResponse(amfHandle, invocation, SA_AIS_OK);

  clprintf (SAFplus::LOG_SEV_INFO, "Component [%.*s] : PID [%d]. Terminated\n", compName->length, compName->value, SAFplus::pid);

  quitting = true;
  }

/*
 * safAssignWork
 * ------------------
 * This function is invoked when a CSI assignment is made or the state
 * of a CSI is changed.
 */

void safAssignWork(SaInvocationT       invocation,
                   const SaNameT       *compName,
                   SaAmfHAStateT       haState,
                   SaAmfCSIDescriptorT csiDescriptor)
{
    /* Print information about the CSI Set */

    clprintf (SAFplus::LOG_SEV_INFO, "Component [%.*s] : PID [%d]. CSI Set Received\n", 
              compName->length, compName->value, SAFplus::pid);

    //printCSI(csiDescriptor, haState);

    /*
     * Take appropriate action based on state
     */

    switch ( haState )
    {
        /* AMF asks this process to take the standby HA state for the work
           described in the csiDescriptor variable */
        case SA_AMF_HA_ACTIVE:
        {
            /* Typically you would spawn a thread here to initiate active 
               processing of the work. */
            pthread_t thr;
            clprintf(SAFplus::LOG_SEV_INFO,"Management active state requested, activating service");
            running = 1;
            //pthread_create(&thr,NULL,activeLoop,NULL);


            /* The AMF times the interval between the assignment and acceptance
               of the work (the time interval is configurable).
               So it is important to call this saAmfResponse function ASAP.
             */
            saAmfResponse(amfHandle, invocation, SA_AIS_OK);
            break;
        }

        /* AMF asks this process to take the standby HA state for the work
           described in the csiDescriptor variable */
        case SA_AMF_HA_STANDBY:
        {
            /* If your standby has ongoing maintenance, you would spawn a thread
               here to do it. */

            clprintf(SAFplus::LOG_SEV_INFO,"Management standby state requested");
            running = 2;

            /* The AMF times the interval between the assignment and acceptance
               of the work (the time interval is configurable).
               So it is important to call this saAmfResponse function ASAP.
             */
            saAmfResponse(amfHandle, invocation, SA_AIS_OK);  
            break;
        }

        case SA_AMF_HA_QUIESCED:
        {
            /*
             * AMF has requested application to quiesce the CSI currently
             * assigned the active or quiescing HA state. The application 
             * must stop work associated with the CSI immediately.
             */
            clprintf(SAFplus::LOG_SEV_INFO, "Management quiesced state");
            running = 0;

            saAmfResponse(amfHandle, invocation, SA_AIS_OK);
            break;
        }

        case SA_AMF_HA_QUIESCING:
        {
            /*
             * AMF has requested application to quiesce the CSI currently
             * assigned the active HA state. The application must stop work
             * associated with the CSI gracefully and not accept any new
             * workloads while the work is being terminated.
             */

            /* There are two typical cases for quiescing.  Chooose one!
               CASE 1: Its possible to quiesce rapidly within this thread context */
            if (1)
              {
              /* App code here: Now finish your work and cleanly stop the work*/
              //clprintf(SAFplus::LOG_SEV_INFO, "csa101: Signaling completion of QUIESCING");
              running = 0;
            
              /* Call saAmfCSIQuiescingComplete when stopping the work is done */
              saAmfCSIQuiescingComplete(amfHandle, invocation, SA_AIS_OK);
              }
            else
              {
              /* CASE 2: You can't quiesce within this thread context or quiesce
               rapidly. */

              /* Respond immediately to the quiescing request */
              saAmfResponse(amfHandle, invocation, SA_AIS_OK);

              /* App code here: Signal or spawn a thread to cleanly stop the work*/
              /* When that thread is done, it should call:
                 saAmfCSIQuiescingComplete(amfHandle, invocation, SA_AIS_OK);
              */
              }

            break;
        }

        default:
        {
            assert(0);
            break;
        }
    }

    return;
}

/*
 * safRemoveWork
 * ---------------------
 * This function is invoked when a CSI assignment is to be removed.
 */

void safRemoveWork(SaInvocationT  invocation,
                   const SaNameT  *compName,
                   const SaNameT  *csiName,
                   SaAmfCSIFlagsT csiFlags)
{
    clprintf (SAFplus::LOG_SEV_INFO, "Management component [%.*s] : PID [%d]. Work removal received\n", compName->length, compName->value, SAFplus::pid);

    clprintf (SAFplus::LOG_SEV_INFO, "   CSI                     : %.*s\n", csiName->length, csiName->value);
    clprintf (SAFplus::LOG_SEV_INFO, "   CSI Flags               : 0x%d\n", csiFlags);

    /*
     * Add application specific logic for removing the work for this CSI.
     */

    saAmfResponse(amfHandle, invocation, SA_AIS_OK);
}


void initializeAmf(void)
  {
  SaAmfCallbacksT     callbacks;
  SaAisErrorT         rc = SA_AIS_OK;

  callbacks.saAmfHealthcheckCallback          = NULL; /* rarely necessary because SAFplus monitors the process */
  callbacks.saAmfComponentTerminateCallback   = safTerminate;
  callbacks.saAmfCSISetCallback               = safAssignWork;
  callbacks.saAmfCSIRemoveCallback            = safRemoveWork;
  callbacks.saAmfProtectionGroupTrackCallback = NULL;

  /* Initialize AMF client library. */
  if ( (rc = saAmfInitialize(&amfHandle, &callbacks, &SAFplus::safVersion)) != SA_AIS_OK)
    errorExit(rc);

#if 1
  if ( (rc = saAmfComponentNameGet(amfHandle, &appName)) != SA_AIS_OK)
    {
    if (rc == SA_AIS_ERR_UNAVAILABLE)  // This component was not run by the AMF.  You must tell us what the component name is.
      {
      logInfo("APP","MAIN","Application NOT started by AMF");
      SAFplus::saNameSet(&appName, DEFAULT_APP_NAME);
      }
    }

  if ( (rc = saAmfComponentRegister(amfHandle, &appName, NULL)) != SA_AIS_OK) 
    errorExit(rc);
#else

  // If spawned by the AMF, you can just pass NULL as the component name since the AMF has told this app its name (SAFplus::COMP_NAME variable)
  if ( (rc = saAmfComponentRegister(amfHandle, NULL, NULL)) != SA_AIS_OK)
    errorExit(rc);
#endif

  logInfo("APP","MAIN","Component [%s] registered successfully", appName.value);
  }


/*
 * SAFplus Node attribute
 */
int main(int argc, char *argv[])
{
  ClRcT rc = CL_OK;

  logAlert("NCF","INI","Starting OpenClovis NETCONF/SNMP Agent version %d.%d.%d %s %s", SAFplus::VersionMajor, SAFplus::VersionMinor, SAFplus::VersionBugfix, __DATE__, __TIME__);

#if 0
  po::options_description desc("Allowed options");
  desc.add_options()
    ("help", "produce help message")
    ("withAmf", "run with the SAFplus AMF.  If the AMF spawns this application this parameter unnecessary");

  po::variables_map vm;        
  po::store(po::parse_command_line(argc, argv, desc), vm);
  po::notify(vm);    
  if (vm.count("help")) 
    {
      cout << desc << "\n";
      return 0;
    }
  if (vm.count("standalone")) nodeRepresentative=true;
#endif

  // If this env variable is set, SAFPLUS AMF started this process, so it is the node representative.  Otherwise this mgt server is running in standalone mode (without SAFplus).
  // TODO: nodeRepresentative is used to choose to start the GroupServer.  Instead, use the group's shared memory to negotiate for nodeRepresentative (like checkpoint does)
  char* port = getenv("SAFPLUS_RECOMMENDED_MSG_PORT");
  if (port)
    {
      nodeRepresentative = false;
    }

  SAFplus::logSeverity = SAFplus::LOG_SEV_DEBUG;
  logDebug("NETCONF","SRV","NETCONF server daemon debug 0");
  initSafLibraries();
  logDebug("NETCONF","SRV","NETCONF server daemon debug 1");
  if (port) initializeAmf();
  logDebug("NETCONF","SRV","NETCONF server daemon debug 3");
  MgtServer::MgtMsgHandler msghandle(1);
  logDebug("NETCONF","SRV","NETCONF server daemon debug 4");
  initMsgServer(&msghandle);
  logDebug("NETCONF","SRV","NETCONF server daemon debug 5");
  rc = startServer(argc,argv);
  if(rc != CL_OK)
  {
    logError("MGTSRV","MAIN","Servers failed to start");
    stopServer();
    return 1;
  }
  logDebug("NETCONF","SRV","NETCONF server daemon debug 5");
  // Polling mgtCkpt every 2s
  threadMgtCkptPolling = boost::thread(mgtCkptPolling);

  // Andrew: I think this is not necessary because bindings are now stored in checkpoint 
  //sendBindRequest();

  logInfo("NETCONF","SRV","NETCONF server daemon is running");
  while(running)
  {
    boost::this_thread::sleep(boost::posix_time::milliseconds(50));
  }
  logDebug("NETCONF","SRV","NETCONF server daemon stopped");
  stopServer();
  return 0;
}


