/*
 * Copyright (C) 2002-2014 OpenClovis Solutions Inc.  All Rights Reserved.
 *
 * This file is available  under  a  commercial  license  from  the
 * copyright  holder or the GNU General Public License Version 2.0.
 *
 * The source code for  this program is not published  or otherwise
 * divested of  its trade secrets, irrespective  of  what  has been
 * deposited with the U.S. Copyright office.
 *
 * This program is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * For more  information, see  the file  COPYING provided with this
 * material.
 */

#include <cctype>
#include <iostream>
#include <google/protobuf/descriptor.h>
#include <google/protobuf/message.h>
#include <clMsgServer.hxx>
#include <clMgtApi.hxx>
#include <clLogApi.hxx>

#include "clMgtMsgHandler.hxx"
#include "clMgtNetconfHandler.hxx"
#include "clMgtSnmpHandler.hxx"

using namespace std;
using namespace MgtServer;
namespace MgtServer
{

  MgtMsgHandler::MgtMsgHandler()
  {

  }

  MgtMsgHandler::~MgtMsgHandler()
  {

  }

  bool isSnmpOid(const std::string& s)
  {
    // Should I make sure it is ONLY digits or .?

    if (std::isdigit(s[0])) return true;
    return false;
  }

  void MgtMsgHandler::msgHandler(SAFplus::Handle from, SAFplus::MsgServer* svr, ClPtrT msg, ClWordT msglen, ClPtrT cookie)
  {
    Mgt::Msg::MsgMgt mgtMsgReq;
    mgtMsgReq.ParseFromArray(msg, msglen);

    if (mgtMsgReq.type() == Mgt::Msg::MsgMgt::CL_MGT_MSG_BIND_REQUEST)
      return;

    Mgt::Msg::MsgBind bindData;
    bindData.ParseFromString(mgtMsgReq.bind());
    logDebug("MGT", "MSG", "Handle [%" PRIx64 ":%" PRIx64 "], Module [%s], Route [%s], ", bindData.handle().id0(), bindData.handle().id1(), bindData.module().c_str(),bindData.route().c_str());
    const char *route = bindData.route().c_str();
    switch (mgtMsgReq.type())
      {
    case Mgt::Msg::MsgMgt::CL_MGT_MSG_BIND:
      {
        if (route != NULL)
        {
          if (!isSnmpOid(route))
          {
            MgtNetconfHandler::getInstance()->mgtMsgBindHandle(from, (void *) &bindData);
          }
          else // bind snmp
          {
            MgtSnmpHandler::getInstance()->mgtMsgBindHandle(from, (void *) &bindData);
          }
        }
        break;
      }
    case Mgt::Msg::MsgMgt::CL_MGT_MSG_NOTIF:
      {
        if (!isSnmpOid(route))
        {
          MgtNetconfHandler::getInstance()->mgtMsgNotifyHandle(from, &mgtMsgReq);
        }
        else
        {
          MgtSnmpHandler::getInstance()->mgtMsgNotifHandle(from, &mgtMsgReq);
        }
        break;
      }
      case Mgt::Msg::MsgMgt::CL_MGT_MSG_BIND_RPC:
      {
        if (!isSnmpOid(route))
        {
          //TODO: handle RPC for NETCONF
        }
        break;
      }
    }
  }
} /* namespace MgtServer */
