/*
 * Copyright (C) 2002-2015 OpenClovis Solutions Inc.  All Rights Reserved.
 *
 * This file is available  under  a  commercial  license  from  the
 * copyright  holder or the GNU General Public License Version 2.0.
 *
 * The source code for  this program is not published  or otherwise
 * divested of  its trade secrets, irrespective  of  what  has been
 * deposited with the U.S. Copyright office.
 *
 * This program is distributed in the  hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * For more  information, see  the file  COPYING provided with this
 * material.
 */

#include "clMgtSnmpOM.hxx"
#include <clMgtCommon.hxx>

#ifdef __cplusplus
extern "C"
{
#endif
/*
 * ASP Client Includes.
 */
#include <string.h>
#ifdef __cplusplus
} /* end extern 'C' */
#endif

using namespace SAFplus;

namespace MgtServer
{

  MgtSnmpBaseInfo::MgtSnmpBaseInfo()
  {
    len = 0;
  }

  MgtSnmpBaseInfo::MgtSnmpBaseInfo(const std::string &module, const std::string &name, const std::string &xpathStr)
  {
    // MIB module
    this->module.assign(module);
    // Name scalar/table
    this->name.assign(name);
    this->xpath.assign(xpathStr);
    len = 0;
  }

  MgtSnmpBaseInfo::~MgtSnmpBaseInfo()
  {
  }

  MgtSnmpScalarInfo::MgtSnmpScalarInfo()
  {
    type = 0;
  }

  MgtSnmpScalarInfo::MgtSnmpScalarInfo(const std::string &module, const std::string &name, const std::string &xpathStr): MgtSnmpBaseInfo(module, name, xpathStr)
  {
    type = 0;
  }

  MgtSnmpScalarInfo::~MgtSnmpScalarInfo()
  {
  }

  MgtSnmpTableInfo::MgtSnmpTableInfo()
  {
  }

  MgtSnmpTableInfo::MgtSnmpTableInfo(const std::string &module, const std::string &name, const std::string &xpathStr): MgtSnmpBaseInfo(module, name, xpathStr)
  {
  }

  MgtSnmpTableInfo::~MgtSnmpTableInfo()
  {
    destroyMap(mColumns);
  }

  // Notification class
  MgtSnmpNotificationInfo::MgtSnmpNotificationInfo()
  {
  }

  MgtSnmpNotificationInfo::MgtSnmpNotificationInfo(const std::string &module, const std::string &name, const std::string &xpathStr): MgtSnmpBaseInfo(module, name, xpathStr)
  {
  }

  MgtSnmpNotificationInfo::~MgtSnmpNotificationInfo()
  {
    destroyMap(mColumns);
  }

  MgtMibModuleOM::MgtMibModuleOM()
  {

  }

  MgtMibModuleOM::~MgtMibModuleOM()
  {
    // Clean map Objects Management
    destroyMap(mOMList);
    // Clean map Notification Objects Management
    destroyMap(mNotifOMList);
    // Clean map General Objects Management
    destroyMap(mSnmpInfos);
  }

  MgtSnmpOM::MgtSnmpOM(Mgt::Msg::MsgBind *bindDataComp)
  {
    bindData = *bindDataComp;
    rev = 0;
    len = 0;
    memset(oid, 0, CL_SNMP_MAX_OID_LEN);
  }

  /**
   * Write cache data to database
   */
  ClRcT MgtSnmpOM::write(std::string &xml, SAFplus::MgtDatabase *db)
  {
    ClRcT rc = CL_OK;
    std::string value;
    std::string key;
    if (db == NULL)
        return CL_FALSE;

    /* Update or Insert record */
    toKey(key);
    rc = db->getRecord(key, value);
    if (rc != CL_OK)
    {
      return db->insertRecord(key, xml);
    }
    else
    {
      return db->setRecord(key, xml);
    }
  }

  /**
   * Read cache data from database
   */
  ClRcT MgtSnmpOM::read(std::string &value, SAFplus::MgtDatabase *db)
  {
    ClRcT rc = CL_OK;
    std::string key;
    if (db == NULL)
      return CL_FALSE;
    toKey(key);
    rc = db->getRecord(key, value);
    return rc;
  }

  /*
   * To string
   */
  const void MgtSnmpOM::toKey(std::string &key)
  {
    std::stringstream ss;
    ss << bindData.module().c_str();
    ss << ":";
    ss << bindData.route().c_str();
    ss << ":";
    ss << bindData.handle().id0();
    ss << ":";
    ss << bindData.handle().id1();
    key.assign(ss.str());
  }

  MgtSnmpOM::MgtSnmpOM() : bindData(), rev(0), len(0)
  {
    memset(oid, 0, CL_SNMP_MAX_OID_LEN);
  }

  MgtSnmpOM::~MgtSnmpOM()
  {

  }
}

/* namespace MgtServer */
