#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string>
#include <clGlobals.hxx>
#include <clLogApi.hxx>
#include <clIocPortList.hxx>
#include "clMgtRoot.hxx"
#include "Interfaces.hxx"
#include "Get_interface.hxx"
#include "Ethernet.hxx"
#include "clSafplusMsgServer.hxx"

using namespace std;
using namespace SAFplus;
using namespace network;


ClBoolT   gIsNodeRepresentative = CL_FALSE;
ClBoolT   stop = CL_FALSE;
static unsigned int MAX_MSGS=25;
static unsigned int MAX_HANDLER_THREADS=10;
unsigned int port = 50;
void sig_handler(int signum);
/* MGT */
/* My application */
MgtModule         dataModule("network");
MgtNotify         addrFailoverNotify("addressFailover");
Interfaces        netif_eth0("eth0");
Interfaces        netif_eth1("eth1");
Get_interface     getIfRpc;
Ethernet           ethernet;
void beActive(void)
{
  SAFplus::Handle hdl = SAFplus::Handle::create(port);
  /* Bind the "active" network interface to a "/ethernet/interfaces" subtree within the "network" module.*/
  ethernet.bind(hdl,"network","/ethernet");

  netif_eth0.bind(hdl,"network", "1.3.6.1.4.1.99840.1.1.1");
  netif_eth1.bind(hdl,"network", "1.3.6.1.4.1.99840.1.1.2");


  addrFailoverNotify.setLeaf("ipAddress", netif_eth0.ipAddress);
  addrFailoverNotify.sendNotification(hdl,"/AddressFailover");
  addrFailoverNotify.sendNotification(hdl,"1.3.6.1.4.1.99840.2");

}
void clMgtTest()
{
    /* Load "network" yang module into the system */
    dataModule.loadModule();

    /* Initialize non-config data of the "network" module */
    dataModule.initialize();

    addrFailoverNotify.Module = "network";
    /* Add notification leaf */
    addrFailoverNotify.addLeaf("ipAddress", netif_eth0.ipAddress);

    /* Eth0 sample data */
    netif_eth0.setAdminStatus(true);
    netif_eth0.setNetMask("255.255.255.0");
    netif_eth0.setIpAddress("10.20.18.150");
    /* Eth1 sample data */
    netif_eth1.setBytesSent(9999);
    netif_eth1.setAdminStatus(false);
    /* Add "AddressFailover" notification to the module */
    dataModule.addMgtNotify(&addrFailoverNotify);

    std::string eth0_name("eth0");
    std::string eth1_name("eth1");
    ethernet.interfacesList.addChildObject(&netif_eth0,eth0_name);
    ethernet.interfacesList.addChildObject(&netif_eth1,eth1_name);

}
int main()
{
  SAFplus::ASP_NODEADDR = 1;
  /* Give a chance to stop program */
  signal(SIGINT, sig_handler);
  /* Log */
  logInitialize();
  logEchoToFd = 1;
  logSeverity = LOG_SEV_MAX;
  /* Util */
  utilsInitialize();
  /* Others */
  ClRcT rc;
  if ((rc = clOsalInitialize(NULL)) != CL_OK || (rc = clHeapInit()) != CL_OK || (rc = clTimerInitialize(NULL)) != CL_OK || (rc = clBufferInitialize(NULL)) != CL_OK)
  {
    assert(0);
  }
  /* IOC */
  rc = clIocLibInitialize(NULL);
  assert(rc==CL_OK);
  /* Global server */
  safplusMsgServer.init(port, MAX_MSGS, MAX_HANDLER_THREADS);
  safplusMsgServer.Start();
  /* MGT Test */
  clMgtTest();
  beActive();
  while(!stop)
  {
    boost::this_thread::sleep(boost::posix_time::milliseconds(50));
  }
  safplusMsgServer.Stop();
  cout << "Program stopped \n" ;
  return 0;

}

void sig_handler(int signum)
{
  cout << "Received signal " <<  signum << "\n";
  stop = CL_TRUE;
}
